@@ -31,6 +31,7 @@
 #include "doxywizard.h"
 #include "expert.h"
 #include "config.h"
+#include "version.h"
 
 const int messageTimeout = 5000; //!< status bar message timeout in millisec.
 
@@ -46,31 +47,14 @@ QCString getResourcePath()
   return result;
 }
 
-#define GRAPHVIZ_PATH    "/Applications/Graphviz.app"
-#define DOT_PATH         GRAPHVIZ_PATH "/Contents/MacOS"
-#define DOT_LOCATION     DOT_PATH "/dot"
-
-bool checkIfDotInstalled()
-{
-  QFileInfo fi(GRAPHVIZ_PATH);
-  if (fi.exists() && fi.isDir())
-  {
-    fi.setFile(DOT_LOCATION);
-    if (fi.exists() && fi.isFile())
-    {
-      return TRUE;
-    }
-  }
-  return FALSE;
-}
-
 void setDotPath()
 {
-  if (checkIfDotInstalled())
-  {
-    Config_getString("DOT_PATH")=DOT_PATH;
-    Config_getBool("HAVE_DOT")=TRUE;
-  }
+  Config_getString("DOT_PATH")=getResourcePath();
+}
+
+void setMscgenPath()
+{
+  Config_getString("MSCGEN_PATH")=getResourcePath();
 }
 
 #endif
@@ -227,11 +211,12 @@ Step2::Step2(QWidget *parent) : QWidget(parent,"Step2")
   layout->addWidget(f);
 
   m_optimizeLang = new QButtonGroup("Select programming language to optimize the results for",this);
-  gbox = new QGridLayout( m_optimizeLang, 4, 1, 8, 0 );
+  gbox = new QGridLayout( m_optimizeLang, 5, 1, 8, 0 );
   gbox->addRowSpacing( 0, fontMetrics().lineSpacing()+2 );
   gbox->addWidget(new QRadioButton("Optimize for C++ output",m_optimizeLang),1,0);
   gbox->addWidget(new QRadioButton("Optimize for Java output",m_optimizeLang),2,0);
   gbox->addWidget(new QRadioButton("Optimize for C output",m_optimizeLang),3,0);
+  gbox->addWidget(new QRadioButton("Optimize for C# output",m_optimizeLang),4,0);
   m_optimizeLang->setButton(0);
   layout->addWidget(m_optimizeLang);
 
@@ -271,6 +256,7 @@ OptLang Step2::optimizeFor() const
     case 0:  return Lang_Cpp;
     case 1:  return Lang_Java;
     case 2:  return Lang_C;
+    case 3:  return Lang_CS;
   }
   return Lang_Cpp;
 }
@@ -294,6 +280,7 @@ void Step2::setOptimizeFor(OptLang lang)
     case Lang_Cpp:  m_optimizeLang->setButton(0); break;
     case Lang_Java: m_optimizeLang->setButton(1); break;
     case Lang_C:    m_optimizeLang->setButton(2); break;
+    case Lang_CS:   m_optimizeLang->setButton(3); break;
   }
 }
 
@@ -511,7 +498,12 @@ Step4::Step4(QWidget *parent) : QWidget(parent,"Step4")
     dotGroup->setButton(0);
     m_dotOptions->setEnabled(FALSE);
   gbox->addWidget(w,4,0);
+
+#if defined(Q_OS_MACX) // we bundle dot with the mac package
+  m_diagramMode->setButton(2);
+#else
   m_diagramMode->setButton(1);
+#endif
   layout->addWidget(m_diagramMode);
   layout->addStretch(1);
 
@@ -677,8 +669,10 @@ MainWidget::MainWidget(QWidget *parent)
   // initialize config settings
   Config::instance()->init();
   Config::instance()->check();
+  Config_getBool("HAVE_DOT")=TRUE;
 #if defined(Q_OS_MACX)
   setDotPath();
+  setMscgenPath();
 #endif
   
   QWidget *w = new QWidget(this);
@@ -823,12 +817,12 @@ MainWidget::MainWidget(QWidget *parent)
   // load default settings
   m_settings.setPath("www.doxygen.org","Doxygen GUI");
   bool ok;
-  QString config = m_settings.readEntry("/config/default",QString::null,&ok);
+  QString config = m_settings.readEntry("/doxywizard/config/default",QString::null,&ok);
   if (ok && !config.isEmpty())
   {
     Config::instance()->parseString("default settings",config);
   }
-  QString workingDir = m_settings.readEntry("/config/workingdir",QString::null,&ok);
+  QString workingDir = m_settings.readEntry("/doxywizard/config/workingdir",QString::null,&ok);
   if (ok && !workingDir.isEmpty())
   {
     m_workingDir->setText(workingDir);
@@ -836,7 +830,7 @@ MainWidget::MainWidget(QWidget *parent)
   setConfigSaved(FALSE);
   for (int i=0;i<10;i++)
   {
-    QString entry = m_settings.readEntry(QString().sprintf("/recent/config%d",i));
+    QString entry = m_settings.readEntry(QString().sprintf("/doxywizard/recent/config%d",i));
     if (!entry.isEmpty())
     {
       addRecentFile(entry);
@@ -886,8 +880,8 @@ void MainWidget::launchWizard()
   // --------  Initialize the dialog ----------------
   
   // step1
-  wizard.setProjectName(Config_getString("PROJECT_NAME"));
-  wizard.setProjectNumber(Config_getString("PROJECT_NUMBER"));
+  wizard.setProjectName(QString::fromLocal8Bit(Config_getString("PROJECT_NAME")));
+  wizard.setProjectNumber(QString::fromLocal8Bit(Config_getString("PROJECT_NUMBER")));
   if (Config_getList("INPUT").count()>0)
   {
     QString dirName=Config_getList("INPUT").getFirst();
@@ -898,7 +892,7 @@ void MainWidget::launchWizard()
     }
   }
   wizard.setRecursiveScan(Config_getBool("RECURSIVE"));
-  wizard.setDestinationDir(Config_getString("OUTPUT_DIRECTORY"));
+  wizard.setDestinationDir(QString::fromLocal8Bit(Config_getString("OUTPUT_DIRECTORY")));
 
   // step2
   wizard.setExtractAll(Config_getBool("EXTRACT_ALL"));
@@ -978,12 +972,12 @@ void MainWidget::launchWizard()
     // -------- Store the results ----------------
 
     // step1
-    Config_getString("PROJECT_NAME")=wizard.getProjectName();
+    Config_getString("PROJECT_NAME")=wizard.getProjectName().local8Bit();
     Config_getString("PROJECT_NUMBER")=wizard.getProjectNumber();
     Config_getList("INPUT").clear();
     Config_getList("INPUT").append(wizard.getSourceDir());
     Config_getBool("RECURSIVE")=wizard.scanRecursively();
-    Config_getString("OUTPUT_DIRECTORY")=wizard.getDestinationDir();
+    Config_getString("OUTPUT_DIRECTORY")=wizard.getDestinationDir().local8Bit();
 
     // step2
     if (wizard.extractAll())
@@ -1034,6 +1028,12 @@ void MainWidget::launchWizard()
         Config_getBool("OPTIMIZE_OUTPUT_FOR_C")=TRUE;
         Config_getBool("OPTIMIZE_OUTPUT_JAVA")=FALSE;
         break;
+      case Lang_CS:
+        Config_getBool("OPTIMIZE_OUTPUT_FOR_C")=FALSE;
+        Config_getBool("OPTIMIZE_OUTPUT_JAVA")=TRUE;
+        Config_getBool("EXTRACT_STATIC")=TRUE;
+        Config_getBool("EXTRACT_LOCAL_CLASSES")=TRUE;
+        break;
     }
 
     // step3
@@ -1103,10 +1103,14 @@ void MainWidget::launchWizard()
         break;
     }
 #if defined(Q_OS_MACX)
-    if (Config_getBool("HAVE_DOT"))
+    if (Config_getString("DOT_PATH").isEmpty())
     {
       setDotPath();
     }
+    if (Config_getString("MSCGEN_PATH").isEmpty())
+    {
+      setMscgenPath();
+    }
 #endif
     setConfigSaved(FALSE);
   }
@@ -1124,7 +1128,7 @@ void MainWidget::loadConfigFromFile(const QString &fn)
   else
   {
     Config::instance()->convertStrToVal();
-#if defined(Q_OS_MACX)
+#if 0 //defined(Q_OS_MACX)
     if (checkIfDotInstalled() && 
         qstricmp(Config_getString("DOT_PATH"),DOT_PATH)!=0
        )
@@ -1142,6 +1146,18 @@ void MainWidget::loadConfigFromFile(const QString &fn)
     addRecentFile(fn);
     m_workingDir->setText(QFileInfo(fn).dirPath(TRUE));
     m_configFileName = fn;
+#if defined(Q_OS_MACX)
+    if (Config_getString("DOT_PATH").isEmpty())
+    {
+      setDotPath();
+      setConfigSaved(FALSE);
+    }
+    if (Config_getString("MSCGEN_PATH").isEmpty())
+    {
+      setMscgenPath();
+      setConfigSaved(FALSE);
+    }
+#endif
     statusBar()->message("New configuration loaded",messageTimeout);
   }
 }
@@ -1162,9 +1178,9 @@ void MainWidget::launchExpert()
   Expert expert(this);
   expert.init();
   expert.exec();
-#if defined(Q_OS_MACX)
-  setDotPath();
-#endif
+//#if defined(Q_OS_MACX)
+//  setDotPath();
+//#endif
   if (expert.hasChanged()) setConfigSaved(FALSE);
 }
 
@@ -1178,8 +1194,8 @@ void MainWidget::saveDefaults()
     QString newConfig;
     QTextStream t(&newConfig,IO_WriteOnly);
     Config::instance()->writeTemplate(t,TRUE,FALSE);
-    m_settings.writeEntry("/config/default",newConfig);
-    m_settings.writeEntry("/config/workingdir",m_workingDir->text());
+    m_settings.writeEntry("/doxywizard/config/default",newConfig);
+    m_settings.writeEntry("/doxywizard/config/workingdir",m_workingDir->text());
     statusBar()->message("Current configuration saved as default",messageTimeout);
   }
 }
@@ -1190,6 +1206,16 @@ void MainWidget::saveConfigFile()
   if (!fn.isEmpty())
   {
     QFile f(fn);
+    if (f.exists() && QMessageBox::question(this,
+          "Overwrite file? -- Doxywizard",
+          "A file called "+fn+" already exists. Do you want to overwrite it?",
+          "&Yes","&no",
+          QString::null, 0, 1)
+       )
+    {
+      statusBar()->message("Save request aborted",messageTimeout);
+      return;
+    }
     if (f.open(IO_WriteOnly))
     {
       QTextStream t(&f);
@@ -1218,6 +1244,7 @@ void MainWidget::resetConfig()
     Config::instance()->init();
 #if defined(Q_OS_MACX)
     setDotPath();
+    setMscgenPath();
 #endif
 
     m_configFileName = "";
@@ -1310,9 +1337,9 @@ void MainWidget::about()
 {
   QString msg;
   QTextStream t(&msg,IO_WriteOnly);
-  t << "<qt><center>A tool to configure and run doxygen "
-       "on your source files.</center><p><br>"
-       "<center>Written by<br> Dimitri van Heesch<br>&copy; 2000-2004</center><p>"
+  t << QString("<qt><center>A tool to configure and run doxygen version ")+versionString+
+       " on your source files.</center><p><br>"
+       "<center>Written by<br> Dimitri van Heesch<br>&copy; 2000-2006</center><p>"
        "</qt>";
   QMessageBox::about(this,"Doxygen GUI",msg);
 }
@@ -1361,7 +1388,7 @@ void MainWidget::addRecentFile(const QString &file)
   for ( it = m_recentFiles.begin(); it != m_recentFiles.end(); ++it, ++i )
   {
     m_recentMenu->insertItem(*it);
-    m_settings.writeEntry(QString().sprintf("/recent/config%d",i),*it);
+    m_settings.writeEntry(QString().sprintf("/doxywizard/recent/config%d",i),*it);
   }
 }
 
