@@ -2,7 +2,7 @@
  *
  * $Id: classlist.cpp,v 1.14 2001/03/19 19:27:39 root Exp $
  *
- * Copyright (C) 1997-2004 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -34,15 +34,18 @@ static int compItems(void *item1,void *item2)
 {
   ClassDef *c1=(ClassDef *)item1;
   ClassDef *c2=(ClassDef *)item2;
-  int p1=0,p2=0;
   static bool b = Config_getBool("SORT_BY_SCOPE_NAME");
-  if (!b) 
+  //printf("compItems: %d %s<->%s\n",b,c1->qualifiedName().data(),c2->qualifiedName().data());
+  if (b) 
   { 
-    p1=getPrefixIndex(c1->className()); 
-    p2=getPrefixIndex(c2->className()); 
+     return stricmp(c1->name(),
+                    c2->name());
+  }
+  else
+  {
+     return stricmp(c1->className(),
+                    c2->className());
   }
-  return stricmp(c1->className().data()+p1,
-                 c2->className().data()+p2);
 }
 
 int ClassList::compareItems(GCI item1, GCI item2)
@@ -60,7 +63,8 @@ ClassListIterator::ClassListIterator(const ClassList &cllist) :
 {
 }
 
-void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *filter,const char *header)
+void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *filter,
+                                  const char *header,bool localNames)
 {
   if (count()>0)
   {
@@ -95,7 +99,8 @@ void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *f
             ol.startMemberList();
             found=TRUE;
           }
-          if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
+          if (!Config_getString("GENERATE_TAGFILE").isEmpty() &&
+              !cd->isReference())  // skip classes found in tag files
           {
             Doxygen::tagFile << "    <class kind=\"" << cd->compoundTypeString() 
                     << "\">" << convertToXML(cd->name()) << "</class>" << endl;
@@ -103,13 +108,13 @@ void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *f
           ol.startMemberItem(FALSE);
           QCString tmp = cd->compoundTypeString();
           QCString cname;
-          if (Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
+          if (localNames)
           {
-            cname = substitute(cd->className(),"::",".");
+            cname = cd->localName();
           }
           else
           {
-            cname = cd->className();
+            cname = cd->displayName();
           }
           ol.writeString(tmp);
           ol.writeString(" ");
@@ -141,12 +146,12 @@ void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *f
             {
               ol.pushGeneratorState();
               ol.disableAllBut(OutputGenerator::Html);
-              ol.endEmphasis();
+              //ol.endEmphasis();
               ol.docify(" ");
               ol.startTextLink(cd->getOutputFileBase(),"_details");
               ol.parseText(theTranslator->trMore());
               ol.endTextLink();
-              ol.startEmphasis();
+              //ol.startEmphasis();
               ol.popGeneratorState();
             }
             ol.endMemberDescription();
