@@ -3,7 +3,7 @@
  * $Id: diagram.cpp,v 1.30 2001/03/19 19:27:40 root Exp $
  *
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -33,6 +33,7 @@
 //#include "latexgen.h"
 //#include "htmlgen.h"
 #include "doxygen.h"
+#include "portable.h"
 
 //-----------------------------------------------------------------------------
 
@@ -151,7 +152,8 @@ static void writeVectorBox(QTextStream &t,DiagramItem *di,
   if (di->virtualness()==Virtual) t << "solid\n";
 }
 
-static void writeMapArea(QTextStream &t,ClassDef *cd,int x,int y,int w,int h)
+static void writeMapArea(QTextStream &t,ClassDef *cd,QCString relPath,
+                         int x,int y,int w,int h)
 {
   if (cd->isLinkable())
   {
@@ -169,6 +171,10 @@ static void writeMapArea(QTextStream &t,ClassDef *cd,int x,int y,int w,int h)
     {
       if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
     }
+    else
+    {
+      t << relPath;
+    }
     t << cd->getOutputFileBase() << Doxygen::htmlFileExtension << "\" ";
     t << "alt=\"" << cd->displayName(); 
     t << "\" shape=\"rect\" coords=\"" << x << "," << y << ",";
@@ -253,14 +259,17 @@ void DiagramRow::insertClass(DiagramItem *parent,ClassDef *cd,bool doBases,
   di->move(count()*gridWidth,level*gridHeight);
   append(di);
   BaseClassList *bcl=doBases ? cd->baseClasses() : cd->subClasses();
-  /* there are base/sub classes */
   int count=0;
-  BaseClassDef *bcd=bcl->first();
-  while (bcd)
+  if (bcl)
   {
-    ClassDef *ccd=bcd->classDef;
-    if (ccd && ccd->isVisibleInHierarchy() /*&& !ccd->visited*/) count++;
-    bcd=bcl->next();
+    /* there are base/sub classes */
+    BaseClassDef *bcd=bcl->first();
+    while (bcd)
+    {
+      ClassDef *ccd=bcd->classDef;
+      if (ccd && ccd->isVisibleInHierarchy() /*&& !ccd->visited*/) count++;
+      bcd=bcl->next();
+    }
   }
   if (count>0 && (prot!=Private || !doBases))
   {
@@ -479,6 +488,7 @@ void TreeDiagram::drawBoxes(QTextStream &t,Image *image,
                             bool doBase,bool bitmap,
                             uint baseRows,uint superRows,
                             uint cellWidth,uint cellHeight,
+                            QCString relPath,
                             bool generateMap)
 {
   DiagramRow *dr=first();
@@ -548,7 +558,7 @@ void TreeDiagram::drawBoxes(QTextStream &t,Image *image,
           writeBitmapBox(di,image,x,y,cellWidth,cellHeight,firstRow,
               hasDocs,di->getChildren()->count()>0); 
           if (!firstRow && generateMap) 
-            writeMapArea(t,di->getClassDef(),x,y,cellWidth,cellHeight);
+            writeMapArea(t,di->getClassDef(),relPath,x,y,cellWidth,cellHeight);
         }
         else
         {
@@ -581,7 +591,7 @@ void TreeDiagram::drawBoxes(QTextStream &t,Image *image,
           bool hasDocs=di->getClassDef()->isLinkable();
           writeBitmapBox(di,image,x,y,cellWidth,cellHeight,firstRow,hasDocs); 
           if (!firstRow && generateMap) 
-            writeMapArea(t,di->getClassDef(),x,y,cellWidth,cellHeight);
+            writeMapArea(t,di->getClassDef(),relPath,x,y,cellWidth,cellHeight);
         }
         else
         {
@@ -919,7 +929,7 @@ void TreeDiagram::drawConnectors(QTextStream &t,Image *image,
 
 void clearVisitFlags()
 {
-  ClassSDict::Iterator cli(Doxygen::classSDict);
+  ClassSDict::Iterator cli(*Doxygen::classSDict);
   ClassDef *cd;
   for (;(cd=cli.current());++cli)
   {
@@ -958,7 +968,7 @@ ClassDiagram::~ClassDiagram()
 }
 
 void ClassDiagram::writeFigure(QTextStream &output,const char *path,
-                               const char *fileName)
+                               const char *fileName) const
 {
   uint baseRows=base->computeRows();
   uint superRows=super->computeRows();
@@ -1242,7 +1252,7 @@ void ClassDiagram::writeFigure(QTextStream &output,const char *path,
     epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
                    epsBaseName.data(),epsBaseName.data());
     //printf("Converting eps using `%s'\n",epstopdfCmd.data());
-    if (iSystem("epstopdf",epstopdfArgs,TRUE)!=0)
+    if (portable_system("epstopdf",epstopdfArgs)!=0)
     {
        err("Error: Problems running epstopdf. Check your TeX installation!\n");
        return;
@@ -1252,7 +1262,8 @@ void ClassDiagram::writeFigure(QTextStream &output,const char *path,
 
 
 void ClassDiagram::writeImage(QTextStream &t,const char *path,
-                              const char *fileName, bool generateMap)
+                              const char *relPath,const char *fileName, 
+                              bool generateMap) const
 {
   uint baseRows=base->computeRows();
   uint superRows=super->computeRows();
@@ -1272,8 +1283,8 @@ void ClassDiagram::writeImage(QTextStream &t,const char *path,
 
   Image image(imageWidth,imageHeight);
 
-  base->drawBoxes(t,&image,TRUE,TRUE,baseRows,superRows,cellWidth,cellHeight,generateMap);
-  super->drawBoxes(t,&image,FALSE,TRUE,baseRows,superRows,cellWidth,cellHeight,generateMap);
+  base->drawBoxes(t,&image,TRUE,TRUE,baseRows,superRows,cellWidth,cellHeight,relPath,generateMap);
+  super->drawBoxes(t,&image,FALSE,TRUE,baseRows,superRows,cellWidth,cellHeight,relPath,generateMap);
   base->drawConnectors(t,&image,TRUE,TRUE,baseRows,superRows,cellWidth,cellHeight);
   super->drawConnectors(t,&image,FALSE,TRUE,baseRows,superRows,cellWidth,cellHeight);
 
