@@ -2,7 +2,7 @@
  *
  * $Id: $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -49,7 +49,7 @@ class DirDef : public Definition
     virtual ~DirDef();
 
     // accessors
-    virtual DefType definitionType() { return TypeDir; }
+    virtual DefType definitionType() const { return TypeDir; }
     virtual QCString getOutputFileBase() const;
     virtual bool isLinkableInProject() const;
     virtual bool isLinkable() const;
@@ -70,8 +70,6 @@ class DirDef : public Definition
     void writeDetailedDocumentation(OutputList &ol);
     void writeDocumentation(OutputList &ol);
     void writeDepGraph(QTextStream &t);
-    //void writePathFragment(OutputList &ol) const;
-    //void writeNavigationPath(OutputList &ol);
 
     static DirDef *mergeDirectoryInTree(const QCString &path);
     bool visited;
