@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -57,7 +57,8 @@ enum Tokens
   RetVal_TableHCell     = 0x1000E,
   RetVal_EndTable       = 0x1000F,
   RetVal_Internal       = 0x10010,
-  RetVal_SwitchLang     = 0x10011
+  RetVal_SwitchLang     = 0x10011,
+  RetVal_CloseXml       = 0x10012
 };
 
 struct TokenInfo
@@ -82,6 +83,7 @@ struct TokenInfo
 
   // simple section
   QString simpleSectName;
+  QString simpleSectText;
 
   // verbatim fragment
   QString verb;
@@ -92,6 +94,7 @@ struct TokenInfo
   // html tag
   HtmlAttribList attribs;
   bool endTag;
+  bool emptyTag;
 
   // whitespace
   QString chars;
@@ -122,13 +125,16 @@ bool doctokenizerYYpopContext();
 int  doctokenizerYYlex();
 void doctokenizerYYsetStatePara();
 void doctokenizerYYsetStateTitle();
+void doctokenizerYYsetStateTitleAttrValue();
 void doctokenizerYYsetStateCode();
+void doctokenizerYYsetStateXmlCode();
 void doctokenizerYYsetStateHtmlOnly();
 void doctokenizerYYsetStateManOnly();
 void doctokenizerYYsetStateLatexOnly();
 void doctokenizerYYsetStateXmlOnly();
 void doctokenizerYYsetStateVerbatim();
 void doctokenizerYYsetStateDot();
+void doctokenizerYYsetStateMsc();
 void doctokenizerYYsetStateParam();
 void doctokenizerYYsetStateXRefItem();
 void doctokenizerYYsetStateFile();
@@ -139,5 +145,6 @@ void doctokenizerYYsetStateInternalRef();
 void doctokenizerYYsetStateText();
 void doctokenizerYYsetStateSkipTitle();
 void doctokenizerYYsetInsidePre(bool b);
+void doctokenizerYYpushBackHtmlTag(const char *tag);
 
 #endif
