@@ -3,7 +3,7 @@
  * $Id: doxygen.h,v 1.39 2001/03/19 19:27:40 root Exp $
  *
  *
- * Copyright (C) 1997-2004 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,6 +22,7 @@
 #include "qtbc.h"
 #include <qtextstream.h>
 #include <qdatetime.h>
+#include <qcache.h>
 #include "groupdef.h"
 #include "filedef.h"
 #include "classdef.h"
@@ -34,12 +35,19 @@
 #include "section.h"
 #include "membergroup.h"
 #include "reflist.h"
-//#include "packagedef.h"
+#include "dirdef.h"
+#include "index.h"
 
 class PageSList;
 class PageSDict;
 class PageDef;
 class SearchIndex;
+class DirDef;
+class ParserManager;
+class ObjCache;
+class Store;
+class QFileInfo;
+class BufStr;
 
 typedef QList<QCString>    StringList;
 typedef QDict<FileDef>     FileDict;
@@ -52,6 +60,16 @@ class StringDict : public QDict<QCString>
     virtual ~StringDict() {}
 };
 
+struct LookupInfo
+{
+  LookupInfo() : classDef(0), typeDef(0) {}
+  LookupInfo(ClassDef *cd,MemberDef *td,QCString ts,QCString rt) 
+    : classDef(cd), typeDef(td), templSpec(ts),resolvedType(rt) {}
+  ClassDef  *classDef;
+  MemberDef *typeDef;
+  QCString   templSpec;
+  QCString   resolvedType;
+};
 
 extern QCString spaces;
 
@@ -62,8 +80,8 @@ extern QCString spaces;
 class Doxygen
 {
   public:
-    static ClassSDict                classSDict;
-    static ClassSDict                hiddenClasses;
+    static ClassSDict               *classSDict;
+    static ClassSDict               *hiddenClasses;
     static PageSDict                *exampleSDict;
     static PageSDict                *pageSDict;
     static PageDef                  *mainPage;
@@ -71,20 +89,17 @@ class Doxygen
     static FileNameDict             *includeNameDict;
     static FileNameDict             *exampleNameDict;
     static FileNameDict             *inputNameDict;
-    static FileNameList              inputNameList;
+    static FileNameList             *inputNameList;
     static FileNameDict             *imageNameDict;
     static FileNameDict             *dotFileNameDict;
     static QStrList                  tagfileList;
-    static MemberNameSDict           memberNameSDict;
-    static MemberNameSDict           functionNameSDict;
-    static FileList                  fileList;
-    static FileDict                  fileDict;
-    static ClassDef                  unrelatedClass; 
+    static MemberNameSDict          *memberNameSDict;
+    static MemberNameSDict          *functionNameSDict;
     static QTextStream               tagFile;
     static SectionDict               sectionDict;
     static StringDict                namespaceAliasDict;
-    static GroupSDict                groupSDict;
-    static NamespaceSDict            namespaceSDict;
+    static GroupSDict               *groupSDict;
+    static NamespaceSDict           *namespaceSDict;
     static FormulaList               formulaList;
     static FormulaDict               formulaDict;
     static FormulaDict               formulaNameDict;
@@ -99,13 +114,56 @@ class Doxygen
     static double                    sysElapsedTime;
     static QTime                     runningTime;
     static SearchIndex              *searchIndex;
-    static SDict<DefinitionList>    *symbolMap;
+    static QDict<DefinitionIntf>    *symbolMap;
     static bool                      outputToWizard;
+    static QDict<int>               *htmlDirMap;
+    static QCache<LookupInfo>        lookupCache;
+    static DirSDict                 *directories;
+    static SDict<DirRelation>        dirRelations;
+    static ParserManager            *parserManager;
+    static bool                      suppressDocWarnings;
+    static ObjCache                 *symbolCache;
+    static Store                    *symbolStorage;
+    static QCString                  objDBFileName;
+    static QCString                  entryDBFileName;
+    static bool                      gatherDefines;
+    static bool                      userComments;
+    static IndexList                 indexList;
 };
 
 void initDoxygen();
 void readConfiguration(int argc, char **argv);
+void checkConfiguration();
 void parseInput();
 void generateOutput();
+void readAliases();
+void readFormulaRepository();
+int readFileOrDirectory(const char *s,
+                        FileNameList *fnList,
+                        FileNameDict *fnDict,
+                        StringDict *exclDict,
+                        QStrList *patList,
+                        QStrList *exclPatList,
+                        StringList *resultList,
+                        StringDict *resultDict,
+                        bool recursive,
+                        bool errorIfNotExist=TRUE,
+                        QDict<void> *killDict = 0
+                       );
+int readDir(QFileInfo *fi,
+            FileNameList *fnList,
+            FileNameDict *fnDict,
+            StringDict  *exclDict,
+            QStrList *patList,
+            QStrList *exclPatList,
+            StringList *resultList,
+            StringDict *resultDict,
+            bool errorIfNotExist,
+            bool recursive,
+            QDict<void> *killDict
+           );
+void copyAndFilterFile(const char *fileName,BufStr &dest);
+
+#define NEWMATCH
 
 #endif
