@@ -2,7 +2,7 @@
  *
  * $Id: filedef.cpp,v 1.55 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -29,23 +29,18 @@
 #include "dot.h"
 #include "message.h"
 #include "docparser.h"
-#include "ftvhelp.h"
 #include "searchindex.h"
 #include "htags.h"
 #include "parserintf.h"
-
-#if defined(_MSC_VER) || defined(__BORLANDC__)
-#define popen _popen
-#define pclose _pclose
-#endif
+#include "portable.h"
 
 class DevNullCodeDocInterface : public CodeOutputInterface
 {
   public:
     virtual void codify(const char *) {}
-    virtual void writeCodeLink(const char *ref,const char *file,
-                               const char *anchor,const char *name) 
-    { ref=ref; file=file; anchor=anchor; name=name; }
+    virtual void writeCodeLink(const char *,const char *,
+                               const char *,const char *,
+                               const char *) {}
     virtual void writeLineNumber(const char *,const char *,
                                  const char *,int) {}
     virtual void startCodeLine() {}
@@ -55,10 +50,9 @@ class DevNullCodeDocInterface : public CodeOutputInterface
     virtual void startFontClass(const char *) {}
     virtual void endFontClass() {}
     virtual void writeCodeAnchor(const char *) {}
+    virtual void linkableSymbol(int, const char *,Definition *,Definition *) {}
 };
 
-
-
 /*! create a new file definition, where \a p is the file path, 
     \a nm the file name, and \a ref is an HTML anchor name if the
     file was read from a tag file or 0 otherwise
@@ -73,39 +67,28 @@ FileDef::FileDef(const char *p,const char *nm,
   diskname=dn;
   if (diskname.isEmpty()) diskname=nm;
   setReference(lref);
-  //printf("new FileDef(path=%s,name=%s,ref=%s)\n",p,nm,lref);
-  classSDict     = new ClassSDict(17);
-  includeList   = new QList<IncludeInfo>;
-  includeList->setAutoDelete(TRUE);
-  includeDict   = new QDict<IncludeInfo>(61);
-  includedByList = new QList<IncludeInfo>;
-  includedByList->setAutoDelete(TRUE);
-  includedByDict = new QDict<IncludeInfo>(61);
-  namespaceSDict = new NamespaceSDict;
-  srcDefDict = 0;
-  srcMemberDict = 0;
-  usingDirList = 0;
-  usingDeclList = 0;
-  package = 0;
-  isSource = FALSE; 
-  docname = nm;
-  dir = 0;
+  classSDict        = 0;
+  includeList       = 0;
+  includeDict       = 0; 
+  includedByList    = 0;
+  includedByDict    = 0; 
+  namespaceSDict    = 0; 
+  srcDefDict        = 0;
+  srcMemberDict     = 0;
+  usingDirList      = 0;
+  usingDeclList     = 0;
+  package           = 0;
+  isSource          = FALSE; 
+  docname           = nm;
+  dir               = 0;
   if (Config_getBool("FULL_PATH_NAMES"))
   {
     docname.prepend(stripFromPath(path.copy()));
   }
-  memberGroupSDict = new MemberGroupSDict;
-  memberGroupSDict->setAutoDelete(TRUE);
+  m_isJava          = name().right(5)==".java";
+  memberGroupSDict = 0;
   acquireFileVersion();
-
-  // members in the detailed part of the documentation
-  docDefineMembers.setInFile(TRUE);
-  docProtoMembers.setInFile(TRUE);
-  docTypedefMembers.setInFile(TRUE);
-  docEnumMembers.setInFile(TRUE);
-  docFuncMembers.setInFile(TRUE);
-  docVarMembers.setInFile(TRUE);
-
+  m_subGrouping=Config_getBool("SUBGROUPING");
 }
 
 /*! destroy the file definition */
@@ -127,35 +110,46 @@ FileDef::~FileDef()
 /*! Compute the HTML anchor names for all members in the class */ 
 void FileDef::computeAnchors()
 {
-  setAnchors(0,'a',&allMemberList);
+  MemberList *ml = getMemberList(MemberList::allMembersList);
+  if (ml) setAnchors(0,'a',ml);
 }
 
 void FileDef::distributeMemberGroupDocumentation()
 {
   //printf("FileDef::distributeMemberGroupDocumentation()\n");
-  MemberGroupSDict::Iterator mgli(*memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
+  if (memberGroupSDict)
   {
-    mg->distributeMemberGroupDocumentation();
+    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      mg->distributeMemberGroupDocumentation();
+    }
   }
 }
 
 void FileDef::findSectionsInDocumentation()
 {
   docFindSections(documentation(),this,0,docFile());
-  MemberGroupSDict::Iterator mgli(*memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
+  if (memberGroupSDict)
   {
-    mg->findSectionsInDocumentation();
+    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      mg->findSectionsInDocumentation();
+    }
+  }
+
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::declarationLists)
+    {
+      ml->findSectionsInDocumentation();
+    }
   }
-  decDefineMembers.findSectionsInDocumentation();
-  decProtoMembers.findSectionsInDocumentation();
-  decTypedefMembers.findSectionsInDocumentation();
-  decEnumMembers.findSectionsInDocumentation();
-  decFuncMembers.findSectionsInDocumentation();
-  decVarMembers.findSectionsInDocumentation();
 }
 
 void FileDef::writeDetailedDocumentation(OutputList &ol)
@@ -173,6 +167,7 @@ void FileDef::writeDetailedDocumentation(OutputList &ol)
     ol.startGroupHeader();
     ol.parseText(theTranslator->trDetailedDescription());
     ol.endGroupHeader();
+    ol.startTextBlock();
     if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
     {
       ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
@@ -197,7 +192,7 @@ void FileDef::writeDetailedDocumentation(OutputList &ol)
     //printf("Writing source ref for file %s\n",name().data());
     if (Config_getBool("SOURCE_BROWSER")) 
     {
-      ol.newParagraph();
+      ol.startParagraph();
       QCString refText = theTranslator->trDefinedInSourceFile();
       int fileMarkerPos = refText.find("@0");
       if (fileMarkerPos!=-1) // should always pass this.
@@ -208,7 +203,9 @@ void FileDef::writeDetailedDocumentation(OutputList &ol)
         ol.parseText(refText.right(
               refText.length()-fileMarkerPos-2)); // text right from marker 2
       }
+      ol.endParagraph();
     }
+    ol.endTextBlock();
   }
 }
 
@@ -313,7 +310,8 @@ void FileDef::writeDocumentation(OutputList &ol)
   }
   ol.writeSynopsis();
  
-  if (Config_getBool("SHOW_INCLUDE_FILES") && includeList->count()>0)
+  if (Config_getBool("SHOW_INCLUDE_FILES") && includeList && 
+      includeList->count()>0)
   {
     ol.startTextBlock(TRUE);
     QListIterator<IncludeInfo> ili(*includeList);
@@ -324,9 +322,10 @@ void FileDef::writeDocumentation(OutputList &ol)
       bool isIDLorJava = FALSE;
       if (fd)
       {
-        isIDLorJava = fd->name().right(4)==".idl" || 
+        isIDLorJava = fd->name().right(4)==".idl" ||   // M$ or Corba IDL
                       fd->name().right(5)==".pidl" ||
-                      fd->name().right(5)==".java";
+                      fd->name().right(5)==".java" ||  // Sun's Java
+                      fd->name().right(4)==".jsl";     // M$ J#
       }
       ol.startTypewriter();
       if (isIDLorJava) // IDL/Java include
@@ -393,8 +392,8 @@ void FileDef::writeDocumentation(OutputList &ol)
   if (Config_getBool("HAVE_DOT") && Config_getBool("INCLUDE_GRAPH"))
   {
     //printf("Graph for file %s\n",name().data());
-    DotInclDepGraph incDepGraph(this,Config_getInt("MAX_DOT_GRAPH_DEPTH"),FALSE);
-    if (!incDepGraph.isTrivial())
+    DotInclDepGraph incDepGraph(this,FALSE);
+    if (!incDepGraph.isTrivial() && !incDepGraph.isTooBig())
     {
       ol.startTextBlock(); 
       ol.disable(OutputGenerator::Man);
@@ -411,8 +410,8 @@ void FileDef::writeDocumentation(OutputList &ol)
   if (Config_getBool("HAVE_DOT") && Config_getBool("INCLUDED_BY_GRAPH"))
   {
     //printf("Graph for file %s\n",name().data());
-    DotInclDepGraph incDepGraph(this,Config_getInt("MAX_DOT_GRAPH_DEPTH"),TRUE);
-    if (!incDepGraph.isTrivial())
+    DotInclDepGraph incDepGraph(this,TRUE);
+    if (!incDepGraph.isTrivial() && !incDepGraph.isTooBig())
     {
       ol.startTextBlock(); 
       ol.disable(OutputGenerator::Man);
@@ -439,7 +438,7 @@ void FileDef::writeDocumentation(OutputList &ol)
   
   ol.startMemberSections();
 
-  if (namespaceSDict->count()>0)
+  if (namespaceSDict && namespaceSDict->count()>0)
   {
     NamespaceSDict::Iterator ndi(*namespaceSDict);
     NamespaceDef *nd;
@@ -483,26 +482,30 @@ void FileDef::writeDocumentation(OutputList &ol)
     if (found) ol.endMemberList();
   }
 
-  classSDict->writeDeclaration(ol);
+  if (classSDict) classSDict->writeDeclaration(ol);
   
   /* write user defined member groups */
-  MemberGroupSDict::Iterator mgli(*memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
+  if (memberGroupSDict)
   {
-    if (mg->header()!="[NOHEADER]")
+    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
     {
-      mg->writeDeclarations(ol,0,0,this,0);
+      if ((!mg->allMembersInSameSection() || !m_subGrouping) 
+          && mg->header()!="[NOHEADER]")
+      {
+        mg->writeDeclarations(ol,0,0,this,0);
+      }
     }
   }
 
   //allMemberList.writeDeclarations(ol,0,0,this,0,0,0);
-  decDefineMembers.writeDeclarations(ol,0,0,this,0,theTranslator->trDefines(),0);
-  decProtoMembers.writeDeclarations(ol,0,0,this,0,theTranslator->trFuncProtos(),0);
-  decTypedefMembers.writeDeclarations(ol,0,0,this,0,theTranslator->trTypedefs(),0);
-  decEnumMembers.writeDeclarations(ol,0,0,this,0,theTranslator->trEnumerations(),0);
-  decFuncMembers.writeDeclarations(ol,0,0,this,0,theTranslator->trFunctions(),0);
-  decVarMembers.writeDeclarations(ol,0,0,this,0,theTranslator->trVariables(),0);
+  writeMemberDeclarations(ol,MemberList::decDefineMembers,theTranslator->trDefines());
+  writeMemberDeclarations(ol,MemberList::decProtoMembers,theTranslator->trFuncProtos());
+  writeMemberDeclarations(ol,MemberList::decTypedefMembers,theTranslator->trTypedefs());
+  writeMemberDeclarations(ol,MemberList::decEnumMembers,theTranslator->trEnumerations());
+  writeMemberDeclarations(ol,MemberList::decFuncMembers,theTranslator->trFunctions());
+  writeMemberDeclarations(ol,MemberList::decVarMembers,theTranslator->trVariables());
   ol.endMemberSections();
 
   if (!Config_getBool("DETAILS_AT_TOP"))
@@ -530,7 +533,8 @@ void FileDef::writeDocumentation(OutputList &ol)
 
   if (Config_getBool("SEPARATE_MEMBER_PAGES"))
   {
-    allMemberList.sort();
+    MemberList *ml = getMemberList(MemberList::allMembersList);
+    if (ml) ml->sort();
     writeMemberPages(ol);
   }
 }
@@ -541,24 +545,13 @@ void FileDef::writeMemberDocumentation(OutputList &ol)
   {
     ol.disable(OutputGenerator::Html);
   }
-  
-  docDefineMembers.writeDocumentation(ol,name(),this,
-                            theTranslator->trDefineDocumentation());
-  
-  docProtoMembers.writeDocumentation(ol,name(),this,
-                            theTranslator->trFunctionPrototypeDocumentation());
 
-  docTypedefMembers.writeDocumentation(ol,name(),this,
-                            theTranslator->trTypedefDocumentation());
-  
-  docEnumMembers.writeDocumentation(ol,name(),this,
-                            theTranslator->trEnumerationTypeDocumentation());
-
-  docFuncMembers.writeDocumentation(ol,name(),this,
-                            theTranslator->trFunctionDocumentation());
-  
-  docVarMembers.writeDocumentation(ol,name(),this,
-                            theTranslator->trVariableDocumentation());
+  writeMemberDocumentation(ol,MemberList::docDefineMembers,theTranslator->trDefineDocumentation());
+  writeMemberDocumentation(ol,MemberList::docProtoMembers,theTranslator->trFunctionPrototypeDocumentation());
+  writeMemberDocumentation(ol,MemberList::docTypedefMembers,theTranslator->trTypedefDocumentation());
+  writeMemberDocumentation(ol,MemberList::docEnumMembers,theTranslator->trEnumerationTypeDocumentation());
+  writeMemberDocumentation(ol,MemberList::docFuncMembers,theTranslator->trFunctionDocumentation());
+  writeMemberDocumentation(ol,MemberList::docVarMembers,theTranslator->trVariableDocumentation());
 
   if (Config_getBool("SEPARATE_MEMBER_PAGES"))
   {
@@ -571,12 +564,15 @@ void FileDef::writeMemberPages(OutputList &ol)
   ol.pushGeneratorState();
   ol.disableAllBut(OutputGenerator::Html);
   
-  docDefineMembers.writeDocumentationPage(ol,name(),this);
-  docProtoMembers.writeDocumentationPage(ol,name(),this);
-  docTypedefMembers.writeDocumentationPage(ol,name(),this);
-  docEnumMembers.writeDocumentationPage(ol,name(),this);
-  docFuncMembers.writeDocumentationPage(ol,name(),this);
-  docVarMembers.writeDocumentationPage(ol,name(),this);
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::documentationLists)
+    {
+      ml->writeDocumentationPage(ol,name(),this);
+    }
+  }
 
   ol.popGeneratorState();
 }
@@ -588,31 +584,35 @@ void FileDef::writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const
   ol.writeString("      <div class=\"navtab\">\n");
   ol.writeString("        <table>\n");
 
-  MemberListIterator mli(allMemberList);
-  MemberDef *md;
-  for (mli.toFirst();(md=mli.current());++mli)
+  MemberList *allMemberList = getMemberList(MemberList::allMembersList);
+  if (allMemberList)
   {
-    if (md->getFileDef()==this && md->getNamespaceDef()==0 && md->isLinkable())
+    MemberListIterator mli(*allMemberList);
+    MemberDef *md;
+    for (mli.toFirst();(md=mli.current());++mli)
     {
-      ol.writeString("          <tr><td class=\"navtab\">");
-      if (md->isLinkableInProject())
+      if (md->getFileDef()==this && md->getNamespaceDef()==0 && md->isLinkable())
       {
-        if (md==currentMd) // selected item => highlight
+        ol.writeString("          <tr><td class=\"navtab\">");
+        if (md->isLinkableInProject())
         {
-          ol.writeString("<a class=\"qindexHL\" ");
+          if (md==currentMd) // selected item => highlight
+          {
+            ol.writeString("<a class=\"qindexHL\" ");
+          }
+          else
+          {
+            ol.writeString("<a class=\"qindex\" ");
+          }
+          ol.writeString("href=\"");
+          if (createSubDirs) ol.writeString("../../");
+          ol.writeString(md->getOutputFileBase()+Doxygen::htmlFileExtension+"#"+md->anchor());
+          ol.writeString("\">");
+          ol.writeString(md->localName());
+          ol.writeString("</a>");
         }
-        else
-        {
-          ol.writeString("<a class=\"qindex\" ");
-        }
-        ol.writeString("href=\"");
-        if (createSubDirs) ol.writeString("../../");
-        ol.writeString(md->getOutputFileBase()+Doxygen::htmlFileExtension+"#"+md->anchor());
-        ol.writeString("\">");
-        ol.writeString(md->localName());
-        ol.writeString("</a>");
+        ol.writeString("</td></tr>\n");
       }
-      ol.writeString("</td></tr>\n");
     }
   }
 
@@ -623,6 +623,7 @@ void FileDef::writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const
 /*! Write a source listing of this file to the output */
 void FileDef::writeSource(OutputList &ol)
 {
+  static bool filterSourceFiles = Config_getBool("FILTER_SOURCE_FILES");
   QCString title = docname;
   if (!fileVersion.isEmpty())
   {
@@ -657,7 +658,7 @@ void FileDef::writeSource(OutputList &ol)
   pIntf->resetCodeParserState();
   ol.startCodeFragment();
   pIntf->parseCode(ol,0,
-            fileToString(absFilePath(),Config_getBool("FILTER_SOURCE_FILES")),
+            fileToString(absFilePath(),filterSourceFiles),
             FALSE,0,this
            );
   ol.endCodeFragment();
@@ -667,73 +668,98 @@ void FileDef::writeSource(OutputList &ol)
 
 void FileDef::parseSource()
 {
+  static bool filterSourceFiles = Config_getBool("FILTER_SOURCE_FILES");
   DevNullCodeDocInterface devNullIntf;
   ParserInterface *pIntf = Doxygen::parserManager->getParser(getDefFileExtension());
   pIntf->resetCodeParserState();
   pIntf->parseCode(
             devNullIntf,0,
-            fileToString(absFilePath(),Config_getBool("FILTER_SOURCE_FILES")),
+            fileToString(absFilePath(),filterSourceFiles),
             FALSE,0,this
            );
 }
 
 void FileDef::addMembersToMemberGroup()
 {
-  ::addMembersToMemberGroup(&decDefineMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decProtoMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decTypedefMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decEnumMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decFuncMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decVarMembers,memberGroupSDict,this);
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::declarationLists)
+    {
+      ::addMembersToMemberGroup(ml,&memberGroupSDict,this);
+    }
+  }
+
+  // add members inside sections to their groups
+  if (memberGroupSDict)
+  {
+    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      if (mg->allMembersInSameSection() && m_subGrouping)
+      {
+        //printf("----> addToDeclarationSection(%s)\n",mg->header().data());
+        mg->addToDeclarationSection();
+      }
+    }
+  }
 }
 
 /*! Adds member definition \a md to the list of all members of this file */
 void FileDef::insertMember(MemberDef *md)
 {
+  if (md->isHidden()) return;
   //printf("%s:FileDef::insertMember(%s (=%p) list has %d elements)\n",
   //    name().data(),md->name().data(),md,allMemberList.count());
-  if (allMemberList.findRef(md)!=-1) 
+  MemberList *allMemberList = getMemberList(MemberList::allMembersList);
+  if (allMemberList && allMemberList->findRef(md)!=-1)  // TODO optimize the findRef!
   { 
     return;
   }
 
-  allMemberList.append(md); 
-  bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
-  bool sortMemberDocs = Config_getBool("SORT_MEMBER_DOCS");
+  if (allMemberList==0)
+  {
+    allMemberList = new MemberList(MemberList::allMembersList);;
+    m_memberLists.append(allMemberList);
+  }
+  allMemberList->append(md); 
+  //::addFileMemberNameToIndex(md);
   switch (md->memberType())
   {
     case MemberDef::Variable:     
     case MemberDef::Property:     
-      if (sortBriefDocs)  decVarMembers.inSort(md); else decVarMembers.append(md);
-      if (sortMemberDocs) docVarMembers.inSort(md); else docVarMembers.append(md);
+      addMemberToList(MemberList::decVarMembers,md);
+      addMemberToList(MemberList::docVarMembers,md);
       break;
     case MemberDef::Function: 
-      if (sortBriefDocs)  decFuncMembers.inSort(md); else decFuncMembers.append(md);
-      if (sortMemberDocs) docFuncMembers.inSort(md); else docFuncMembers.append(md);
+      addMemberToList(MemberList::decFuncMembers,md);
+      addMemberToList(MemberList::docFuncMembers,md);
       break;
     case MemberDef::Typedef:      
-      if (sortBriefDocs)  decTypedefMembers.inSort(md); else decTypedefMembers.append(md);
-      if (sortMemberDocs) docTypedefMembers.inSort(md); else docTypedefMembers.append(md);
+      addMemberToList(MemberList::decTypedefMembers,md);
+      addMemberToList(MemberList::docTypedefMembers,md);
       break;
     case MemberDef::Enumeration:  
-      if (sortBriefDocs)  decEnumMembers.inSort(md); else decEnumMembers.append(md);
-      if (sortMemberDocs) docEnumMembers.inSort(md); else docEnumMembers.append(md);
+      addMemberToList(MemberList::decEnumMembers,md);
+      addMemberToList(MemberList::docEnumMembers,md);
       break;
     case MemberDef::EnumValue:    // enum values are shown inside their enums
       break;
     case MemberDef::Prototype:    
-      if (sortBriefDocs)  decProtoMembers.inSort(md); else decProtoMembers.append(md);
-      if (sortMemberDocs) docProtoMembers.inSort(md); else docProtoMembers.append(md);
+      addMemberToList(MemberList::decProtoMembers,md);
+      addMemberToList(MemberList::docProtoMembers,md);
       break;
     case MemberDef::Define:       
-      if (sortBriefDocs)  decDefineMembers.inSort(md); else decDefineMembers.append(md);
-      if (sortMemberDocs) docDefineMembers.inSort(md); else docDefineMembers.append(md);
+      addMemberToList(MemberList::decDefineMembers,md);
+      addMemberToList(MemberList::docDefineMembers,md);
       break;
     default:
        err("FileDef::insertMembers(): "
            "member `%s' with class scope `%s' inserted in file scope `%s'!\n",
            md->name().data(),
-           md->getClassDef() ? md->getClassDef()->name().data() : "",
+           md->getClassDef() ? md->getClassDef()->name().data() : "<global>",
            name().data());
   }
   //addMemberToGroup(md,groupId);
@@ -742,6 +768,11 @@ void FileDef::insertMember(MemberDef *md)
 /*! Adds compound definition \a cd to the list of all compounds of this file */
 void FileDef::insertClass(ClassDef *cd)
 {
+  if (cd->isHidden()) return;
+  if (classSDict==0)
+  {
+    classSDict = new ClassSDict(17);
+  }
   if (Config_getBool("SORT_BRIEF_DOCS"))
     classSDict->inSort(cd->name(),cd);
   else
@@ -751,8 +782,14 @@ void FileDef::insertClass(ClassDef *cd)
 /*! Adds namespace definition \a nd to the list of all compounds of this file */
 void FileDef::insertNamespace(NamespaceDef *nd)
 {
-  if (!nd->name().isEmpty() && namespaceSDict->find(nd->name())==0)
+  if (nd->isHidden()) return;
+  if (!nd->name().isEmpty() && 
+      (namespaceSDict==0 || namespaceSDict->find(nd->name())==0))
   {
+    if (namespaceSDict==0)
+    {
+      namespaceSDict = new NamespaceSDict;
+    }
     if (Config_getBool("SORT_BRIEF_DOCS"))
       namespaceSDict->inSort(nd->name(),nd);
     else
@@ -805,6 +842,13 @@ void FileDef::addUsingDirective(NamespaceDef *nd)
   {
     usingDirList->append(nd->qualifiedName(),nd);
   }
+  //printf("%p: FileDef::addUsingDirective: %s:%d\n",this,name().data(),usingDirList->count());
+}
+
+NamespaceSDict *FileDef::getUsedNamespaces() const 
+{ 
+  //printf("%p: FileDef::getUsedNamespace: %s:%d\n",this,name().data(),usingDirList?usingDirList->count():0);
+  return usingDirList; 
 }
 
 void FileDef::addUsingDeclaration(Definition *d)
@@ -824,8 +868,14 @@ void FileDef::addIncludeDependency(FileDef *fd,const char *incName,bool local,
 {
   //printf("FileDef::addIncludeDependency(%p,%s,%d)\n",fd,incName,local);
   QCString iName = fd ? fd->absFilePath().data() : incName;
-  if (!iName.isEmpty() && includeDict->find(iName)==0)
+  if (!iName.isEmpty() && (!includeDict || includeDict->find(iName)==0))
   {
+    if (includeDict==0)
+    {
+      includeDict   = new QDict<IncludeInfo>(61);
+      includeList   = new QList<IncludeInfo>;
+      includeList->setAutoDelete(TRUE);
+    }
     IncludeInfo *ii = new IncludeInfo;
     ii->fileDef     = fd;
     ii->includeName = incName;
@@ -876,7 +926,10 @@ void FileDef::addIncludedUsingDirectives()
               if (usingDirList==0) usingDirList = new NamespaceSDict;
               //printf("Prepending used namespace %s to the list of file %s\n",
               //    nd->name().data(),name().data());
-              usingDirList->prepend(nd->qualifiedName(),nd);
+              if (usingDirList->find(nd->qualifiedName())==0) // not yet added
+              {
+                usingDirList->prepend(nd->qualifiedName(),nd);
+              }
             }
           }
         }
@@ -892,8 +945,14 @@ void FileDef::addIncludedByDependency(FileDef *fd,const char *incName,
 {
   //printf("FileDef::addIncludedByDependency(%p,%s,%d)\n",fd,incName,local);
   QCString iName = fd ? fd->absFilePath().data() : incName;
-  if (!iName.isEmpty() && includedByDict->find(iName)==0)
+  if (!iName.isEmpty() && (includedByDict==0 || includedByDict->find(iName)==0))
   {
+    if (includedByDict==0)
+    {
+      includedByDict = new QDict<IncludeInfo>(61);
+      includedByList = new QList<IncludeInfo>;
+      includedByList->setAutoDelete(TRUE);
+    }
     IncludeInfo *ii = new IncludeInfo;
     ii->fileDef     = fd;
     ii->includeName = incName;
@@ -907,7 +966,7 @@ void FileDef::addIncludedByDependency(FileDef *fd,const char *incName,
 bool FileDef::isIncluded(const QCString &name) const
 {
   if (name.isEmpty()) return FALSE;
-  return includeDict->find(name)!=0;
+  return includeDict!=0 && includeDict->find(name)!=0;
 }
 
 bool FileDef::generateSourceFile() const 
@@ -923,22 +982,31 @@ bool FileDef::generateSourceFile() const
 
 void FileDef::addListReferences()
 {
-  addRefItem(xrefListItems(),
-             theTranslator->trFile(TRUE,TRUE),
-             getOutputFileBase(),name()
-            );
-  MemberGroupSDict::Iterator mgli(*memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
   {
-    mg->addListReferences(this);
+    LockingPtr< QList<ListItemInfo> > xrefItems = xrefListItems();
+    addRefItem(xrefItems.pointer(),
+               theTranslator->trFile(TRUE,TRUE),
+               getOutputFileBase(),name()
+              );
+  }
+  if (memberGroupSDict)
+  {
+    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      mg->addListReferences(this);
+    }
+  }
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::documentationLists)
+    {
+      ml->addListReferences(this);
+    }
   }
-  docDefineMembers.addListReferences(this);
-  docProtoMembers.addListReferences(this);
-  docTypedefMembers.addListReferences(this);
-  docEnumMembers.addListReferences(this);
-  docFuncMembers.addListReferences(this);
-  docVarMembers.addListReferences(this);
 }
 
 //-------------------------------------------------------------------
@@ -1150,7 +1218,7 @@ static void addDirsAsGroups(Directory *root,GroupDef *parent,int level)
     }
     else
     {
-      Doxygen::groupSDict.append(root->path(),gd);
+      Doxygen::groupSDict->append(root->path(),gd);
     }
   }
   QListIterator<DirEntry> dli(root->children());
@@ -1166,11 +1234,9 @@ static void addDirsAsGroups(Directory *root,GroupDef *parent,int level)
 
 void generateFileTree()
 {
-  FTVHelp::generateTreeViewImages();
-  
   Directory *root=new Directory(0,"root");
   root->setLast(TRUE);
-  FileNameListIterator fnli(Doxygen::inputNameList); 
+  FileNameListIterator fnli(*Doxygen::inputNameList); 
   FileName *fn;
   for (fnli.toFirst();(fn=fnli.current());++fnli)
   {
@@ -1243,7 +1309,7 @@ void FileDef::acquireFileVersion()
   if (!vercmd.isEmpty()) 
   {
     msg("Version of %s : ",filepath.data());
-    FILE *f=popen("\""+vercmd+"\" \""+filepath+"\"","r");
+    FILE *f=portable_popen(vercmd+" \""+filepath+"\"","r");
     if (!f)
     {
       err("Error: could not execute %s\n",vercmd.data());
@@ -1252,7 +1318,7 @@ void FileDef::acquireFileVersion()
     const int bufSize=1024;
     char buf[bufSize];
     int numRead = fread(buf,1,bufSize,f);
-    pclose(f);
+    portable_pclose(f);
     if (numRead > 0) 
     {
       fileVersion = QCString(buf,numRead).stripWhiteSpace();
@@ -1291,3 +1357,65 @@ QCString FileDef::includeName() const
   }
 }
 
+MemberList *FileDef::createMemberList(MemberList::ListType lt)
+{
+  m_memberLists.setAutoDelete(TRUE);
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()==lt)
+    {
+      return ml;
+    }
+  }
+  // not found, create a new member list
+  ml = new MemberList(lt);
+  m_memberLists.append(ml);
+  return ml;
+}
+
+void FileDef::addMemberToList(MemberList::ListType lt,MemberDef *md)
+{
+  static bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
+  static bool sortMemberDocs = Config_getBool("SORT_MEMBER_DOCS");
+  MemberList *ml = createMemberList(lt);
+  if (((ml->listType()&MemberList::declarationLists) && sortBriefDocs) ||
+      ((ml->listType()&MemberList::documentationLists) && sortMemberDocs)
+     )
+    ml->inSort(md);
+  else
+    ml->append(md);
+  if (lt&MemberList::documentationLists)
+  {
+    ml->setInFile(TRUE);
+  }
+  if (ml->listType()&MemberList::declarationLists) md->setSectionList(this,ml);
+}
+
+MemberList *FileDef::getMemberList(MemberList::ListType lt) const
+{
+  FileDef *that = (FileDef*)this;
+  MemberList *ml = that->m_memberLists.first();
+  while (ml)
+  {
+    if (ml->listType()==lt)
+    {
+      return ml;
+    }
+    ml = that->m_memberLists.next();
+  }
+  return 0;
+}
+
+void FileDef::writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title)
+{
+  MemberList * ml = getMemberList(lt);
+  if (ml) ml->writeDeclarations(ol,0,0,this,0,title,0);
+}
+
+void FileDef::writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title)
+{
+  MemberList * ml = getMemberList(lt);
+  if (ml) ml->writeDocumentation(ol,name(),this,title);
+}
