@@ -347,6 +347,7 @@ FTVHelp::~FTVHelp()
   delete[] m_indentNodes;
 }
 
+#if 0
 /*! return a reference to the one and only instance of this class. 
  */
 FTVHelp *FTVHelp::getInstance()
@@ -354,6 +355,7 @@ FTVHelp *FTVHelp::getInstance()
   if (m_theInstance==0) m_theInstance = new FTVHelp;
   return m_theInstance;
 }
+#endif
 
 /*! This will create a folder tree view table of contents file (tree.js).
  *  \sa finalize()
@@ -369,6 +371,7 @@ void FTVHelp::initialize()
 void FTVHelp::finalize()
 {
   generateTreeView();
+  generateTreeViewImages();
   delete m_theInstance;
   m_theInstance=0;
 }
@@ -377,19 +380,19 @@ void FTVHelp::finalize()
  *  This will start a new sublist in contents file.
  *  \sa decContentsDepth()
  */
-int FTVHelp::incContentsDepth()
+void FTVHelp::incContentsDepth()
 {
   //int i; for (i=0;i<m_dc+1;i++) m_cts << "  ";
   m_indent++;
   ASSERT(m_indent<MAX_INDENT);
-  return m_indent;
+  //return m_indent;
 }
 
 /*! Decrease the level of the contents hierarchy.
  *  This will end the current sublist.
  *  \sa incContentsDepth()
  */
-int FTVHelp::decContentsDepth()
+void FTVHelp::decContentsDepth()
 {
   //int i; for (i=0;i<m_dc;i++) m_cts << "  ";
 
@@ -405,7 +408,7 @@ int FTVHelp::decContentsDepth()
       parent->children.append(children->take(0));
     }
   }
-  return m_indent;
+  //return m_indent;
 }
 
 /*! Add a list item to the contents file.
@@ -416,10 +419,10 @@ int FTVHelp::decContentsDepth()
  *  \param name the name of the item.
  */
 void FTVHelp::addContentsItem(bool isDir,
+                              const char *name,
                               const char *ref,
                               const char *file,
-                              const char *anchor, 
-                              const char *name
+                              const char *anchor
                              )
 {
   QList<FTVNode> *nl = &m_indentNodes[m_indent];
@@ -594,10 +597,9 @@ void FTVHelp::generateTreeView()
 #if QT_VERSION >= 200
     t.setEncoding(QTextStream::Latin1);
 #endif
-    t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n";
+    t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Frameset//EN\">\n";
     t << "<html><head>";
-    t << "<meta http-equiv=\"Content-Type\" content=\"text/html;charset="
-      << theTranslator->idLanguageCharset() << "\">\n";
+    t << "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=UTF-8\">\n";
     t << "<title>"; 
     if (Config_getString("PROJECT_NAME").isEmpty())
     {
@@ -611,6 +613,9 @@ void FTVHelp::generateTreeView()
     t << "<frameset cols=\"" << Config_getInt("TREEVIEW_WIDTH") << ",*\">" << endl;
     t << "  <frame src=\"tree" << Doxygen::htmlFileExtension << "\" name=\"treefrm\">" << endl;
     t << "  <frame src=\"main" << Doxygen::htmlFileExtension << "\" name=\"basefrm\">" << endl;
+    t << "  <noframes>" << endl;
+    t << "    <a href=\"main" << Doxygen::htmlFileExtension << "\">Frames are disabled. Click here to go to the main page.</a>" << endl;
+    t << "  </noframes>" << endl;
     t << "</frameset>" << endl;
     t << "</html>" << endl;
     f.close();
@@ -632,8 +637,7 @@ void FTVHelp::generateTreeView()
 #endif
     t << "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
     t << "  <head>\n";
-    t << "    <meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset="
-      << theTranslator->idLanguageCharset() << "\" />\n";
+    t << "    <meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset=UTF-8\" />\n";
     t << "    <meta http-equiv=\"Content-Style-Type\" content=\"text/css\" />\n";
     t << "    <meta http-equiv=\"Content-Language\" content=\"en\" />\n";
     t << "    <link rel=\"stylesheet\" href=\"";
@@ -653,15 +657,6 @@ void FTVHelp::generateTreeView()
     }
     t << "\">" << endl;
     t << "    <title>TreeView</title>\n";
-    t << "    <style type=\"text/css\">\n";
-    t << "    <!--\n";
-    t << "    .directory { font-size: 10pt; font-weight: bold; }\n";
-    t << "    .directory h3 { margin: 0px; margin-top: 1em; font-size: 11pt; }\n";
-    t << "    .directory p { margin: 0px; white-space: nowrap; }\n";
-    t << "    .directory div { display: none; margin: 0px; }\n";
-    t << "    .directory img { vertical-align: middle; }\n";
-    t << "    -->\n";
-    t << "    </style>\n";
     t << "    <script type=\"text/javascript\">\n";
     t << "    <!-- // Hide script from old browsers\n";
     t << "    \n";
@@ -722,7 +717,7 @@ void FTVHelp::generateTreeView()
     t << "    </script>\n";
     t << "  </head>\n";
     t << "\n";
-    t << "  <body>\n";
+    t << "  <body class=\"ftvtree\">\n";
     t << "    <div class=\"directory\">\n";
     t << "      <h3>";
     QCString &projName = Config_getString("PROJECT_NAME");
