@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,11 +22,12 @@
 #include "language.h"
 #include "doxygen.h"
 #include "outputgen.h"
-#include "code.h"
 #include "dot.h"
 #include "message.h"
 #include "config.h"
 #include "htmlgen.h"
+#include "parserintf.h"
+#include "msc.h"
 
 
 static const int NUM_HTML_LIST_TYPES = 4;
@@ -49,8 +50,10 @@ static QString htmlAttribsToString(const HtmlAttribList &attribs)
 
 //-------------------------------------------------------------------------
 
-HtmlDocVisitor::HtmlDocVisitor(QTextStream &t,BaseCodeDocInterface &ci) 
-  : DocVisitor(DocVisitor_Html), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
+HtmlDocVisitor::HtmlDocVisitor(QTextStream &t,CodeOutputInterface &ci,
+                               const char *langExt) 
+  : DocVisitor(DocVisitor_Html), m_t(t), m_ci(ci), m_insidePre(FALSE), 
+                                 m_hide(FALSE), m_langExt(langExt)
 {
 }
 
@@ -67,7 +70,7 @@ void HtmlDocVisitor::visit(DocWord *w)
 void HtmlDocVisitor::visit(DocLinkedWord *w)
 {
   if (m_hide) return;
-  startLink(w->ref(),w->file(),w->relPath(),w->anchor());
+  startLink(w->ref(),w->file(),w->relPath(),w->anchor(),w->tooltip());
   filter(w->word());
   endLink();
 }
@@ -99,10 +102,16 @@ void HtmlDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Hash:    m_t << "#"; break;
     case DocSymbol::Percent: m_t << "%"; break;
     case DocSymbol::Copy:    m_t << "&copy;"; break;
-    case DocSymbol::Tm:      m_t << "&tm;"; break;
+    case DocSymbol::Tm:      m_t << "&trade;"; break;
     case DocSymbol::Reg:     m_t << "&reg;"; break;
     case DocSymbol::Apos:    m_t << "'"; break;
     case DocSymbol::Quot:    m_t << "\""; break;
+    case DocSymbol::Lsquo:   m_t << "&lsquo;"; break;
+    case DocSymbol::Rsquo:   m_t << "&rsquo;"; break;
+    case DocSymbol::Ldquo:   m_t << "&ldquo;"; break;
+    case DocSymbol::Rdquo:   m_t << "&rdquo;"; break;
+    case DocSymbol::Ndash:   m_t << "&ndash;"; break;
+    case DocSymbol::Mdash:   m_t << "&mdash;"; break;
     case DocSymbol::Uml:     m_t << "&" << s->letter() << "uml;"; break;
     case DocSymbol::Acute:   m_t << "&" << s->letter() << "acute;"; break;
     case DocSymbol::Grave:   m_t << "&" << s->letter() << "grave;"; break;
@@ -196,7 +205,9 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
   {
     case DocVerbatim::Code: // fall though
       m_t << PREFRAG_START;
-      parseCode(m_ci,s->context(),s->text().latin1(),s->isExample(),s->exampleFile());
+      Doxygen::parserManager->getParser(m_langExt)
+                            ->parseCode(m_ci,s->context(),s->text().latin1(),
+                                        s->isExample(),s->exampleFile());
       m_t << PREFRAG_END;
       break;
     case DocVerbatim::Verbatim: 
@@ -218,9 +229,10 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
         static int dotindex = 1;
         QCString fileName(4096);
 
-        fileName.sprintf("%s%d", 
+        fileName.sprintf("%s%d%s", 
             (Config_getString("HTML_OUTPUT")+"/inline_dotgraph_").data(), 
-            dotindex++
+            dotindex++,
+            ".dot"
            );
         QFile file(fileName);
         if (!file.open(IO_WriteOnly))
@@ -231,10 +243,37 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
         file.close();
 
         m_t << "<div align=\"center\">" << endl;
-        writeDotFile(fileName,s->relPath());
+        writeDotFile(fileName,s->relPath(),s->context());
         m_t << "</div>" << endl;
 
-        file.remove();
+        if (Config_getBool("DOT_CLEANUP")) file.remove();
+      }
+      break;
+    case DocVerbatim::Msc:
+      {
+        static int mscindex = 1;
+        QCString baseName(4096);
+
+        baseName.sprintf("%s%d", 
+            (Config_getString("HTML_OUTPUT")+"/inline_mscgraph_").data(), 
+            mscindex++
+           );
+        QFile file(baseName+".msc");
+        if (!file.open(IO_WriteOnly))
+        {
+          err("Could not open file %s.msc for writing\n",baseName.data());
+        }
+        QCString text = "msc {";
+        text+=s->text();
+        text+="}";
+        file.writeBlock( text, text.length() );
+        file.close();
+
+        m_t << "<div align=\"center\">" << endl;
+        writeMscFile(baseName,s->relPath(),s->context());
+        m_t << "</div>" << endl;
+
+        if (Config_getBool("DOT_CLEANUP")) file.remove();
       }
       break;
   }
@@ -253,7 +292,17 @@ void HtmlDocVisitor::visit(DocInclude *inc)
   {
     case DocInclude::Include: 
       m_t << PREFRAG_START;
-      parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile());
+      Doxygen::parserManager->getParser(inc->extension())
+                            ->parseCode(m_ci,                 
+                                        inc->context(),
+                                        inc->text().latin1(),
+                                        inc->isExample(),
+                                        inc->exampleFile(),
+                                        0,   // fd
+                                        -1,  // startLine
+                                        -1,  // endLine
+                                        TRUE // inlineFragment
+                                       );
       m_t << PREFRAG_END;
       break;
     case DocInclude::IncWithLines:
@@ -261,7 +310,12 @@ void HtmlDocVisitor::visit(DocInclude *inc)
          m_t << PREFRAG_START;
          QFileInfo cfi( inc->file() );
          FileDef fd( cfi.dirPath(), cfi.fileName() );
-         parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile(), &fd);
+         Doxygen::parserManager->getParser(inc->extension())
+                               ->parseCode(m_ci,
+                                           inc->context(),
+                                           inc->text().latin1(),
+                                           inc->isExample(),
+                                           inc->exampleFile(), &fd);
          m_t << PREFRAG_END;
       }
       break;
@@ -291,7 +345,13 @@ void HtmlDocVisitor::visit(DocIncOperator *op)
   if (op->type()!=DocIncOperator::Skip) 
   {
     popEnabled();
-    if (!m_hide) parseCode(m_ci,op->context(),op->text().latin1(),op->isExample(),op->exampleFile());
+    if (!m_hide) 
+    {
+      Doxygen::parserManager->getParser(m_langExt)
+                            ->parseCode(m_ci,op->context(),
+                                op->text().latin1(),op->isExample(),
+                                op->exampleFile());
+    }
     pushEnabled();
     m_hide=TRUE;
   }
@@ -422,7 +482,7 @@ void HtmlDocVisitor::visitPost(DocRoot *)
 void HtmlDocVisitor::visitPre(DocSimpleSect *s)
 {
   if (m_hide) return;
-  m_t << "<dl compact><dt><b>";
+  m_t << "<dl class=\"" << s->typeString() << "\" compact><dt><b>";
   switch(s->type())
   {
     case DocSimpleSect::See: 
@@ -526,18 +586,26 @@ void HtmlDocVisitor::visitPre(DocHtmlList *s)
 {
   if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
+  {
     m_t << "<ol" << htmlAttribsToString(s->attribs()) << ">\n"; 
+  }
   else 
+  {
     m_t << "<ul" << htmlAttribsToString(s->attribs()) << ">\n";
+  }
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlList *s) 
 {
   if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
+  {
     m_t << "</ol>"; 
-  else 
+  }
+  else
+  { 
     m_t << "</ul>";
+  }
   if (!s->isPreformatted()) m_t << "\n";
 }
 
@@ -722,7 +790,7 @@ void HtmlDocVisitor::visitPre(DocImage *img)
     }
     m_t << "<div align=\"center\">" << endl;
     m_t << "<img src=\"" << img->relPath() << img->name() << "\" alt=\"" 
-      << baseName << "\"" << htmlAttribsToString(img->attribs()) << ">" << endl;
+      << baseName << "\"" << ">" << endl;
     if (img->hasCaption())
     {
       m_t << "<p><strong>";
@@ -755,7 +823,7 @@ void HtmlDocVisitor::visitPost(DocImage *img)
 void HtmlDocVisitor::visitPre(DocDotFile *df)
 {
   if (m_hide) return;
-  writeDotFile(df->file(),df->relPath());
+  writeDotFile(df->file(),df->relPath(),df->context());
   m_t << "<div align=\"center\">" << endl;
   if (df->hasCaption())
   { 
@@ -903,13 +971,22 @@ void HtmlDocVisitor::visitPre(DocParamList *pl)
     m_t << "]</tt>&nbsp;";
   }
   m_t << "</td><td valign=\"top\"><em>";
-  QStrListIterator li(pl->parameters());
-  const char *s;
+  //QStrListIterator li(pl->parameters());
+  //const char *s;
+  QListIterator<DocNode> li(pl->parameters());
+  DocNode *param;
   bool first=TRUE;
-  for (li.toFirst();(s=li.current());++li)
+  for (li.toFirst();(param=li.current());++li)
   {
     if (!first) m_t << ","; else first=FALSE;
-    filter(s);
+    if (param->kind()==DocNode::Kind_Word)
+    {
+      visit((DocWord*)param); 
+    }
+    else if (param->kind()==DocNode::Kind_LinkedWord)
+    {
+      visit((DocLinkedWord*)param); 
+    }
   }
   m_t << "</em>&nbsp;</td><td>";
 }
@@ -1031,7 +1108,8 @@ void HtmlDocVisitor::filterQuotedCdataAttr(const char* str)
 }
 
 void HtmlDocVisitor::startLink(const QString &ref,const QString &file,
-                               const QString &relPath,const QString &anchor)
+                               const QString &relPath,const QString &anchor,
+                               const QString &tooltip)
 {
   QCString *dest;
   if (!ref.isEmpty()) // link to entity imported via tag file
@@ -1048,7 +1126,7 @@ void HtmlDocVisitor::startLink(const QString &ref,const QString &file,
   m_t << "href=\"";
   if (!ref.isEmpty())
   {
-    if ((dest=Doxygen::tagDestinationDict[ref])) m_t << relPath << *dest << "/";
+    if ((dest=Doxygen::tagDestinationDict[ref])) m_t << *dest << "/";
   }
   else
   {
@@ -1056,7 +1134,9 @@ void HtmlDocVisitor::startLink(const QString &ref,const QString &file,
   }
   if (!file.isEmpty()) m_t << file << Doxygen::htmlFileExtension;
   if (!anchor.isEmpty()) m_t << "#" << anchor;
-  m_t << "\">";
+  m_t << "\"";
+  if (!tooltip.isEmpty()) m_t << " title=\"" << tooltip << "\"";
+  m_t << ">";
 }
 
 void HtmlDocVisitor::endLink()
@@ -1077,7 +1157,8 @@ void HtmlDocVisitor::popEnabled()
   delete v;
 }
 
-void HtmlDocVisitor::writeDotFile(const QString &fileName,const QString &relPath)
+void HtmlDocVisitor::writeDotFile(const QString &fileName,const QString &relPath,
+                                  const QString &context)
 {
   QString baseName=fileName;
   int i;
@@ -1092,7 +1173,26 @@ void HtmlDocVisitor::writeDotFile(const QString &fileName,const QString &relPath
   m_t << "<img src=\"" << relPath << baseName << "." 
     << Config_getEnum("DOT_IMAGE_FORMAT") << "\" alt=\""
     << baseName << "\" border=\"0\" usemap=\"#" << mapName << "\">" << endl;
-  QString imap = getDotImageMapFromFile(fileName,outDir,relPath.data());
+  QString imap = getDotImageMapFromFile(fileName,outDir,relPath.data(),context);
+  m_t << "<map name=\"" << mapName << "\">" << imap << "</map>" << endl;
+}
+
+void HtmlDocVisitor::writeMscFile(const QString &fileName,const QString &relPath,
+                                  const QString &context)
+{
+  QString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1)
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  } 
+  QString outDir = Config_getString("HTML_OUTPUT");
+  writeMscGraphFromFile(fileName,outDir,baseName,MSC_BITMAP);
+  QString mapName = baseName+".map";
+  QString mapFile = fileName+".map";
+  m_t << "<img src=\"" << relPath << baseName << ".png\" alt=\""
+    << baseName << "\" border=\"0\" usemap=\"#" << mapName << "\">" << endl;
+  QString imap = getMscImageMapFromFile(fileName,outDir,relPath.data(),context);
   m_t << "<map name=\"" << mapName << "\">" << imap << "</map>" << endl;
 }
 
