@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2004 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,16 +21,17 @@
 
 #include "docvisitor.h"
 #include <qstack.h>
+#include <qcstring.h>
 
 class QTextStream;
-class BaseCodeDocInterface;
+class CodeOutputInterface;
 class QString;
 
 /*! @brief Concrete visitor implementation for HTML output. */
 class HtmlDocVisitor : public DocVisitor
 {
   public:
-    HtmlDocVisitor(QTextStream &t,BaseCodeDocInterface &ci);
+    HtmlDocVisitor(QTextStream &t,CodeOutputInterface &ci,const char *langExt);
     
     //--------------------------------------
     // visitor functions for leaf nodes
@@ -77,8 +78,6 @@ class HtmlDocVisitor : public DocVisitor
     void visitPost(DocHtmlList *) ;
     void visitPre(DocHtmlListItem *);
     void visitPost(DocHtmlListItem *);
-    //void visitPre(DocHtmlPre *);
-    //void visitPost(DocHtmlPre *);
     void visitPre(DocHtmlDescList *);
     void visitPost(DocHtmlDescList *);
     void visitPre(DocHtmlDescTitle *);
@@ -111,8 +110,6 @@ class HtmlDocVisitor : public DocVisitor
     void visitPost(DocSecRefItem *);
     void visitPre(DocSecRefList *);
     void visitPost(DocSecRefList *);
-    void visitPre(DocLanguage *);
-    void visitPost(DocLanguage *);
     void visitPre(DocParamSect *);
     void visitPost(DocParamSect *);
     void visitPre(DocParamList *);
@@ -135,9 +132,11 @@ class HtmlDocVisitor : public DocVisitor
     void filter(const char *str);
     void filterQuotedCdataAttr(const char* str);
     void startLink(const QString &ref,const QString &file,
-                   const QString &anchor);
+                   const QString &relPath,const QString &anchor,
+                   const QString &tooltip = QString::null);
     void endLink();
-    void writeDotFile(const QString &fileName);
+    void writeDotFile(const QString &fileName,const QString &relPath,const QString &context);
+    void writeMscFile(const QString &fileName,const QString &relPath,const QString &context);
 
     void pushEnabled();
     void popEnabled();
@@ -147,10 +146,11 @@ class HtmlDocVisitor : public DocVisitor
     //--------------------------------------
 
     QTextStream &m_t;
-    BaseCodeDocInterface &m_ci;
+    CodeOutputInterface &m_ci;
     bool m_insidePre;
     bool m_hide;
     QStack<bool> m_enabled;
+    QCString m_langExt;
 };
 
 #endif
