@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.cpp,v 1.56 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -46,7 +46,7 @@ static const char defaultStyleSheet[] =
 #include "doxygen_css.h"
 ;
 
-const char search_script[]=
+static const char search_script[]=
 #include "search_php.h"
 ;
 
@@ -431,13 +431,13 @@ static const char tabs_css[] =
 "   background-position: 0% -150px;\n"
 "}\n"
 "\n"
-"DIV.tabs LI#current A\n"
+"DIV.tabs LI.current A\n"
 "{\n"
 "   background-position: 100% -150px;\n"
 "   border-width     : 0px;\n"
 "}\n"
 "\n"
-"DIV.tabs LI#current SPAN\n"
+"DIV.tabs LI.current SPAN\n"
 "{\n"
 "   background-position: 0% -150px;\n"
 "   padding-bottom   : 6px;\n"
@@ -488,7 +488,29 @@ static void writeTabData(const char *dir)
   }
 }
 
-//-----------------------------------------------------------------------
+//------------------------------------------------------------------------
+
+unsigned char open_gif[] = {
+  0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x09, 0x00, 0x09, 0x00, 0xf0, 0x00,
+  0x00, 0x8e, 0xaf, 0xc4, 0x00, 0x00, 0x00, 0x21, 0xf9, 0x04, 0x01, 0x00,
+  0x00, 0x01, 0x00, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x09, 0x00, 0x09, 0x00,
+  0x00, 0x02, 0x0d, 0x8c, 0x8f, 0xa9, 0xcb, 0xe0, 0xff, 0x02, 0x8c, 0x66,
+  0x26, 0x7a, 0x51, 0x01, 0x00, 0x3b
+};
+unsigned int open_gif_len = 54;
+
+unsigned char closed_gif[] = {
+  0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x09, 0x00, 0x09, 0x00, 0xf0, 0x00,
+  0x00, 0x8e, 0xaf, 0xc4, 0x00, 0x00, 0x00, 0x21, 0xf9, 0x04, 0x01, 0x00,
+  0x00, 0x01, 0x00, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x09, 0x00, 0x09, 0x00,
+  0x00, 0x02, 0x10, 0x8c, 0x03, 0xa7, 0x98, 0xcb, 0xad, 0x80, 0x84, 0x66,
+  0xca, 0x38, 0x57, 0xd6, 0xf4, 0xd0, 0x02, 0x00, 0x3b
+};
+unsigned int closed_gif_len = 57;
+
+
+
+//------------------------- Pictures for the Tabs ------------------------
 
 HtmlGenerator::HtmlGenerator() : OutputGenerator()
 {
@@ -546,8 +568,9 @@ static void writeDefaultHeaderFile(QTextStream &t, const char *title,
 
   t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
     "<html><head>" 
-    "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=" 
-         << theTranslator->idLanguageCharset() << "\">\n"
+    "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=UTF-8" 
+         //<< theTranslator->idLanguageCharset() 
+         << "\">\n"
     "<title>"; 
   t << convertToHtml(title);
   t << "</title>\n";
@@ -569,7 +592,7 @@ static void writeDefaultHeaderFile(QTextStream &t, const char *title,
   }
   
   t << "\" rel=\"stylesheet\" type=\"text/css\">\n";
-  t << "<link href=\"tabs.css\" rel=\"stylesheet\" type=\"text/css\">\n";
+  t << "<link href=\"" << relPathStr << "tabs.css\" rel=\"stylesheet\" type=\"text/css\">\n";
   t << "</head><body>\n";
 }
 
@@ -586,7 +609,7 @@ void HtmlGenerator::writeHeaderFile(QFile &file)
 void HtmlGenerator::writeFooterFile(QFile &file)
 {
   QTextStream t(&file);
-  t << "<hr size=\"1\"><address style=\"align: right;\"><small>\n";
+  t << "<hr size=\"1\"><address style=\"text-align: right;\"><small>\n";
   t << theTranslator->trGeneratedAt( "$datetime", "$projectname" );
   t << "&nbsp;<a href=\"http://www.doxygen.org/index.html\">"
     << "<img src=\"doxygen.png\" alt=\"doxygen\" " 
@@ -597,6 +620,55 @@ void HtmlGenerator::writeFooterFile(QFile &file)
     << "</html>\n";
 }
 
+static void generateDynamicSections(QTextStream &t,const QCString &relPath)
+{
+  if (Config_getBool("HTML_DYNAMIC_SECTIONS"))
+  { 
+    t << 
+      "<script type=\"text/javascript\">\n"
+      "<!--\n"
+      "function changeDisplayState (e){\n"
+      "  var num=this.id.replace(/[^[0-9]/g,'');\n"
+      "  var button=this.firstChild;\n"
+      "  var sectionDiv=document.getElementById('dynsection'+num);\n"
+      "  if (sectionDiv.style.display=='none'||sectionDiv.style.display==''){\n"
+      "    sectionDiv.style.display='block';\n"
+      "    button.src='" << relPath << "open.gif';\n"
+      "  }else{\n"
+      "    sectionDiv.style.display='none';\n"
+      "    button.src='" << relPath << "closed.gif';\n"
+      "  }\n"
+      "}\n"
+      "function initDynSections(){\n"
+      "  var divs=document.getElementsByTagName('div');\n"
+      "  var sectionCounter=1;\n"
+      "  for(var i=0;i<divs.length-1;i++){\n"
+      "    if(divs[i].className=='dynheader'&&divs[i+1].className=='dynsection'){\n"
+      "      var header=divs[i];\n"
+      "      var section=divs[i+1];\n"
+      "      var button=header.firstChild;\n"
+      "      if (button!='IMG'){\n"
+      "        divs[i].insertBefore(document.createTextNode(' '),divs[i].firstChild);\n"
+      "        button=document.createElement('img');\n"
+      "        divs[i].insertBefore(button,divs[i].firstChild);\n"
+      "      }\n"
+      "      header.style.cursor='pointer';\n"
+      "      header.onclick=changeDisplayState;\n"
+      "      header.id='dynheader'+sectionCounter;\n"
+      "      button.src='" << relPath << "closed.gif';\n"
+      "      section.id='dynsection'+sectionCounter;\n"
+      "      section.style.display='none';\n"
+      "      section.style.marginLeft='14px';\n"
+      "      sectionCounter++;\n"
+      "    }\n"
+      "  }\n"
+      "}\n"
+      "window.onload = initDynSections;\n"
+      "-->\n"
+      "</script>\n";
+  }
+}
+
 
 void HtmlGenerator::startFile(const char *name,const char *,
                               const char *title)
@@ -611,10 +683,11 @@ void HtmlGenerator::startFile(const char *name,const char *,
     fileName+=Doxygen::htmlFileExtension;
   }
   startPlainFile(fileName);
-  if (Config_getBool("GENERATE_HTMLHELP"))
-  {
-    HtmlHelp::getInstance()->addIndexFile(fileName);
-  }
+  Doxygen::indexList.addIndexFile(fileName);
+  //if (Config_getBool("GENERATE_HTMLHELP"))
+  //{
+  //  HtmlHelp::getInstance()->addIndexFile(fileName);
+  //}
   
   QCString dispTitle = title;
   QCString projName = Config_getString("PROJECT_NAME");
@@ -634,15 +707,15 @@ void HtmlGenerator::startFile(const char *name,const char *,
   }
   t << "<!-- " << theTranslator->trGeneratedBy() << " Doxygen " 
     << versionString << " -->" << endl;
+  generateDynamicSections(t,relPath);
 }
 
-
 static void writePageFooter(QTextStream &t,const QCString &lastTitle,
                             const QCString relPath)
 {
   if (g_footer.isEmpty())
   {
-    t << "<hr size=\"1\"><address style=\"align: right;\"><small>";
+    t << "<hr size=\"1\"><address style=\"text-align: right;\"><small>";
     t << theTranslator->trGeneratedAt(
         dateToString(TRUE),
         Config_getString("PROJECT_NAME")
@@ -750,15 +823,24 @@ void HtmlGenerator::newParagraph()
   t << endl << "<p>" << endl;
 }
 
+void HtmlGenerator::startParagraph()
+{
+  t << endl << "<p>";
+}
+
+void HtmlGenerator::endParagraph()
+{
+  t << "</p>" << endl;
+}
+
 void HtmlGenerator::writeString(const char *text)
 {
   t << text;
 }
 
-void HtmlGenerator::writeIndexItem(const char *ref,const char *f,
-                                   const char *name)
+void HtmlGenerator::startIndexItem(const char *ref,const char *f)
 {
-  //printf("HtmlGenerator::writeIndexItem(%s,%s,%s)\n",ref,f,name);
+  //printf("HtmlGenerator::startIndexItem(%s,%s,%s)\n",ref,f,name);
   QCString *dest;
   t << "<li>";
   if (ref || f)
@@ -779,13 +861,21 @@ void HtmlGenerator::writeIndexItem(const char *ref,const char *f,
     {
       if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
     }
+    else
+    {
+      t << relPath;
+    }
     if (f) t << f << Doxygen::htmlFileExtension << "\">";
   }
   else
   {
     t << "<b>";
   }
-  docify(name);
+}
+
+void HtmlGenerator::endIndexItem(const char *ref,const char *f)
+{
+  //printf("HtmlGenerator::endIndexItem(%s,%s,%s)\n",ref,f,name);
   if (ref || f)
   {
     t << "</a>" << endl;
@@ -838,9 +928,11 @@ void HtmlGenerator::writeObjectLink(const char *ref,const char *f,
 }
 
 void HtmlGenerator::writeCodeLink(const char *ref,const char *f,
-                                  const char *anchor, const char *name)
+                                  const char *anchor, const char *name,
+                                  const char *tooltip)
 {
   QCString *dest;
+  //printf("writeCodeLink(ref=%s,f=%s,anchor=%s,name=%s,tooltip=%s)\n",ref,f,anchor,name,tooltip);
   if (ref) 
   {
     t << "<a class=\"codeRef\" ";
@@ -863,7 +955,9 @@ void HtmlGenerator::writeCodeLink(const char *ref,const char *f,
   }
   if (f) t << f << Doxygen::htmlFileExtension;
   if (anchor) t << "#" << anchor;
-  t << "\">";
+  t << "\"";
+  if (tooltip) t << " title=\"" << convertToXML(tooltip) << "\"";
+  t << ">";
   docify(name);
   t << "</a>";
   col+=strlen(name);
@@ -1022,18 +1116,22 @@ void HtmlGenerator::writeChar(char c)
 
 void HtmlGenerator::startClassDiagram()
 {
-  t << "<p>";
+  //t << "<p>";
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endClassDiagram(ClassDiagram &d,
+void HtmlGenerator::endClassDiagram(const ClassDiagram &d,
                                 const char *fileName,const char *name)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   t << "\n<p><center><img src=\""
     << relPath << fileName << ".png\" usemap=\"#" << name << "_map\""
     << " border=\"0\" alt=\"\"></center>" << endl
     << "<map name=\"" << name << "_map\">" << endl;
 
   d.writeImage(t,dir,relPath,fileName);
+  t << "</div>" << endl;
 }
 
 
@@ -1235,37 +1333,60 @@ void HtmlGenerator::endIndexValue(const char *,bool)
   t << "</td></tr>" << endl;
 }
 
+void HtmlGenerator::startMemberDocList()
+{
+  DBG_HTML(t << "<!-- startMemberDocList -->" << endl;)
+  //t << "<table class=\"memlist\">" << endl;
+  //t << "  <tr>" << endl;
+  //t << "    <td>" << endl;
+}
+
+void HtmlGenerator::endMemberDocList()
+{
+  DBG_HTML(t << "<!-- endMemberDocList -->" << endl;)
+  //t << "    </td>" << endl;
+  //t << "  </tr>" << endl;
+  //t << "</table>" << endl;
+}
+
 void HtmlGenerator::startMemberDoc(const char *,const char *,const char *,const char *) 
 { 
   DBG_HTML(t << "<!-- startMemberDoc -->" << endl;)
-  t << "<p>" << endl;
-  t << "<table class=\"mdTable\" cellpadding=\"2\" cellspacing=\"0\">" << endl;
-  t << "  <tr>" << endl;
-  t << "    <td class=\"mdRow\">" << endl; 
-  t << "      <table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" << endl;
+  //t << "<p>" << endl;
+ 
+  t << "\n<div class=\"memitem\">" << endl;
+  //t << "<table>" << endl;
+  //t << "  <tr>" << endl;
+  //t << "    <td>" << endl;
+  //t << "      <table class=\"memproto\">" << endl;
+  t << "<div class=\"memproto\">" << endl;
+  
 }
 
 void HtmlGenerator::startMemberDocPrefixItem()
 {
   DBG_HTML(t << "<!-- startMemberDocPrefixItem -->" << endl;)
-  t << "        <tr>" << endl;
-  t << "          <td class=\"mdPrefix\" colspan=\"4\">" << endl; 
+  //t << "        <tr>" << endl;
+  //t << "          <td class=\"memtemplate\" colspan=\"5\">";
+  t << "<div class=\"memtemplate\">" << endl;
 }
 
 void HtmlGenerator::endMemberDocPrefixItem()
 {
   DBG_HTML(t << "<!-- endMemberDocPrefixItem -->" << endl;)
-  t << "</td>" << endl;
-  t << "        </tr>" << endl;
+  //t << "</td>" << endl;
+  //t << "        </tr>" << endl;
+  t << "</div>" << endl;
 }
 
-void HtmlGenerator::startMemberDocName(bool align)
+void HtmlGenerator::startMemberDocName(bool /*align*/)
 {
   DBG_HTML(t << "<!-- startMemberDocName -->" << endl;)
+
+  t << "      <table class=\"memname\">" << endl;
+    
   t << "        <tr>" << endl;
-  t << "          <td class=\"md\" nowrap valign=\"top\"";
-  if (align) t << " align=\"right\"";
-  t << ">";
+  t << "          <td class=\"memname\">";
 }
 
 void HtmlGenerator::endMemberDocName()
@@ -1277,11 +1398,8 @@ void HtmlGenerator::endMemberDocName()
 void HtmlGenerator::startParameterList(bool openBracket)
 {
   DBG_HTML(t << "<!-- startParameterList -->" << endl;)
-  t << "          <td class=\"md\" valign=\"top\">";
-  if (openBracket)
-  {
-    t << "(&nbsp;";
-  }
+  t << "          <td>";
+  if (openBracket) t << "(";
   t << "</td>" << endl;
 }
 
@@ -1290,17 +1408,17 @@ void HtmlGenerator::startParameterType(bool first,const char *key)
   if (first)
   {
     DBG_HTML(t << "<!-- startFirstParameterType -->" << endl;)
-    t << "          <td class=\"md\" nowrap valign=\"top\">";
+    t << "          <td class=\"paramtype\">";
   }
   else
   {
     DBG_HTML(t << "<!-- startParameterType -->" << endl;)
     t << "        <tr>" << endl;
-    t << "          <td class=\"md\" nowrap align=\"right\">";
+    t << "          <td class=\"paramkey\">";
     if (key) t << key;
     t << "</td>" << endl;
-    t << "          <td class=\"md\"></td>" << endl;
-    t << "          <td class=\"md\" nowrap>";
+    t << "          <td></td>" << endl;
+    t << "          <td class=\"paramtype\">";
   }
 }
 
@@ -1310,15 +1428,10 @@ void HtmlGenerator::endParameterType()
   t << "&nbsp;</td>" << endl;
 }
 
-void HtmlGenerator::startParameterName(bool oneArgOnly)
+void HtmlGenerator::startParameterName(bool /*oneArgOnly*/)
 {
   DBG_HTML(t << "<!-- startParameterName -->" << endl;)
-  t << "          <td class=\"mdname";
-  if (oneArgOnly) 
-  {
-    t << "1\" valign=\"top";
-  }
-  t << "\" nowrap>"; //&nbsp;";
+  t << "          <td class=\"paramname\">";
 }
 
 void HtmlGenerator::endParameterName(bool last,bool emptyList,bool closeBracket)
@@ -1329,21 +1442,21 @@ void HtmlGenerator::endParameterName(bool last,bool emptyList,bool closeBracket)
     if (emptyList)
     {
       t << "          </td>" << endl;
-      t << "          <td class=\"md\" valign=\"top\">";
+      t << "          <td>";
       if (closeBracket) t << "&nbsp;)";
       t << "&nbsp;</td>" << endl;
-      t << "          <td class=\"md\" nowrap>";
+      t << "          <td width=\"100%\">";
     }
     else
     {
-      t << "</td>" << endl;
+      t << "</td><td>&nbsp;</td>" << endl;
       t << "        </tr>" << endl;
       t << "        <tr>" << endl;
-      t << "          <td class=\"md\"></td>" << endl;
-      t << "          <td class=\"md\">";
+      t << "          <td></td>" << endl;
+      t << "          <td>";
       if (closeBracket) t << ")";
-      t << "&nbsp;</td>" << endl;
-      t << "          <td class=\"md\" colspan=\"2\">";
+      t << "</td>" << endl;
+      t << "          <td></td><td></td><td width=\"100%\">";
     }
   }
   else
@@ -1368,59 +1481,86 @@ void HtmlGenerator::endMemberDoc(bool hasArgs)
     t << "        </tr>" << endl;
   }
   t << "      </table>" << endl;
-  t << "    </td>" << endl;
-  t << "  </tr>" << endl;
-  t << "</table>" << endl; 
+  //t << "    </td>" << endl;
+  //t << "  </tr>" << endl;
+  //t << "</table>" << endl;
+  t << "</div>" << endl;
 }
 
 void HtmlGenerator::startDotGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endDotGraph(DotClassGraph &g)
+void HtmlGenerator::endDotGraph(const DotClassGraph &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  if (Config_getBool("GENERATE_LEGEND"))
+  {
+    t << "<center><font size=\"2\">[";
+    startHtmlLink(relPath+"graph_legend"+Doxygen::htmlFileExtension);
+    t << theTranslator->trLegend();
+    endHtmlLink();
+    t << "]</font></center>";
+  }
+  t << "</div>" << endl;
 }
 
 void HtmlGenerator::startInclDepGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endInclDepGraph(DotInclDepGraph &g)
+void HtmlGenerator::endInclDepGraph(const DotInclDepGraph &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
-
 void HtmlGenerator::startGroupCollaboration()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endGroupCollaboration(DotGroupCollaboration &g)
+void HtmlGenerator::endGroupCollaboration(const DotGroupCollaboration &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
-
 void HtmlGenerator::startCallGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endCallGraph(DotCallGraph &g)
+void HtmlGenerator::endCallGraph(const DotCallGraph &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
 void HtmlGenerator::startDirDepGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endDirDepGraph(DotDirDeps &g)
+void HtmlGenerator::endDirDepGraph(const DotDirDeps &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
-void HtmlGenerator::writeGraphicalHierarchy(DotGfxHierarchyTable &g)
+void HtmlGenerator::writeGraphicalHierarchy(const DotGfxHierarchyTable &g)
 {
   g.writeGraph(t,dir);
 }
@@ -1455,27 +1595,15 @@ void HtmlGenerator::endMemberGroup(bool)
 
 void HtmlGenerator::startIndent()        
 { 
-  // I really wanted to use CSS here to provide an indented section, but
-  // alas, Netscape is buggy enough to sometimes "forget" to end the
-  // indent cause a staircase effect where the indent continuously increases.
-  // It's back to abusing tables :-(
-  
-  //t << "<div class=\"in\">" << endl; 
+  DBG_HTML(t << "<!-- startIndent -->" << endl;)
 
-  t << "<table cellspacing=\"5\" cellpadding=\"0\" border=\"0\">\n"
-       "  <tr>\n"
-       "    <td>\n"
-       "      &nbsp;\n"
-       "    </td>\n"
-       "    <td>\n";
+  t << "<div class=\"memdoc\">\n";
 }
 
 void HtmlGenerator::endIndent()          
 { 
-  t << "    </td>\n"
-       "  </tr>\n"
-       "</table>\n";
-  //t << "</div>" << endl; 
+  DBG_HTML(t << "<!-- endIndent -->" << endl;)
+  t << endl << "</div>" << endl << "</div><p>" << endl; 
 }
 
 void HtmlGenerator::addIndexItem(const char *,const char *)
@@ -1501,7 +1629,7 @@ void HtmlGenerator::writeLineNumber(const char *ref,const char *file,
   if (file)
   {
     startCodeAnchor(lineAnchor);
-    writeCodeLink(ref,file,anchor,lineNumber);
+    writeCodeLink(ref,file,anchor,lineNumber,0);
     endCodeAnchor();
   }
   else
@@ -1554,20 +1682,14 @@ void HtmlGenerator::printDoc(DocNode *n,const char *langExt)
   delete visitor; 
 }
 
+//---------------- helpers for index generation -----------------------------
+
 static void startQuickIndexList(QTextStream &t,bool compact)
 {
-  bool fancyTabs=TRUE;
   if (compact) 
   {
-    if (fancyTabs)
-    {
-      t << "<div class=\"tabs\">\n";
-      t << "  <ul>\n"; 
-    }
-    else
-    {
-      t << "<div class=\"qindex\">"; 
-    }
+    t << "<div class=\"tabs\">\n";
+    t << "  <ul>\n"; 
   }
   else 
   {
@@ -1577,13 +1699,9 @@ static void startQuickIndexList(QTextStream &t,bool compact)
 
 static void endQuickIndexList(QTextStream &t,bool compact)
 {
-  bool fancyTabs=TRUE;
   if (compact) 
   {
-    if (fancyTabs)
-    {
-      t << "  </ul>";
-    }
+    t << "  </ul>\n";
     t << "</div>\n";
   }
   else 
@@ -1593,42 +1711,20 @@ static void endQuickIndexList(QTextStream &t,bool compact)
 }
 
 static void startQuickIndexItem(QTextStream &t,const char *l,
-                                bool hl,bool compact,bool &first,
+                                bool hl,bool /*compact*/,
                                 const QCString &relPath)
 {
-  bool fancyTabs=TRUE;
-  if (!first && compact && !fancyTabs) t << " | ";
-  first=FALSE;
-  if (fancyTabs)
-  {
-    t << "    <li"; if (hl) t << " id=\"current\"";
-    t << "><a ";
-  }
-  else
-  {
-    if (!compact) t << "<li>";
-    if (hl && compact)
-    {
-      t << "<a class=\"qindexHL\" ";
-    }
-    else
-    {
-      t << "<a class=\"qindex\" ";
-    }
-  }
+  t << "    <li"; if (hl) t << " class=\"current\"";
+  t << "><a ";
   t << "href=\"" << relPath << l << "\">";
-  if (fancyTabs)
-  {
-    t << "<span>";
-  }
+  t << "<span>";
 }
 
 static void endQuickIndexItem(QTextStream &t)
 {
-  bool fancyTabs=TRUE;
-  if (fancyTabs) t << "</span>";
+  t << "</span>";
   t << "</a>";
-  if (fancyTabs) t << "</li>\n";
+  t << "</li>\n";
 }
 
 static QCString fixSpaces(const QCString &s)
@@ -1636,28 +1732,113 @@ static QCString fixSpaces(const QCString &s)
   return substitute(s," ","&nbsp;");
 }
 
+static void writeNamespaceSubIndex(QTextStream &t,bool compact,
+                                  HighlightedItem hli,const QCString &relPath
+                                 )
+{
+  startQuickIndexList(t,compact);
+  if (documentedNamespaces>0)
+  {
+    startQuickIndexItem(t,"namespaces"+Doxygen::htmlFileExtension,
+        hli==HLI_Namespaces,compact,relPath);
+    if (Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
+    {
+      t << fixSpaces(theTranslator->trPackages());
+    }
+    else
+    {
+      t << theTranslator->trNamespaceList();
+    }
+    endQuickIndexItem(t);
+  }
+  if (documentedNamespaceMembers[NMHL_All]>0)
+  {
+    startQuickIndexItem(t,"namespacemembers"+Doxygen::htmlFileExtension,
+        hli==HLI_NamespaceMembers,compact,relPath);
+    if (Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
+    {
+      t << fixSpaces(theTranslator->trPackageMembers());
+    }
+    else
+    {
+      t << fixSpaces(theTranslator->trNamespaceMembers());
+    }
+    endQuickIndexItem(t);
+  }
+  endQuickIndexList(t,compact);
+}
+
+static void writeClassSubIndex(QTextStream &t,bool compact,
+                               HighlightedItem hli,const QCString &relPath
+                              )
+{
+  startQuickIndexList(t,compact);
+  if (annotatedClasses>0)
+  {
+    if (Config_getBool("ALPHABETICAL_INDEX"))
+    {
+      startQuickIndexItem(t,"classes"+Doxygen::htmlFileExtension,
+          hli==HLI_Classes,compact,relPath);
+      t << fixSpaces(theTranslator->trAlphabeticalList());
+      endQuickIndexItem(t);
+    }
+    startQuickIndexItem(t,"annotated"+Doxygen::htmlFileExtension,
+        hli==HLI_Annotated,compact,relPath);
+    t << fixSpaces(theTranslator->trCompoundList());
+    endQuickIndexItem(t);
+  } 
+  if (hierarchyClasses>0)
+  {
+    startQuickIndexItem(t,"hierarchy"+Doxygen::htmlFileExtension,
+        hli==HLI_Hierarchy,compact,relPath);
+    t << fixSpaces(theTranslator->trClassHierarchy());
+    endQuickIndexItem(t);
+  } 
+  if (documentedClassMembers[CMHL_All]>0)
+  {
+    startQuickIndexItem(t,"functions"+Doxygen::htmlFileExtension,
+        hli==HLI_Functions,compact,relPath);
+    t << fixSpaces(theTranslator->trCompoundMembers());
+    endQuickIndexItem(t);
+  } 
+  endQuickIndexList(t,compact);
+}
+
+static void writeFileSubIndex(QTextStream &t,bool compact,
+                              HighlightedItem hli,const QCString &relPath)
+{
+  startQuickIndexList(t,compact);
+  if (documentedHtmlFiles>0)
+  {
+    startQuickIndexItem(t,"files"+Doxygen::htmlFileExtension,
+        hli==HLI_Files,compact,relPath);
+    t << fixSpaces(theTranslator->trFileList());
+    endQuickIndexItem(t);
+  } 
+  if (documentedFileMembers[FMHL_All]>0)
+  {
+    startQuickIndexItem(t,"globals"+Doxygen::htmlFileExtension,
+        hli==HLI_Globals,compact,relPath);
+    t << fixSpaces(theTranslator->trFileMembers());
+    endQuickIndexItem(t);
+  } 
+  endQuickIndexList(t,compact);
+}
+
 static void writeDefaultQuickLinks(QTextStream &t,bool compact,
                                    HighlightedItem hli,const QCString &relPath)
 {
-  bool first=TRUE;
-  bool fancyTabs=TRUE;
   startQuickIndexList(t,compact);
 
-  if (Config_getBool("SEARCHENGINE") && !fancyTabs)
-  {
-    t << "  <form class=\"search\" action=\"" << relPath 
-      << "search.php\" method=\"get\">\n";
-  }
-
   if (Config_getBool("GENERATE_TREEVIEW"))
   {
     startQuickIndexItem(t,"main"+Doxygen::htmlFileExtension,
-                        hli==HLI_Main,compact,first,relPath);
+                        hli==HLI_Main,compact,relPath);
   }
   else
   {
     startQuickIndexItem(t,"index"+Doxygen::htmlFileExtension,
-                        hli==HLI_Main,compact,first,relPath);
+                        hli==HLI_Main,compact,relPath);
   }
   t << fixSpaces(theTranslator->trMainPage());
   endQuickIndexItem(t);
@@ -1665,7 +1846,7 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
   if (documentedGroups>0)
   {
     startQuickIndexItem(t,"modules"+Doxygen::htmlFileExtension,
-                        hli==HLI_Modules,compact,first,relPath);
+                        hli==HLI_Modules,compact,relPath);
     t << fixSpaces(theTranslator->trModules());
     endQuickIndexItem(t);
   } 
@@ -1676,7 +1857,7 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
   {
     startQuickIndexItem(t,"namespaces"+Doxygen::htmlFileExtension,
         hli==HLI_Namespaces || hli==HLI_NamespaceMembers || hli==HLI_NamespaceVisible,
-        compact,first,relPath);
+        compact,relPath);
     if (Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
     {
       t << fixSpaces(theTranslator->trPackages());
@@ -1686,6 +1867,10 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
       t << theTranslator->trNamespaces();
     }
     endQuickIndexItem(t);
+    if (!compact)
+    {
+      writeNamespaceSubIndex(t,compact,hli,relPath);
+    }
   }
 
 
@@ -1693,12 +1878,19 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
 
   if (annotatedClasses>0)
   {
-    startQuickIndexItem(t,"annotated"+Doxygen::htmlFileExtension,
-        hli==HLI_Hierarchy || hli==HLI_Classes || 
-        hli==HLI_Annotated || hli==HLI_Functions || hli==HLI_ClassVisible,
-        compact,first,relPath);
+    static bool alphaIndex=Config_getBool("ALPHABETICAL_INDEX");
+    {
+      startQuickIndexItem(t,QCString(alphaIndex?"classes":"annotated")+Doxygen::htmlFileExtension,
+          hli==HLI_Hierarchy || hli==HLI_Classes || 
+          hli==HLI_Annotated || hli==HLI_Functions || hli==HLI_ClassVisible,
+          compact,relPath);
+    }
     t << fixSpaces(theTranslator->trClasses());
     endQuickIndexItem(t);
+    if (!compact)
+    {
+      writeClassSubIndex(t,compact,hli,relPath);
+    }
   }
 
 
@@ -1708,72 +1900,63 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
   {
     startQuickIndexItem(t,"files"+Doxygen::htmlFileExtension,
                         hli==HLI_Files || hli==HLI_Globals || hli==HLI_FileVisible,
-                        compact,first,relPath);
+                        compact,relPath);
     t << fixSpaces(theTranslator->trFile(TRUE,FALSE));
     endQuickIndexItem(t);
+    if (!compact)
+    {
+      writeFileSubIndex(t,compact,hli,relPath);
+    }
   } 
 
-  // ------------------------------------------------
+  // -------------- Directories
   
   if (Config_getBool("SHOW_DIRECTORIES") && documentedDirs>0)
   {
     startQuickIndexItem(t,"dirs"+Doxygen::htmlFileExtension,
-                        hli==HLI_Directories,compact,first,relPath);
+                        hli==HLI_Directories,compact,relPath);
     t << fixSpaces(theTranslator->trDirectories());
     endQuickIndexItem(t);
   } 
+
+  // -------------- Related pages
+  
   if (indexedPages>0)
   {
     startQuickIndexItem(t,"pages"+Doxygen::htmlFileExtension,
-                        hli==HLI_Pages,compact,first,relPath);
+                        hli==HLI_Pages,compact,relPath);
     t << fixSpaces(theTranslator->trRelatedPages());
     endQuickIndexItem(t);
   } 
+
+  // -------------- Examples
+  
   if (Doxygen::exampleSDict->count()>0)
   {
     startQuickIndexItem(t,"examples"+Doxygen::htmlFileExtension,
-                        hli==HLI_Examples,compact,first,relPath);
+                        hli==HLI_Examples,compact,relPath);
     t << fixSpaces(theTranslator->trExamples());
     endQuickIndexItem(t);
   } 
+
+  // -------------- Search field
+
   if (Config_getBool("SEARCHENGINE"))
   {
     QCString searchFor = fixSpaces(theTranslator->trSearchForIndex());
     if (searchFor.at(0)=='S') searchFor="<u>S</u>"+searchFor.mid(1);
-    if (fancyTabs)
+    t << "    <li>\n";
+    t << "      <form action=\"" << relPath << "search.php\" method=\"get\">\n";
+    t << "        <table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n";
+    t << "          <tr>\n";
+    t << "            <td><label>&nbsp;" << searchFor << "&nbsp;</label></td>\n";
+    if (hli!=HLI_Search)
     {
-      t << "    <li>\n";
-      t << "      <form action=\"search.php\" method=\"get\">\n";
-      t << "        <table cellspacing=\"0\" callpadding=\"0\" border=\"0\">\n";
-      t << "          <tr>\n";
-      t << "            <td><label>&nbsp;" << searchFor << "&nbsp;</label></td>\n";
-      if (hli!=HLI_Search)
-      {
-        t << "            <td><input type=\"text\" name=\"query\" value=\"\" size=\"20\" accesskey=\"s\"/></td>\n";
-        t << "          </tr>\n";
-        t << "        </table>\n";
-        t << "      </form>\n";
-        t << "    </li>\n";
-      }
-    }
-    else
-    {
-      if (compact)
-      {
-        t << "  | ";
-      }
-      else
-      {
-        t << "<li>";
-      }
-      t << "<span class=\"search";
-      if (hli==HLI_Search) t << "HL";
-      t << "\">" << searchFor << "&nbsp;";
-      if (hli!=HLI_Search)
-      {
-        t << "<input class=\"search\" type=\"text\" name=\"query\" value=\"\" size=\"20\" accesskey=\"s\"/>"
-          "</span></form>";
-      }
+      t << "            <td><input type=\"text\" name=\"query\" value=\"\" size=\"20\" accesskey=\"s\"/></td>\n";
+      t << "          </tr>\n";
+      t << "        </table>\n";
+      t << "      </form>\n";
+      t << "    </li>\n";
     }
   } 
   if (hli!=HLI_Search) // on the search page the page will be ended by the
@@ -1783,93 +1966,30 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
   }
 
   //-------------------------------------------------------------------------
-  // write sub indices
+  // write sub indices in compact mode
 
-  if ((hli==HLI_Namespaces || hli==HLI_NamespaceMembers || hli==HLI_NamespaceVisible) && 
+  if ((hli==HLI_Namespaces || hli==HLI_NamespaceMembers || 
+       hli==HLI_NamespaceVisible
+      ) && 
       documentedNamespaces>0 &&
       documentedNamespaceMembers[NMHL_All]>0)
   {
-    startQuickIndexList(t,compact);
-    if (documentedNamespaces>0)
-    {
-      startQuickIndexItem(t,"namespaces"+Doxygen::htmlFileExtension,
-          hli==HLI_Namespaces,compact,first,relPath);
-      if (Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
-      {
-        t << fixSpaces(theTranslator->trPackages());
-      }
-      else
-      {
-        t << theTranslator->trNamespaceList();
-      }
-      endQuickIndexItem(t);
-    }
-    if (documentedNamespaceMembers[NMHL_All]>0)
-    {
-      startQuickIndexItem(t,"namespacemembers"+Doxygen::htmlFileExtension,
-          hli==HLI_NamespaceMembers,compact,first,relPath);
-      t << fixSpaces(theTranslator->trNamespaceMembers());
-      endQuickIndexItem(t);
-    }
-    endQuickIndexList(t,compact);
+    writeNamespaceSubIndex(t,compact,hli,relPath);
   }
   else if ((hli==HLI_Hierarchy || hli==HLI_Classes || 
             hli==HLI_Annotated || hli==HLI_Functions || 
-            hli==HLI_ClassVisible
+            hli==HLI_ClassVisible || !compact
            ) && annotatedClasses>0
           )
   {
-    startQuickIndexList(t,compact);
-    if (annotatedClasses>0)
-    {
-      startQuickIndexItem(t,"annotated"+Doxygen::htmlFileExtension,
-          hli==HLI_Annotated,compact,first,relPath);
-      t << fixSpaces(theTranslator->trCompoundList());
-      endQuickIndexItem(t);
-      if (Config_getBool("ALPHABETICAL_INDEX"))
-      {
-        startQuickIndexItem(t,"classes"+Doxygen::htmlFileExtension,
-            hli==HLI_Classes,compact,first,relPath);
-        t << fixSpaces(theTranslator->trAlphabeticalList());
-        endQuickIndexItem(t);
-      }
-    } 
-    if (hierarchyClasses>0)
-    {
-      startQuickIndexItem(t,"hierarchy"+Doxygen::htmlFileExtension,
-          hli==HLI_Hierarchy,compact,first,relPath);
-      t << fixSpaces(theTranslator->trClassHierarchy());
-      endQuickIndexItem(t);
-    } 
-    if (documentedClassMembers[CMHL_All]>0)
-    {
-      startQuickIndexItem(t,"functions"+Doxygen::htmlFileExtension,
-          hli==HLI_Functions,compact,first,relPath);
-      t << fixSpaces(theTranslator->trCompoundMembers());
-      endQuickIndexItem(t);
-    } 
-    endQuickIndexList(t,compact);
+    writeClassSubIndex(t,compact,hli,relPath);
   }
-  else if ((hli==HLI_Files || hli==HLI_Globals || hli==HLI_FileVisible) &&
-           documentedHtmlFiles>0 && documentedFileMembers[FMHL_All]>0
+  else if ((hli==HLI_Files || hli==HLI_Globals || 
+            hli==HLI_FileVisible || !compact
+           ) && documentedHtmlFiles>0 && documentedFileMembers[FMHL_All]>0
           )
   {
-    startQuickIndexList(t,compact);
-    if (documentedHtmlFiles>0)
-    {
-      startQuickIndexItem(t,"files"+Doxygen::htmlFileExtension,
-          hli==HLI_Files,compact,first,relPath);
-      t << fixSpaces(theTranslator->trFileList());
-      endQuickIndexItem(t);
-    } 
-    if (documentedFileMembers[FMHL_All]>0)
-    {
-      startQuickIndexItem(t,"globals"+Doxygen::htmlFileExtension,
-          hli==HLI_Globals,compact,first,relPath);
-      t << fixSpaces(theTranslator->trFileMembers());
-      endQuickIndexItem(t);
-    } 
-    endQuickIndexList(t,compact);
+    writeFileSubIndex(t,compact,hli,relPath);
   }
   
 }
@@ -1959,3 +2079,68 @@ void HtmlGenerator::writeSearchPage()
   }
 }
 
+void HtmlGenerator::generateSectionImages()
+{
+  {
+    QCString fileName = Config_getString("HTML_OUTPUT")+"/open.gif";
+    QFile f(fileName);
+    if (f.open(IO_WriteOnly))
+    {
+      f.writeBlock((char*)open_gif,open_gif_len);
+    }
+  }
+  {
+    QCString fileName = Config_getString("HTML_OUTPUT")+"/closed.gif";
+    QFile f(fileName);
+    if (f.open(IO_WriteOnly))
+    {
+      f.writeBlock((char*)closed_gif,closed_gif_len);
+    }
+  }
+}
+
+void HtmlGenerator::startConstraintList(const char *header)
+{
+  t << "<div class=\"typeconstraint\">" << endl;
+  t << "<dl compact><dt><b>" << header << "</b><dt><dd>" << endl;
+  t << "<table border=\"0\" cellspacing=\"2\" cellpadding=\"0\">" << endl;
+}
+
+void HtmlGenerator::startConstraintParam()
+{
+  t << "<tr><td valign=\"top\"><em>";
+}
+
+void HtmlGenerator::endConstraintParam()
+{
+  t << "</em></td>";
+}
+
+void HtmlGenerator::startConstraintType()
+{
+  t << "<td>&nbsp;:</td><td valign=\"top\"><em>";
+}
+
+void HtmlGenerator::endConstraintType()
+{
+  t << "</em></td>";
+}
+
+void HtmlGenerator::startConstraintDocs()
+{
+  t << "<td>&nbsp;";
+}
+
+void HtmlGenerator::endConstraintDocs()
+{
+  t << "</td></tr>" << endl;
+}
+
+void HtmlGenerator::endConstraintList()
+{
+  t << "</table>" << endl;
+  t << "</dl>" << endl;
+  t << "</div>" << endl;
+}
+
+
