@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.h,v 1.51 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,7 +36,8 @@ class HtmlGenerator : public OutputGenerator
     static void writeHeaderFile(QFile &f);
     static void writeFooterFile(QFile &f);
    
-    void enable() { active=TRUE; }
+    void enable() 
+    { if (genStack->top()) active=*genStack->top(); else active=TRUE; }
     void disable() { active=FALSE; }
     void enableIf(OutputType o)  { if (o==Html) active=TRUE;  }
     void disableIf(OutputType o) { if (o==Html) active=FALSE; }
@@ -44,7 +45,7 @@ class HtmlGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==Html && active); } 
     OutputGenerator *get(OutputType o) { return (o==Html) ? this : 0; }
 
-    void printDoc(DocNode *);
+    void printDoc(DocNode *,const char *);
 
     void startFile(const char *name,const char *manName,const char *title);
     void writeFooter();
@@ -62,6 +63,8 @@ class HtmlGenerator : public OutputGenerator
     void endTitle() { t << "</h1>"; }
     
     void newParagraph();
+    void startParagraph();
+    void endParagraph();
     void writeString(const char *text);
     void startIndexList();
     void endIndexList();
@@ -71,13 +74,15 @@ class HtmlGenerator : public OutputGenerator
     void endIndexValue(const char *,bool);
     void startItemList()  { t << "<ul>"  << endl; }
     void endItemList()    { t << "</ul>" << endl; }
-    void writeIndexItem(const char *ref,const char *file,const char *name);
+    void startIndexItem(const char *ref,const char *file);
+    void endIndexItem(const char *ref,const char *file);
     void docify(const char *text);
     void codify(const char *text);
     void writeObjectLink(const char *ref,const char *file,
                          const char *anchor,const char *name);
     void writeCodeLink(const char *ref,const char *file,
-                       const char *anchor,const char *name);
+                       const char *anchor,const char *name,
+                       const char *tooltip);
     void startTextLink(const char *file,const char *anchor);
     void endTextLink();
     void startHtmlLink(const char *url);
@@ -94,6 +99,8 @@ class HtmlGenerator : public OutputGenerator
     void endMemberHeader();
     void startMemberSubtitle();
     void endMemberSubtitle();
+    void startMemberDocList();
+    void endMemberDocList();
     void startMemberList();
     void endMemberList();
     void startAnonTypeScope(int) {}
@@ -135,7 +142,8 @@ class HtmlGenerator : public OutputGenerator
     void startMemberDoc(const char *,const char *,const char *,const char *);
     void endMemberDoc(bool); 
     void startDoxyAnchor(const char *fName,const char *manName,
-                         const char *anchor,const char *name);
+                         const char *anchor,const char *name,
+                         const char *args);
     void endDoxyAnchor(const char *fName,const char *anchor);
     void startCodeAnchor(const char *label) { t << "<a name=\"" << label << "\"></a>"; }
     void endCodeAnchor() { }
@@ -165,11 +173,9 @@ class HtmlGenerator : public OutputGenerator
     void endIndent();
     void writeSynopsis() {}
     void startClassDiagram();
-    void endClassDiagram(ClassDiagram &,const char *,const char *);
+    void endClassDiagram(const ClassDiagram &,const char *,const char *);
     void startPageRef() {}
     void endPageRef(const char *,const char *) {}
-    //void startQuickIndexItem(const char *,const char *);
-    //void endQuickIndexItem();
     void writeQuickLinks(bool compact,HighlightedItem hli);
     void writeNonBreakableSpace(int);
     
@@ -187,16 +193,16 @@ class HtmlGenerator : public OutputGenerator
     { t << "</td></tr>" << endl; }
     
     void startDotGraph();
-    void endDotGraph(DotClassGraph &g);
+    void endDotGraph(const DotClassGraph &g);
     void startInclDepGraph();
-    void endInclDepGraph(DotInclDepGraph &g);
+    void endInclDepGraph(const DotInclDepGraph &g);
     void startGroupCollaboration();
-    void endGroupCollaboration(DotGroupCollaboration &g);
+    void endGroupCollaboration(const DotGroupCollaboration &g);
     void startCallGraph();
-    void endCallGraph(DotCallGraph &g);
+    void endCallGraph(const DotCallGraph &g);
     void startDirDepGraph();
-    void endDirDepGraph(DotDirDeps &g);
-    void writeGraphicalHierarchy(DotGfxHierarchyTable &g);
+    void endDirDepGraph(const DotDirDeps &g);
+    void writeGraphicalHierarchy(const DotGfxHierarchyTable &g);
 
     void startTextBlock(bool) {}
     void endTextBlock(bool) {}
@@ -213,12 +219,24 @@ class HtmlGenerator : public OutputGenerator
     void startParameterList(bool);
     void endParameterList();
 
+    void startConstraintList(const char *);
+    void startConstraintParam();
+    void endConstraintParam();
+    void startConstraintType();
+    void endConstraintType();
+    void startConstraintDocs();
+    void endConstraintDocs();
+    void endConstraintList();
+
     void startFontClass(const char *s) { t << "<span class=\"" << s << "\">"; }
     void endFontClass() { t << "</span>"; }
 
     void writeCodeAnchor(const char *anchor) 
     { t << "<a name=\"" << anchor << "\"></a>"; }
+    void linkableSymbol(int,const char *,Definition *,Definition *) {}
+
     static void writeSearchPage();
+    static void generateSectionImages();
 
   private:
     QCString lastTitle;
