@@ -2,7 +2,7 @@
  *
  * $Id: htmlhelp.cpp,v 1.8 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -37,6 +37,7 @@ struct IndexField
   QCString url;
   QCString anchor;
   bool     link;
+  bool     reversed;
 };
 
 class IndexFieldList : public QList<IndexField>
@@ -72,7 +73,8 @@ class HtmlHelpIndex
     HtmlHelpIndex();
    ~HtmlHelpIndex();
     void addItem(const char *first,const char *second, 
-                 const char *url, const char *anchor,bool hasLink);
+                 const char *url, const char *anchor,
+                 bool hasLink,bool reversed);
     void writeFields(QTextStream &t);
   private:
     IndexFieldList *list;
@@ -104,9 +106,12 @@ HtmlHelpIndex::~HtmlHelpIndex()
  *  \param anchor the anchor of the documentation within the page.
  *  \param hasLink if true, the url (without anchor) can be used in the 
  *         level1 item, when writing the header of a list of level2 items.
+ *  \param reversed TRUE if level1 is the member name and level2 the compound
+ *         name.
  */
 void HtmlHelpIndex::addItem(const char *level1,const char *level2,
-                       const char *url,const char *anchor,bool hasLink)
+                       const char *url,const char *anchor,bool hasLink,
+                       bool reversed)
 {
   QCString key = level1; 
   if (level2) key+= (QCString)"?" + level2;
@@ -115,10 +120,11 @@ void HtmlHelpIndex::addItem(const char *level1,const char *level2,
     //printf(">>>>>>>>> HtmlHelpIndex::addItem(%s,%s,%s,%s)\n",
     //      level1,level2,url,anchor);
     IndexField *f = new IndexField;
-    f->name   = key;
-    f->url    = url;
-    f->anchor = anchor;
-    f->link   = hasLink;
+    f->name     = key;
+    f->url      = url;
+    f->anchor   = anchor;
+    f->link     = hasLink;
+    f->reversed = reversed;
     list->inSort(f);
     dict->insert(key,f);
   }
@@ -199,7 +205,7 @@ void HtmlHelpIndex::writeFields(QTextStream &t)
       {
         t << "  <LI><OBJECT type=\"text/sitemap\">";
         t << "<param name=\"Local\" value=\"" << f->url << Doxygen::htmlFileExtension;
-        if (!f->anchor.isEmpty()) t << "#" << f->anchor;  
+        if (!f->anchor.isEmpty() && f->reversed) t << "#" << f->anchor;  
         t << "\">";
         t << "<param name=\"Name\" value=\"" << level1 << "\">"
            "</OBJECT>\n";
@@ -210,7 +216,7 @@ void HtmlHelpIndex::writeFields(QTextStream &t)
         {
           t << "  <LI><OBJECT type=\"text/sitemap\">";
           t << "<param name=\"Local\" value=\"" << f->url << Doxygen::htmlFileExtension;
-          if (!f->anchor.isEmpty()) t << "#" << f->anchor;  
+          if (!f->anchor.isEmpty() && f->reversed) t << "#" << f->anchor;  
           t << "\">";
           t << "<param name=\"Name\" value=\"" << level1 << "\">"
                "</OBJECT>\n";
@@ -245,6 +251,7 @@ void HtmlHelpIndex::writeFields(QTextStream &t)
     }
     lastLevel1 = level1.copy();
   } 
+  if (level2Started) t << "  </UL>" << endl;
 }
 
 //----------------------------------------------------------------------------
@@ -263,6 +270,7 @@ HtmlHelp::HtmlHelp() : indexFileDict(1009)
   index = new HtmlHelpIndex;
 }
 
+#if 0
 /*! return a reference to the one and only instance of this class. 
  */
 HtmlHelp *HtmlHelp::getInstance()
@@ -270,6 +278,7 @@ HtmlHelp *HtmlHelp::getInstance()
   if (theInstance==0) theInstance = new HtmlHelp;
   return theInstance;
 }
+#endif
 
 static QDict<QCString> s_languageDict;
 
@@ -439,6 +448,15 @@ void HtmlHelp::createProjectFile()
       t << s << endl;
       s = indexFiles.next();
     }
+    t << "tabs.css" << endl;
+    t << "tab_b.gif" << endl;
+    t << "tab_l.gif" << endl;
+    t << "tab_r.gif" << endl;
+    if (Config_getBool("HTML_DYNAMIC_SECTIONS"))
+    {
+      t << "open.gif" << endl;
+      t << "closed.gif" << endl;
+    }
     f.close();
   }
   else
@@ -464,6 +482,8 @@ void HtmlHelp::finalize()
 {
   // end the contents file
   cts << "</UL>\n";
+  cts << "</BODY>\n";
+  cts << "</HTML>\n";
   cts.unsetDevice();
   cf->close();
   delete cf;
@@ -472,6 +492,8 @@ void HtmlHelp::finalize()
   
   // end the index file
   kts << "</UL>\n";
+  kts << "</BODY>\n";
+  kts << "</HTML>\n";
   kts.unsetDevice();
   kf->close();
   delete kf;
@@ -484,47 +506,50 @@ void HtmlHelp::finalize()
  *  This will start a new unnumbered HTML list in contents file.
  *  \sa decContentsDepth()
  */
-int HtmlHelp::incContentsDepth()
+void HtmlHelp::incContentsDepth()
 {
   int i; for (i=0;i<dc+1;i++) cts << "  ";
   cts << "<UL>\n";
-  return ++dc;
+  ++dc;
 }
 
 /*! Decrease the level of the contents hierarchy.
  *  This will end the unnumber HTML list.
  *  \sa incContentsDepth()
  */
-int HtmlHelp::decContentsDepth()
+void HtmlHelp::decContentsDepth()
 {
   int i; for (i=0;i<dc;i++) cts << "  ";
   cts << "</UL>\n";
-  return --dc;
+  --dc;
 }
 
 /*! Add an list item to the contents file.
  *  \param isDir boolean indicating if this is a dir or file entry
  *  \param name the name of the item.
  *  \param ref  the URL of to the item.
+ *  \param file the file in which the item is defined.
  *  \param anchor the anchor of the item.
  */
 void HtmlHelp::addContentsItem(bool isDir,
-                               const char *name,const char *ref, 
+                               const char *name,
+                               const char * /*ref*/, 
+                               const char *file,
                                const char *anchor)
 {
   // If we're using a binary toc then folders cannot have links. 
   if(Config_getBool("BINARY_TOC") && isDir) 
   {
-    ref = 0;
+    file = 0;
     anchor = 0;
   }
   
   int i; for (i=0;i<dc;i++) cts << "  ";
   cts << "<LI><OBJECT type=\"text/sitemap\">";
   cts << "<param name=\"Name\" value=\"" << name << "\">";
-  if (ref)      // made ref optional param - KPW
+  if (file)      // made file optional param - KPW
   {
-    cts << "<param name=\"Local\" value=\"" << ref << Doxygen::htmlFileExtension;
+    cts << "<param name=\"Local\" value=\"" << file << Doxygen::htmlFileExtension;
     if (anchor) cts << "#" << anchor;  
     cts << "\">";
   }
@@ -544,14 +569,16 @@ void HtmlHelp::addContentsItem(bool isDir,
 /*! Add an list item to the index file.
  *  \param level1 the main index of the item.
  *  \param level2 the sub index of the item.
- *  \param ref  the URL of to the item.
+ *  \param contRef the output file refering to the container.
+ *  \param memRef  the output file containing to the member documentation.
  *  \param anchor  the anchor of the item.
  *  \sa HtmlHelpIndex
  */
 void HtmlHelp::addIndexItem(const char *level1, const char *level2,
-                            const char *ref, const char *anchor)
+                            const char *contRef, const char *memRef, 
+                            const char *anchor)
 {
-  index->addItem(level1,level2,ref,anchor,TRUE);
-  index->addItem(level2,level1,ref,anchor,TRUE);
+  index->addItem(level1,level2,contRef,anchor,TRUE,FALSE);
+  index->addItem(level2,level1,memRef,anchor,TRUE,TRUE);
 }
 
