@@ -2,7 +2,7 @@
  *
  * $Id: latexgen.cpp,v 1.58 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -53,47 +53,6 @@
 //}
 
 
-static QCString escapeLabelName(const char *s)
-{
-  QCString result;
-  const char *p=s;
-  char c;
-  while ((c=*p++))
-  {
-    switch (c)
-    {
-      case '%': result+="\\%"; break;
-      case '|': result+="\\texttt{\"|}"; break;
-      case '!': result+="\"!"; break;
-      default: result+=c;
-    }
-  }
-  return result;
-}
-
-static QCString escapeMakeIndexChars(LatexGenerator *g,QTextStream &t,const char *s)
-{
-  QCString result;
-  const char *p=s;
-  char str[2];
-  str[1]=0;
-  char c;
-  while ((c=*p++))
-  {
-    switch (c)
-    {
-      case '!': t << "\"!"; break;
-      case '"': t << "\"\""; break;
-      case '@': t << "\"@"; break;
-      case '|': t << "\\texttt{\"|}"; break;
-      case '[': t << "["; break;
-      case ']': t << "]"; break;
-      default:  str[0]=c; g->docify(str); break;
-    }
-  }
-  return result;
-}
-
 
 LatexGenerator::LatexGenerator() : OutputGenerator()
 {
@@ -102,7 +61,9 @@ LatexGenerator::LatexGenerator() : OutputGenerator()
   //printf("LatexGenerator::LatexGenerator() insideTabbing=FALSE\n");
   insideTabbing=FALSE;
   firstDescItem=TRUE;
+  disableLinks=FALSE;
   m_indent=0;
+  templateMemberItem = FALSE;
 }
 
 LatexGenerator::~LatexGenerator()
@@ -182,11 +143,12 @@ void LatexGenerator::init()
   }
   else // use pdflatex for higher quality output
   {
-    t << "all: clean refman.pdf" << endl << endl;
+    t << "all: clean refman.pdf" << endl << endl
+      << "pdf: refman.pdf" << endl << endl;
     t << "refman.pdf: refman.tex" << endl;
     t << "\tpdflatex refman.tex" << endl;
     t << "\tmakeindex refman.idx" << endl;
-    t << "\tpdflatex refman.tex" << endl << endl
+    t << "\tpdflatex refman.tex" << endl
       << "\tlatex_count=5 ; \\" << endl
       << "\twhile egrep -s 'Rerun (LaTeX|to get cross-references right)' refman.log && [ $$latex_count -gt 0 ] ;\\" << endl
       << "\t    do \\" << endl
@@ -235,23 +197,27 @@ static void writeDefaultHeaderPart1(QTextStream &t)
   }
   if (Config_getBool("PDF_HYPERLINKS")) 
   {
-    t << "\\ifx\\pdfoutput\\undefined" << endl
-      << "\\usepackage[ps2pdf," << endl
-      << "            pagebackref=true," << endl
-      << "            colorlinks=true," << endl
-      << "            linkcolor=blue" << endl
-      << "           ]{hyperref}" << endl
-      << "\\usepackage{pspicture}" << endl
-      << "\\else" << endl
+    t << "\\usepackage{ifpdf}" << endl
+      << "\\ifpdf" << endl
       << "\\usepackage[pdftex," << endl
       << "            pagebackref=true," << endl
       << "            colorlinks=true," << endl
-      << "            linkcolor=blue" << endl
+      << "            linkcolor=blue," << endl
+      << "            unicode" << endl
       << "           ]{hyperref}" << endl
+      << "\\else" << endl
+      << "\\usepackage[ps2pdf," << endl
+      << "            pagebackref=true," << endl
+      << "            colorlinks=true," << endl
+      << "            linkcolor=blue," << endl
+      << "            unicode" << endl
+      << "           ]{hyperref}" << endl
+      << "\\usepackage{pspicture}" << endl
       << "\\fi" << endl;
   }
   // Try to get the command for switching on the language
   // support
+  t << "\\usepackage[utf8]{inputenc}" << endl;
   QCString sLanguageSupportCommand(
       theTranslator->latexLanguageSupportCommand());
 
@@ -261,7 +227,6 @@ static void writeDefaultHeaderPart1(QTextStream &t)
     // if the command is empty, no output is needed.
     t << sLanguageSupportCommand << endl;
   }
-
   t << "\\usepackage{doxygen}\n";
   QStrList &extraPackages = Config_getList("EXTRA_PACKAGES");
   const char *s=extraPackages.first();
@@ -335,13 +300,27 @@ static void writeDefaultStyleSheetPart3(QTextStream &t)
 {
   t << "}}\n";
   t << "\\cfoot{}\n";
+  t << "\\newenvironment{Code}\n";
+  t << "{\\footnotesize}\n";
+  t << "{\\normalsize}\n";
+  t << "\\newcommand{\\doxyref}[3]{\\textbf{#1} (\\textnormal{#2}\\,\\pageref{#3})}\n";
+  t << "\\newenvironment{DocInclude}\n";
+  t << "{\\footnotesize}\n";
+  t << "{\\normalsize}\n";
+  t << "\\newenvironment{VerbInclude}\n";
+  t << "{\\footnotesize}\n";
+  t << "{\\normalsize}\n";
+  t << "\\newenvironment{Image}\n";
+  t << "{\\begin{figure}[H]}\n";
+  t << "{\\end{figure}}\n";
+  t << "\\newenvironment{ImageNoCaption}{}{}\n";
   t << "\\newenvironment{CompactList}\n";
   t << "{\\begin{list}{}{\n";
   t << "  \\setlength{\\leftmargin}{0.5cm}\n";
   t << "  \\setlength{\\itemsep}{0pt}\n";
   t << "  \\setlength{\\parsep}{0pt}\n";
   t << "  \\setlength{\\topsep}{0pt}\n";
-  t << "  \\renewcommand{\\makelabel}{}}}\n";
+  t << "  \\renewcommand{\\makelabel}{\\hfill}}}\n";
   t << "{\\end{list}}\n";
   t << "\\newenvironment{CompactItemize}\n";
   t << "{\n";
@@ -363,7 +342,7 @@ static void writeDefaultStyleSheetPart3(QTextStream &t)
   t << "}\n";
   t << "{\\end{tabular*}\\par}\n";
   t << "\\newcommand{\\entrylabel}[1]{\n";
-  t << "   {\\parbox[b]{\\labelwidth-4pt}{\\makebox[0pt][l]{\\textbf{#1}}\\\\}}}\n";
+  t << "   {\\parbox[b]{\\labelwidth-4pt}{\\makebox[0pt][l]{\\textbf{#1}}\\vspace{1.5\\baselineskip}}}}\n";
   t << "\\newenvironment{Desc}\n";
   t << "{\\begin{list}{}\n";
   t << "  {\n";
@@ -411,17 +390,20 @@ void LatexGenerator::writeStyleSheetFile(QFile &f)
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2005";
+  //t << "Dimitri van Heesch \\copyright~1997-2007";
   writeDefaultStyleSheetPart2(t);
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2005";
+  //t << "Dimitri van Heesch \\copyright~1997-2007";
   writeDefaultStyleSheetPart3(t);
 }
 
 void LatexGenerator::startFile(const char *name,const char *,const char *)
 {
+#if 0
+  setEncoding(Config_getString("LATEX_OUTPUT_ENCODING"));
+#endif
   QCString fileName=name;
   relPath = relativePathToRoot(fileName);
   if (fileName.right(4)!=".tex" && fileName.right(4)!=".sty") fileName+=".tex";
@@ -506,7 +488,7 @@ void LatexGenerator::startIndexSection(IndexSections is)
       break;
     case isModuleDocumentation:
       {
-        GroupSDict::Iterator gli(Doxygen::groupSDict);
+        GroupSDict::Iterator gli(*Doxygen::groupSDict);
         GroupDef *gd;
         bool found=FALSE;
         for (gli.toFirst();(gd=gli.current()) && !found;++gli)
@@ -522,7 +504,7 @@ void LatexGenerator::startIndexSection(IndexSections is)
       break;
     case isDirDocumentation:
       {
-        SDict<DirDef>::Iterator dli(Doxygen::directories);
+        SDict<DirDef>::Iterator dli(*Doxygen::directories);
         DirDef *dd;
         bool found=FALSE;
         for (dli.toFirst();(dd=dli.current()) && !found;++dli)
@@ -538,7 +520,7 @@ void LatexGenerator::startIndexSection(IndexSections is)
       break;
     case isNamespaceDocumentation:
       {
-        NamespaceSDict::Iterator nli(Doxygen::namespaceSDict);
+        NamespaceSDict::Iterator nli(*Doxygen::namespaceSDict);
         NamespaceDef *nd;
         bool found=FALSE;
         for (nli.toFirst();(nd=nli.current()) && !found;++nli)
@@ -554,7 +536,7 @@ void LatexGenerator::startIndexSection(IndexSections is)
       break;
     case isClassDocumentation:
       {
-        ClassSDict::Iterator cli(Doxygen::classSDict);
+        ClassSDict::Iterator cli(*Doxygen::classSDict);
         ClassDef *cd=0;
         bool found=FALSE;
         for (cli.toFirst();(cd=cli.current()) && !found;++cli)
@@ -571,7 +553,7 @@ void LatexGenerator::startIndexSection(IndexSections is)
     case isFileDocumentation:
       {
         bool isFirst=TRUE;
-        FileName *fn=Doxygen::inputNameList.first();
+        FileName *fn=Doxygen::inputNameList->first();
         while (fn)
         {
           FileDef *fd=fn->first();
@@ -589,7 +571,7 @@ void LatexGenerator::startIndexSection(IndexSections is)
             }
             fd=fn->next();
           }
-          fn=Doxygen::inputNameList.next();
+          fn=Doxygen::inputNameList->next();
         }
       }
       break;
@@ -656,7 +638,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
       break;
     case isModuleDocumentation:
       {
-        GroupSDict::Iterator gli(Doxygen::groupSDict);
+        GroupSDict::Iterator gli(*Doxygen::groupSDict);
         GroupDef *gd;
         bool found=FALSE;
         for (gli.toFirst();(gd=gli.current()) && !found;++gli)
@@ -679,7 +661,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
       break;
     case isDirDocumentation:
       {
-        SDict<DirDef>::Iterator dli(Doxygen::directories);
+        SDict<DirDef>::Iterator dli(*Doxygen::directories);
         DirDef *dd;
         bool found=FALSE;
         for (dli.toFirst();(dd=dli.current()) && !found;++dli)
@@ -702,7 +684,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
       break;
     case isNamespaceDocumentation:
       {
-        NamespaceSDict::Iterator nli(Doxygen::namespaceSDict);
+        NamespaceSDict::Iterator nli(*Doxygen::namespaceSDict);
         NamespaceDef *nd;
         bool found=FALSE;
         for (nli.toFirst();(nd=nli.current()) && !found;++nli)
@@ -726,7 +708,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
       break;
     case isClassDocumentation:
       {
-        ClassSDict::Iterator cli(Doxygen::classSDict);
+        ClassSDict::Iterator cli(*Doxygen::classSDict);
         ClassDef *cd=0;
         bool found=FALSE;
         for (cli.toFirst();(cd=cli.current()) && !found;++cli)
@@ -750,7 +732,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
     case isFileDocumentation:
       {
         bool isFirst=TRUE;
-        FileName *fn=Doxygen::inputNameList.first();
+        FileName *fn=Doxygen::inputNameList->first();
         while (fn)
         {
           FileDef *fd=fn->first();
@@ -771,7 +753,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
             }
             fd=fn->next();
           }
-          fn=Doxygen::inputNameList.next();
+          fn=Doxygen::inputNameList->next();
         }
       }
       break;
@@ -831,14 +813,14 @@ void LatexGenerator::writeStyleInfo(int part)
       break;
     case 2:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2005";
+        //t << " Dimitri van Heesch \\copyright~1997-2007";
         t << "}]{}\n";
         writeDefaultStyleSheetPart2(t);
       }
       break;
     case 4:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2005";
+        //t << " Dimitri van Heesch \\copyright~1997-2007";
         writeDefaultStyleSheetPart3(t);
         endPlainFile();
       }
@@ -851,23 +833,35 @@ void LatexGenerator::newParagraph()
   t << endl << endl;
 }
 
+void LatexGenerator::startParagraph()
+{
+  t << endl << endl;
+}
+
+void LatexGenerator::endParagraph()
+{
+}
+
 void LatexGenerator::writeString(const char *text)
 {
   t << text;
 }
 
-void LatexGenerator::writeIndexItem(const char *ref,const char *fn,
-                                    const char *name)
+void LatexGenerator::startIndexItem(const char *ref,const char *fn)
 {
   t << "\\item ";
   if (!ref && fn)
   {
     t << "\\contentsline{section}{";
-    docify(name);
+  }
+}
+
+void LatexGenerator::endIndexItem(const char *ref,const char *fn)
+{
+  if (!ref && fn)
+  {
     t << "}{\\pageref{" << fn << "}}{}" << endl;
   }
-  else
-    docify(name);
 }
 
 //void LatexGenerator::writeIndexFileItem(const char *,const char *text)
@@ -910,7 +904,7 @@ void LatexGenerator::endHtmlLink()
 void LatexGenerator::writeStartAnnoItem(const char *,const char *,
                                         const char *path,const char *name)
 {
-  t << "\\item\\contentsline{section}{{\\bf ";
+  t << "\\item\\contentsline{section}{\\bf ";
   if (path) docify(path);
   docify(name); 
   t << "} ";
@@ -952,7 +946,7 @@ void LatexGenerator::endIndexValue(const char *name,bool hasBrief)
 
 void LatexGenerator::startTextLink(const char *f,const char *anchor)
 {
-  if (Config_getBool("PDF_HYPERLINKS"))
+  if (!disableLinks && Config_getBool("PDF_HYPERLINKS"))
   {
     t << "\\hyperlink{";
     if (f) t << stripPath(f);
@@ -973,7 +967,7 @@ void LatexGenerator::endTextLink()
 void LatexGenerator::writeObjectLink(const char *ref, const char *f,
                                      const char *anchor, const char *text)
 {
-  if (!ref && Config_getBool("PDF_HYPERLINKS"))
+  if (!disableLinks && !ref && Config_getBool("PDF_HYPERLINKS"))
   {
     t << "\\hyperlink{";
     if (f) t << stripPath(f);
@@ -993,19 +987,20 @@ void LatexGenerator::writeObjectLink(const char *ref, const char *f,
 
 void LatexGenerator::startPageRef()
 {
-  t << " {\\rm (";
+  t << " \\doxyref{}{";
 }
 
 void LatexGenerator::endPageRef(const char *clname, const char *anchor)
 {
-  t << "\\,\\pageref{";
+  t << "}{";
   if (clname) t << clname; 
   if (anchor) t << "_" << anchor;
-  t << "})}";
+  t << "}";
 }
 
 void LatexGenerator::writeCodeLink(const char *,const char *,
-                                   const char *,const char *name)
+                                   const char *,const char *name,
+                                   const char *)
 {
   t << name;
   col+=strlen(name);
@@ -1032,9 +1027,10 @@ void LatexGenerator::endTitleHead(const char *fileName,const char *name)
   t << "}" << endl;
   if (name)
   {
-    t << "\\label{" << fileName << "}\\index{" 
-      << name << "@{";
-    escapeMakeIndexChars(this,t,name);
+    t << "\\label{" << fileName << "}\\index{";
+    escapeLabelName(name);
+    t << "@{";
+    escapeMakeIndexChars(name);
     t << "}}" << endl;
   }
   if (Config_getBool("PDF_HYPERLINKS") && fileName)
@@ -1065,10 +1061,12 @@ void LatexGenerator::startGroupHeader()
   {
     t << "\\subsection{";
   }
+  disableLinks=TRUE;
 }
 
 void LatexGenerator::endGroupHeader()
 {
+  disableLinks=FALSE;
   t << "}" << endl;
 }
 
@@ -1082,10 +1080,12 @@ void LatexGenerator::startMemberHeader()
   {
     t << "\\subsection*{";
   }
+  disableLinks=TRUE;
 }
 
 void LatexGenerator::endMemberHeader()
 {
+  disableLinks=FALSE;
   t << "}" << endl;
 }
 
@@ -1099,16 +1099,20 @@ void LatexGenerator::startMemberDoc(const char *clname,
     t << "\\index{";
     if (clname)
     {
-      t << clname << "@{";
-      docify(clname);
+      escapeLabelName(clname);
+      t << "@{";
+      escapeMakeIndexChars(clname);
       t << "}!";
     }
-    t << escapeLabelName(memname) << "@{";
-    escapeMakeIndexChars(this,t,memname);
+    escapeLabelName(memname);
+    t << "@{";
+    escapeMakeIndexChars(memname);
     t << "}}" << endl;
 
-    t << "\\index{" << escapeLabelName(memname) << "@{";
-    escapeMakeIndexChars(this,t,memname);
+    t << "\\index{";
+    escapeLabelName(memname);
+    t << "@{";
+    escapeMakeIndexChars(memname);
     t << "}";
     if (clname)
     {
@@ -1119,18 +1123,19 @@ void LatexGenerator::startMemberDoc(const char *clname,
     t << "}" << endl;
   }
   if (Config_getBool("COMPACT_LATEX")) t << "\\paragraph"; else t << "\\subsubsection";
-  if (Config_getBool("PDF_HYPERLINKS") && memname) 
-  {
-    t << "["; 
-    escapeMakeIndexChars(this,t,memname);
-    //filterTitle(title) 
-    t << "]";
-  }
+  //if (Config_getBool("PDF_HYPERLINKS") && memname) 
+  //{
+  //  t << "["; 
+  //  escapeMakeIndexChars(this,t,memname);
+  //  t << "]";
+  //}
   t << "{\\setlength{\\rightskip}{0pt plus 5cm}";
+  disableLinks=TRUE;
 }
 
 void LatexGenerator::endMemberDoc(bool) 
 { 
+  disableLinks=FALSE;
   t << "}";
   if (Config_getBool("COMPACT_LATEX")) t << "\\hfill";
 }
@@ -1180,13 +1185,17 @@ void LatexGenerator::addIndexItem(const char *s1,const char *s2)
 {
   if (s1)
   {
-    t << "\\index{" << escapeLabelName(s1) << "@{";
-    escapeMakeIndexChars(this,t,s1);
+    t << "\\index{";
+    escapeLabelName(s1);
+    t << "@{";
+    escapeMakeIndexChars(s1);
     t << "}";
     if (s2)
     {
-      t << "!" << escapeLabelName(s2) << "@{";
-      escapeMakeIndexChars(this,t,s2);
+      t << "!";
+      escapeLabelName(s2);
+      t << "@{";
+      escapeMakeIndexChars(s2);
       t << "}";
     }
     t << "}";
@@ -1302,7 +1311,7 @@ void LatexGenerator::startClassDiagram()
   //t << "{";
 }
 
-void LatexGenerator::endClassDiagram(ClassDiagram &d,
+void LatexGenerator::endClassDiagram(const ClassDiagram &d,
                                        const char *fileName,const char *)
 {
   d.writeFigure(t,dir,fileName);
@@ -1315,9 +1324,9 @@ void LatexGenerator::startAnonTypeScope(int indent)
   {
     t << "\\begin{tabbing}" << endl;
     t << "xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=\\kill" << endl;
-    //printf("LatexGenerator::startMemberItem() insideTabbing=TRUE\n");
     insideTabbing=TRUE;
   }
+  m_indent=indent;
 }
 
 void LatexGenerator::endAnonTypeScope(int indent)
@@ -1327,14 +1336,32 @@ void LatexGenerator::endAnonTypeScope(int indent)
     t << endl << "\\end{tabbing}";
     insideTabbing=FALSE;
   }
+  m_indent=indent;
 }
 
-void LatexGenerator::startMemberItem(int) 
+void LatexGenerator::startMemberTemplateParams()
+{
+  if (templateMemberItem)
+  {
+    t << "{\\footnotesize ";
+  }
+}
+
+void LatexGenerator::endMemberTemplateParams()
+{
+  if (templateMemberItem)
+  {
+    t << "}\\\\";
+  }
+}
+
+void LatexGenerator::startMemberItem(int annoType) 
 { 
   //printf("LatexGenerator::startMemberItem(%d)\n",annType);
   if (!insideTabbing)
   {
     t << "\\item " << endl; 
+    templateMemberItem = (annoType == 3);
   }
 }
 
@@ -1343,7 +1370,8 @@ void LatexGenerator::endMemberItem()
   if (insideTabbing)
   {
     t << "\\\\";
- } 
+  } 
+  templateMemberItem = FALSE;
   t << endl; 
 }
 
@@ -1355,7 +1383,7 @@ void LatexGenerator::startMemberDescription()
   }
   else
   {
-    for (int i=0;i<m_indent+1;i++) t << "\\>";
+    for (int i=0;i<m_indent+2;i++) t << "\\>";
     t << "{\\em ";
   }
 }
@@ -1368,18 +1396,17 @@ void LatexGenerator::endMemberDescription()
   }
   else
   {
-    t << "}\\\\";
-    m_indent=0;
+    t << "}\\\\\n";
   }
 }
 
 
 void LatexGenerator::writeNonBreakableSpace(int) 
 {
+  //printf("writeNonBreakbleSpace()\n");
   if (insideTabbing)
   {
     t << "\\>";
-    m_indent++;
   }
   else
     t << "~"; 
@@ -1438,7 +1465,7 @@ void LatexGenerator::startDotGraph()
 {
 }
 
-void LatexGenerator::endDotGraph(DotClassGraph &g) 
+void LatexGenerator::endDotGraph(const DotClassGraph &g) 
 {
   g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
 }
@@ -1447,7 +1474,7 @@ void LatexGenerator::startInclDepGraph()
 {
 }
 
-void LatexGenerator::endInclDepGraph(DotInclDepGraph &g) 
+void LatexGenerator::endInclDepGraph(const DotInclDepGraph &g) 
 {
   g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
 }
@@ -1456,7 +1483,7 @@ void LatexGenerator::startGroupCollaboration()
 {
 }
 
-void LatexGenerator::endGroupCollaboration(DotGroupCollaboration &g) 
+void LatexGenerator::endGroupCollaboration(const DotGroupCollaboration &g) 
 {
   g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
 }
@@ -1465,7 +1492,7 @@ void LatexGenerator::startCallGraph()
 {
 }
 
-void LatexGenerator::endCallGraph(DotCallGraph &g) 
+void LatexGenerator::endCallGraph(const DotCallGraph &g) 
 {
   g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
 }
@@ -1474,7 +1501,7 @@ void LatexGenerator::startDirDepGraph()
 {
 }
 
-void LatexGenerator::endDirDepGraph(DotDirDeps &g) 
+void LatexGenerator::endDirDepGraph(const DotDirDeps &g) 
 {
   g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
 }
@@ -1549,3 +1576,84 @@ void LatexGenerator::printDoc(DocNode *n,const char *langExt)
   delete visitor; 
 }
 
+void LatexGenerator::startConstraintList(const char *header)
+{
+  t << "\\begin{Desc}\n\\item[";
+  docify(header);
+  t << "]";
+  t << "\\begin{description}" << endl;
+}
+
+void LatexGenerator::startConstraintParam()
+{
+  t << "\\item[{\\em ";
+}
+
+void LatexGenerator::endConstraintParam()
+{
+}
+
+void LatexGenerator::startConstraintType()
+{
+  t << "} : {\\em ";
+}
+
+void LatexGenerator::endConstraintType()
+{
+  t << "}]";
+}
+
+void LatexGenerator::startConstraintDocs()
+{
+}
+
+void LatexGenerator::endConstraintDocs()
+{
+}
+
+void LatexGenerator::endConstraintList()
+{
+  t << "\\end{description}" << endl;
+  t << "\\end{Desc}" << endl;
+}
+
+void LatexGenerator::escapeLabelName(const char *s)
+{
+  const char *p=s;
+  char str[2];
+  str[1]=0;
+  char c;
+  while ((c=*p++))
+  {
+    switch (c)
+    {
+      case '%': t << "\\%";       break;
+      case '|': t << "\\tt{\"|}"; break;
+      case '!': t << "\"!";       break;
+      default:  str[0]=c; docify(str); break;
+    }
+  }
+}
+
+void LatexGenerator::escapeMakeIndexChars(const char *s)
+{
+  const char *p=s;
+  char str[2];
+  str[1]=0;
+  char c;
+  while ((c=*p++))
+  {
+    switch (c)
+    {
+      case '!': t << "\"!"; break;
+      case '"': t << "\"\""; break;
+      case '@': t << "\"@"; break;
+      case '|': t << "\\tt{\"|}"; break;
+      case '[': t << "["; break;
+      case ']': t << "]"; break;
+      default:  str[0]=c; docify(str); break;
+    }
+  }
+}
+
+
