@@ -61,6 +61,8 @@ class LatexGenerator : public OutputGenerator
     void endTitle()   { t << "}"; }
 
     void newParagraph();
+    void startParagraph();
+    void endParagraph();
     void writeString(const char *text);
     void startIndexList() { t << "\\begin{CompactList}"    << endl; }
     void endIndexList()   { t << "\\end{CompactList}"      << endl; }
@@ -166,7 +168,7 @@ class LatexGenerator : public OutputGenerator
     void endIndent()         {}
     void writeSynopsis()     {}
     void startClassDiagram();
-    void endClassDiagram(ClassDiagram &,const char *,const char *);
+    void endClassDiagram(const ClassDiagram &,const char *,const char *);
     void startPageRef();
     void endPageRef(const char *,const char *);
     void writeQuickLinks(bool,HighlightedItem) {}
@@ -185,16 +187,16 @@ class LatexGenerator : public OutputGenerator
     void lastIndexPage() {}
 
     void startDotGraph();
-    void endDotGraph(DotClassGraph &);
+    void endDotGraph(const DotClassGraph &);
     void startInclDepGraph();
-    void endInclDepGraph(DotInclDepGraph &);
+    void endInclDepGraph(const DotInclDepGraph &);
     void startCallGraph();
     void startGroupCollaboration();
-    void endGroupCollaboration(DotGroupCollaboration &g);
-    void endCallGraph(DotCallGraph &);
+    void endGroupCollaboration(const DotGroupCollaboration &g);
+    void endCallGraph(const DotCallGraph &);
     void startDirDepGraph();
-    void endDirDepGraph(DotDirDeps &g);
-    void writeGraphicalHierarchy(DotGfxHierarchyTable &) {}
+    void endDirDepGraph(const DotDirDeps &g);
+    void writeGraphicalHierarchy(const DotGfxHierarchyTable &) {}
 
     void startTextBlock(bool) {}
     void endTextBlock(bool) {}
@@ -210,6 +212,15 @@ class LatexGenerator : public OutputGenerator
     void startParameterList(bool) {}
     void endParameterList() {}
 
+    void startConstraintList(const char *);
+    void startConstraintParam();
+    void endConstraintParam();
+    void startConstraintType();
+    void endConstraintType();
+    void startConstraintDocs();
+    void endConstraintDocs();
+    void endConstraintList();
+
     void startFontClass(const char *) {}
     void endFontClass() {}
 
@@ -219,6 +230,8 @@ class LatexGenerator : public OutputGenerator
   private:
     LatexGenerator(const LatexGenerator &);
     LatexGenerator &operator=(const LatexGenerator &);
+    void escapeLabelName(const char *s);
+    void escapeMakeIndexChars(const char *s);
     int col;
     bool insideTabbing;
     bool firstDescItem;
