@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.h,v 1.15 1998/11/28 11:33:19 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -31,7 +31,8 @@ class ManGenerator : public OutputGenerator
     //OutputGenerator *copy()  { return new ManGenerator; } 
     //OutputGenerator *clone() { return new ManGenerator(*this); }
     //void append(const OutputGenerator *o);
-    void enable() { active=TRUE; }
+    void enable() 
+    { if (genStack->top()) active=*genStack->top(); else active=TRUE; }
     void disable() { active=FALSE; }
     void enableIf(OutputType o)  { if (o==Man) active=TRUE;  }
     void disableIf(OutputType o) { if (o==Man) active=FALSE; }
@@ -39,7 +40,7 @@ class ManGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==Man && active); } 
     OutputGenerator *get(OutputType o) { return (o==Man) ? this : 0; }
 
-    void printDoc(DocNode *);
+    void printDoc(DocNode *,const char *);
 
     static void init();
     void startFile(const char *name,const char *manName,const char *title);
@@ -47,7 +48,6 @@ class ManGenerator : public OutputGenerator
     void endFile();
     void clearBuffer();
 
-    //void writeIndex();
     void startIndexSection(IndexSections) {}
     void endIndexSection(IndexSections) {}
     void startProjectNumber() {}
@@ -59,6 +59,8 @@ class ManGenerator : public OutputGenerator
     void endTitle();
     
     void newParagraph();
+    void startParagraph();
+    void endParagraph();
     void writeString(const char *text);
     void startIndexList() {}
     void endIndexList()   { newParagraph(); } 
@@ -68,23 +70,19 @@ class ManGenerator : public OutputGenerator
     void endIndexValue(const char *,bool)   {} 
     void startItemList()  {}
     void endItemList()    { newParagraph(); }
-    //void startEnumList()  {}
-    //void endEnumList()    { newParagraph(); }
-    //void startAlphabeticalIndexList() {}
-    //void endAlphabeticalIndexList() {} 
-    //void writeIndexHeading(const char *) {}
-    void writeIndexItem(const char *ref,const char *file,const char *name);
+    void startIndexItem(const char *ref,const char *file);
+    void endIndexItem(const char *ref,const char *file);
     void docify(const char *text);
     void codify(const char *text);
     void writeObjectLink(const char *ref,const char *file,
                          const char *anchor,const char *name);
     void writeCodeLink(const char *ref,const char *file,
-                       const char *anchor,const char *name);
+                       const char *anchor,const char *name,
+                       const char *tooltip);
     void startTextLink(const char *,const char *) {}
     void endTextLink() {}
     void startHtmlLink(const char *url);
     void endHtmlLink();
-    //void writeMailLink(const char *url);
     void startTypewriter() { t << "\\fC"; firstCol=FALSE; }
     void endTypewriter()   { t << "\\fP"; firstCol=FALSE; }
     void startGroupHeader();
@@ -97,6 +95,8 @@ class ManGenerator : public OutputGenerator
     void startMemberSubtitle() {}
     void endMemberSubtitle() {}
     void writeListItem();
+    void startMemberDocList() {}
+    void endMemberDocList() {}
     void startMemberList();
     void endMemberList();
     void startAnonTypeScope(int);
@@ -117,15 +117,9 @@ class ManGenerator : public OutputGenerator
     void writeAnchor(const char *,const char *) {}
     void startCodeFragment();
     void endCodeFragment();
-    //void startPreFragment()      { startCodeFragment(); }
-    //void endPreFragment()        { endCodeFragment(); }
-    //void startVerbatimFragment() { startCodeFragment(); }
-    //void endVerbatimFragment()   { endCodeFragment(); }
     void writeLineNumber(const char *,const char *,const char *,int l) { t << l << " "; }
     void startCodeLine() {}
     void endCodeLine() { codify("\n"); col=0; }
-    //void writeBoldString(const char *text) 
-    //                     { t << "\\fB"; docify(text); t << "\\fP"; firstCol=FALSE; }
     void startEmphasis() { t << "\\fI"; firstCol=FALSE; }
     void endEmphasis()   { t << "\\fP"; firstCol=FALSE; }
     void startBold()     { t << "\\fB"; firstCol=FALSE; }
@@ -138,12 +132,11 @@ class ManGenerator : public OutputGenerator
     void writeChar(char c);
     void startMemberDoc(const char *,const char *,const char *,const char *);
     void endMemberDoc(bool);
-    void startDoxyAnchor(const char *,const char *,const char *,const char *);
+    void startDoxyAnchor(const char *,const char *,const char *,const char *,const char *);
     void endDoxyAnchor(const char *,const char *) {}
     void startCodeAnchor(const char *) {}
     void endCodeAnchor() {}
     void writeLatexSpacing() {}
-    //void writeLatexLabel(const char *,const char *) {}
     void writeStartAnnoItem(const char *type,const char *file,
                             const char *path,const char *name);
     void writeEndAnnoItem(const char *) { t << endl; firstCol=TRUE; }
@@ -155,33 +148,6 @@ class ManGenerator : public OutputGenerator
     void endCenter()          {}
     void startSmall()         {}
     void endSmall()           {}
-    //void startSubscript()     { t << "\\*<"; firstCol=FALSE; }
-    //void endSubscript()       { t << "\\*>"; firstCol=FALSE; }
-    //void startSuperscript()   { t << "\\*{"; firstCol=FALSE; }
-    //void endSuperscript()     { t << "\\*}"; firstCol=FALSE; }
-    //void startTable(bool,int) {}
-    //void endTable(bool)       {}
-    //void startCaption()       {}
-    //void endCaption()         {}
-    //void nextTableRow()       {}
-    //void endTableRow()        {}
-    //void nextTableColumn()    {}
-    //void endTableColumn()     {}
-    //void writeCopyright()     { t << "(c)"; firstCol=FALSE; }
-    //void writeQuote()         { t << "`"; firstCol=FALSE; }
-    //void writeUmlaut(char c)  { t << c << "\\*'"; firstCol=FALSE; }
-    //void writeAcute(char c)   { t << c << "\\*`"; firstCol=FALSE; }
-    //void writeGrave(char c)   { t << c << "\\*:"; firstCol=FALSE; }
-    //void writeCirc(char c)    { t << c << "\\*^"; firstCol=FALSE; }
-    //void writeTilde(char c)   { t << c << "\\*~"; firstCol=FALSE; } 
-    //void writeRing(char c)    { t << c << "\\*0"; /* just a wild guess, 
-    //                                                 need to check! */ 
-    //                                              firstCol=FALSE; }
-    //void writeSharpS()        { t << "s\\*:";     /* just a wild guess,
-    //                                                 need to check! */
-    //                                              firstCol=FALSE; }
-    //void writeCCedil(char c)  { t << c;           /* TODO: fix this */
-    //                            firstCol=FALSE; }
     void startMemberDescription() { t << "\n.RI \"\\fI"; firstCol=FALSE; }
     void endMemberDescription()   { t << "\\fP\""; firstCol=FALSE; }
     void startDescList(SectionTypes);
@@ -190,33 +156,19 @@ class ManGenerator : public OutputGenerator
     void endSimpleSect();
     void startParamList(ParamListTypes,const char *title);
     void endParamList();
-    //void endDescTitle();
     void writeDescItem();
     void startSection(const char *,const char *,SectionInfo::SectionType);
     void endSection(const char *,SectionInfo::SectionType);
-    //void writeSectionRef(const char *,const char *,const char *,const char *) {}
-    //void writeSectionRefItem(const char *,const char *,const char *) {}
-    //void writeSectionRefAnchor(const char *,const char *,const char *) {}
     void addIndexItem(const char *,const char *) {}
     void startIndent()        {}
     void endIndent()          {}
     void writeSynopsis();
-    //void generateExternalIndex() {}
     void startClassDiagram() {}
-    void endClassDiagram(ClassDiagram &,const char *,const char *) {}
-    //void startColorFont(uchar,uchar,uchar) {}
-    //void endColorFont()   {}
+    void endClassDiagram(const ClassDiagram &,const char *,const char *) {}
     void startPageRef() {}
     void endPageRef(const char *,const char *) {}
-    //void startQuickIndexItem(const char *,const char *) {}
-    //void endQuickIndexItem() {}
     void writeQuickLinks(bool,HighlightedItem) {}
-    //void writeFormula(const char *,const char *) {}
     void writeNonBreakableSpace(int n) { int i; for (i=0;i<n;i++) t << " "; }
-    //void startImage(const char *,const char *,bool) {}
-    //void endImage(bool) {}
-    //void startDotFile(const char *,bool) {}
-    //void endDotFile(bool) {}
     
     void startDescTable() {}
     void endDescTable() {}
@@ -226,16 +178,16 @@ class ManGenerator : public OutputGenerator
     void endDescTableData() {}
 
     void startDotGraph() {}
-    void endDotGraph(DotClassGraph &) {}
+    void endDotGraph(const DotClassGraph &) {}
     void startInclDepGraph() {}
-    void endInclDepGraph(DotInclDepGraph &) {}
+    void endInclDepGraph(const DotInclDepGraph &) {}
     void startGroupCollaboration() {}
-    void endGroupCollaboration(DotGroupCollaboration &) {}
+    void endGroupCollaboration(const DotGroupCollaboration &) {}
     void startCallGraph() {}
-    void endCallGraph(DotCallGraph &) {}
+    void endCallGraph(const DotCallGraph &) {}
     void startDirDepGraph() {} 
-    void endDirDepGraph(DotDirDeps &) {}
-    void writeGraphicalHierarchy(DotGfxHierarchyTable &) {}
+    void endDirDepGraph(const DotDirDeps &) {}
+    void writeGraphicalHierarchy(const DotGfxHierarchyTable &) {}
 
     void startTextBlock(bool) {}
     void endTextBlock(bool) {}
@@ -255,15 +207,17 @@ class ManGenerator : public OutputGenerator
     void startFontClass(const char *) {}
     void endFontClass() {}
 
-    //void startHtmlOnly()  {}
-    //void endHtmlOnly()    {}
-    //void startLatexOnly() {}
-    //void endLatexOnly()   {}
-
-    //void startSectionRefList() {}
-    //void endSectionRefList() {}
+    void startConstraintList(const char *);
+    void startConstraintParam();
+    void endConstraintParam();
+    void startConstraintType();
+    void endConstraintType();
+    void startConstraintDocs();
+    void endConstraintDocs();
+    void endConstraintList();
 
     void writeCodeAnchor(const char *) {}
+    void linkableSymbol(int,const char *,Definition *,Definition *) {}
 
   private:
     bool firstCol;
