@@ -2,7 +2,7 @@
  *
  * $Id: outputgen.h,v 1.48 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,12 +36,14 @@ class DotGroupCollaboration;
 class DocNode;
 class MemberDef;
 class GroupDef;
+class Definition;
 
 /*! \brief Output interface for code parser. 
  */
-class BaseCodeDocInterface
+class CodeOutputInterface
 {
   public:
+    virtual ~CodeOutputInterface() {}
     /*! Writes an ASCII string to the output. This function should keep 
      *  spaces visible, should break lines at a newline and should convert 
      *  tabs to the right number of spaces.
@@ -49,15 +51,17 @@ class BaseCodeDocInterface
     virtual void codify(const char *s) = 0;
 
     /*! Writes a link to an object in a code fragment.
-     *  \param ref    If this is non-zero, the object is to be found in
-     *                an external documentation file.
-     *  \param file   The file in which the object is located.
-     *  \param anchor The anchor uniquely identifying the object within 
-     *                the file. 
-     *  \param name   The text to display as a placeholder for the link.
+     *  \param ref      If this is non-zero, the object is to be found in
+     *                  an external documentation file.
+     *  \param file     The file in which the object is located.
+     *  \param anchor   The anchor uniquely identifying the object within 
+     *                  the file. 
+     *  \param name     The text to display as a placeholder for the link.
+     *  \param tooltip  The tooltip to display when the mouse is on the link.
      */
     virtual void writeCodeLink(const char *ref,const char *file,
-                               const char *anchor,const char *name) = 0;
+                               const char *anchor,const char *name,
+                               const char *tooltip) = 0;
 
     virtual void writeLineNumber(const char *ref,const char *file,
                                  const char *anchor,int lineNumber) = 0;
@@ -68,6 +72,8 @@ class BaseCodeDocInterface
     virtual void startFontClass(const char *) = 0;
     virtual void endFontClass() = 0;
     virtual void writeCodeAnchor(const char *name) = 0;
+    virtual void linkableSymbol(int line,const char *symName,
+                 Definition *symDef,Definition *context) = 0;
 };
 
 /*! \brief Base Interface used for generating output outside of the
@@ -78,9 +84,10 @@ class BaseCodeDocInterface
  *  or a list of formats (see OutputList). This interface
  *  contains functions that generate fragments of the output.
  */
-class BaseOutputDocInterface : public BaseCodeDocInterface
+class BaseOutputDocInterface : public CodeOutputInterface
 {
   public:
+    virtual ~BaseOutputDocInterface() {}
     enum ParamListTypes { Param, RetVal, Exception };
     enum SectionTypes { /*See, Return, Author, Version, 
                         Since, Date, Bug, Note,
@@ -125,6 +132,11 @@ class BaseOutputDocInterface : public BaseCodeDocInterface
     /*! Starts a new paragraph */
     virtual void newParagraph()   = 0;
 
+    /*! Starts a new paragraph */
+    virtual void startParagraph() = 0;
+    /*! End a paragraph */
+    virtual void endParagraph() = 0;
+
     /*! Writes a link to an object in the documentation.
      *  \param ref    If this is non-zero, the object is to be found in
      *                an external documentation file.
@@ -271,8 +283,10 @@ class OutputGenerator : public BaseOutputDocInterface
     bool isEnabled() const { return active; }
     void pushGeneratorState();
     void popGeneratorState();
+    void setEncoding(const QCString &enc) { encoding = enc; }
+    virtual void postProcess(QByteArray &) { }
 
-    virtual void printDoc(DocNode *) = 0;
+    virtual void printDoc(DocNode *,const char *langExt) = 0;
 
     ///////////////////////////////////////////////////////////////
     // structural output interface
@@ -294,8 +308,8 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endIndexKey()   = 0;
     virtual void startIndexValue(bool) = 0;
     virtual void endIndexValue(const char *,bool) = 0;
-    virtual void writeIndexItem(const char *ref,const char *file,
-                                const char *text) = 0;
+    virtual void startIndexItem(const char *ref,const char *file) = 0;
+    virtual void endIndexItem(const char *ref,const char *file) = 0;
     virtual void startGroupHeader() = 0;
     virtual void endGroupHeader() = 0;
     virtual void startMemberSections() = 0;
@@ -304,8 +318,10 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endMemberHeader() = 0;
     virtual void startMemberSubtitle() = 0;
     virtual void endMemberSubtitle() = 0;
-    virtual void startMemberList()  = 0;
-    virtual void endMemberList()    = 0;
+    virtual void startMemberDocList() = 0;
+    virtual void endMemberDocList() = 0;
+    virtual void startMemberList() = 0;
+    virtual void endMemberList() = 0;
     virtual void startAnonTypeScope(int) = 0;
     virtual void endAnonTypeScope(int) = 0;
     virtual void startMemberItem(int) = 0;
@@ -323,7 +339,8 @@ class OutputGenerator : public BaseOutputDocInterface
                                 const char *,const char *) = 0;
     virtual void endMemberDoc(bool) = 0;
     virtual void startDoxyAnchor(const char *fName,const char *manName,
-                                 const char *anchor,const char *name) = 0;
+                                 const char *anchor,const char *name,
+                                 const char *args) = 0;
     virtual void endDoxyAnchor(const char *fileName,const char *anchor) = 0;
     virtual void writeLatexSpacing() = 0;
     virtual void writeStartAnnoItem(const char *type,const char *file,
@@ -335,20 +352,18 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endIndent() = 0;
     virtual void writeSynopsis() = 0;
     virtual void startClassDiagram() = 0;
-    virtual void endClassDiagram(ClassDiagram &,const char *,const char *) = 0;
+    virtual void endClassDiagram(const ClassDiagram &,const char *,const char *) = 0;
     virtual void startDotGraph() = 0;
-    virtual void endDotGraph(DotClassGraph &g) = 0;
+    virtual void endDotGraph(const DotClassGraph &g) = 0;
     virtual void startInclDepGraph() = 0;
-    virtual void endInclDepGraph(DotInclDepGraph &g) = 0;
+    virtual void endInclDepGraph(const DotInclDepGraph &g) = 0;
     virtual void startGroupCollaboration() = 0;
-    virtual void endGroupCollaboration(DotGroupCollaboration &g) = 0;
+    virtual void endGroupCollaboration(const DotGroupCollaboration &g) = 0;
     virtual void startCallGraph() = 0;
-    virtual void endCallGraph(DotCallGraph &g) = 0;
+    virtual void endCallGraph(const DotCallGraph &g) = 0;
     virtual void startDirDepGraph() = 0;
-    virtual void endDirDepGraph(DotDirDeps &g) = 0;
-    virtual void writeGraphicalHierarchy(DotGfxHierarchyTable &g) = 0;
-    //virtual void startQuickIndexItem(const char *s,const char *l) = 0;
-    //virtual void endQuickIndexItem() = 0;
+    virtual void endDirDepGraph(const DotDirDeps &g) = 0;
+    virtual void writeGraphicalHierarchy(const DotGfxHierarchyTable &g) = 0;
     virtual void writeQuickLinks(bool compact,HighlightedItem hli) = 0;
     virtual void startTextBlock(bool) = 0;
     virtual void endTextBlock(bool) = 0;
@@ -364,6 +379,15 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void startParameterList(bool) = 0;
     virtual void endParameterList() = 0;
 
+    virtual void startConstraintList(const char *) = 0;
+    virtual void startConstraintParam() = 0;
+    virtual void endConstraintParam() = 0;
+    virtual void startConstraintType() = 0;
+    virtual void endConstraintType() = 0;
+    virtual void startConstraintDocs() = 0;
+    virtual void endConstraintDocs() = 0;
+    virtual void endConstraintList() = 0;
+
   protected:
     QTextStream fs;
     QByteArray a;
@@ -372,11 +396,12 @@ class OutputGenerator : public BaseOutputDocInterface
     QFile *file;
     QCString dir;
     bool active;
+    QStack<bool> *genStack;
+    QString encoding;
 
   private:
     OutputGenerator(const OutputGenerator &o);
     OutputGenerator &operator=(const OutputGenerator &o);
-    QStack<bool> *genStack;
 };
 
 /*! \brief Interface used for generating documentation.
