@@ -2,7 +2,7 @@
  *
  * $Id: outputlist.cpp,v 1.35 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2004 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -26,6 +26,7 @@
 #include "outputgen.h"
 #include "config.h"
 #include "message.h"
+#include "definition.h"
 
 #include "docparser.h"
 
@@ -132,7 +133,8 @@ void OutputList::popGeneratorState()
 void OutputList::parseDoc(const char *fileName,int startLine,
                   Definition *ctx,MemberDef * md,
                   const QCString &docStr,bool indexWords,
-                  bool isExample,const char *exampleName)
+                  bool isExample,const char *exampleName,
+                  bool singleLine,bool linkFromIndex)
 {
   int count=0;
   if (docStr.isEmpty()) return;
@@ -149,18 +151,22 @@ void OutputList::parseDoc(const char *fileName,int startLine,
   if (docStr.at(docStr.length()-1)=='\n')
   {
     root = validatingParseDoc(fileName,startLine,
-                              ctx,md,docStr,indexWords,isExample,exampleName);
+                              ctx,md,docStr,indexWords,isExample,exampleName,
+                              singleLine,linkFromIndex);
   }
   else
   {
     root = validatingParseDoc(fileName,startLine,
-                              ctx,md,docStr+"\n",indexWords,isExample,exampleName);
+                              ctx,md,docStr+"\n",indexWords,isExample,exampleName,
+                              singleLine,linkFromIndex);
   }
 
   og=outputs->first();
   while (og)
   {
-    if (og->isEnabled()) og->printDoc(root);
+    //printf("og->printDoc(extension=%s)\n",
+    //    ctx?ctx->getDefFileExtension().data():"<null>");
+    if (og->isEnabled()) og->printDoc(root,ctx?ctx->getDefFileExtension():QCString(""));
     og=outputs->next();
   }
 
@@ -183,7 +189,7 @@ void OutputList::parseText(const QCString &textStr)
   og=outputs->first();
   while (og)
   {
-    if (og->isEnabled()) og->printDoc(root);
+    if (og->isEnabled()) og->printDoc(root,0);
     og=outputs->next();
   }
 
@@ -255,15 +261,29 @@ void OutputList::forall(void (OutputGenerator::*func)(a1,a2,a3,a4),a1,a2,a3,a4)
   }                                                                             \
 }                     
 
+// five arguments
+#define FORALL5(a1,a2,a3,a4,a5,p1,p2,p3,p4,p5)                                        \
+void OutputList::forall(void (OutputGenerator::*func)(a1,a2,a3,a4,a5),a1,a2,a3,a4,a5) \
+{                                                                                     \
+  OutputGenerator *og=outputs->first();                                               \
+  while (og)                                                                          \
+  {                                                                                   \
+    if (og->isEnabled()) (og->*func)(p1,p2,p3,p4,p5);                                 \
+    og=outputs->next();                                                               \
+  }                                                                                   \
+}                     
+
 // now instantiate only the ones we need.
 
 FORALL1(const char *a1,a1)
 FORALL1(char a1,a1)
 FORALL1(int a1,a1)
-FORALL1(DotClassGraph &a1,a1)
-FORALL1(DotInclDepGraph &a1,a1)
-FORALL1(DotCallGraph &a1,a1)
-FORALL1(DotGfxHierarchyTable &a1,a1)
+FORALL1(const DotClassGraph &a1,a1)
+FORALL1(const DotInclDepGraph &a1,a1)
+FORALL1(const DotCallGraph &a1,a1)
+FORALL1(const DotDirDeps &a1,a1)
+FORALL1(const DotGfxHierarchyTable &a1,a1)
+FORALL1(const DotGroupCollaboration &a1,a1)
 FORALL1(SectionTypes a1,a1)
 #if defined(HAS_BOOL_TYPE) || defined(Q_HAS_BOOL_TYPE)
 FORALL1(bool a1,a1)
@@ -279,7 +299,8 @@ FORALL1(IndexSections a1,a1)
 FORALL2(const char *a1,const char *a2,a1,a2)
 FORALL2(const char *a1,bool a2,a1,a2)
 FORALL2(const char *a1,SectionInfo::SectionType a2,a1,a2)
-FORALL3(ClassDiagram &a1,const char *a2,const char *a3,a1,a2,a3)
+FORALL3(bool a1,bool a2,bool a3,a1,a2,a3)
+FORALL3(const ClassDiagram &a1,const char *a2,const char *a3,a1,a2,a3)
 FORALL3(const char *a1,const char *a2,const char *a3,a1,a2,a3)
 FORALL3(const char *a1,const char *a2,bool a3,a1,a2,a3)
 FORALL3(const char *a1,const char *a2,SectionInfo::SectionType a3,a1,a2,a3)
@@ -287,6 +308,7 @@ FORALL3(uchar a1,uchar a2,uchar a3,a1,a2,a3)
 FORALL4(SectionTypes a1,const char *a2,const char *a3,const char *a4,a1,a2,a3,a4)
 FORALL4(const char *a1,const char *a2,const char *a3,const char *a4,a1,a2,a3,a4)
 FORALL4(const char *a1,const char *a2,const char *a3,int a4,a1,a2,a3,a4)
+FORALL5(const char *a1,const char *a2,const char *a3,const char *a4,const char *a5,a1,a2,a3,a4,a5)
 
 
 //--------------------------------------------------------------------------
