@@ -2,7 +2,7 @@
  *
  * $Id: outputlist.h,v 1.47 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -31,12 +31,16 @@
   void forall(void (OutputGenerator::*func)(arg1,arg2,arg3),arg1,arg2,arg3)
 #define FORALLPROTO4(arg1,arg2,arg3,arg4) \
   void forall(void (OutputGenerator::*func)(arg1,arg2,arg3,arg4),arg1,arg2,arg3,arg4)
+#define FORALLPROTO5(arg1,arg2,arg3,arg4,arg5) \
+  void forall(void (OutputGenerator::*func)(arg1,arg2,arg3,arg4,arg5),arg1,arg2,arg3,arg4,arg5)
   
 class ClassDiagram;
 class DotClassGraph;
+class DotDirDeps;
 class DotInclDepGraph;
 class DotGfxHierarchyTable;
 class SectionDict;
+class DotGroupCollaboration;
 
 class OutputList : public OutputDocInterface
 {
@@ -62,7 +66,8 @@ class OutputList : public OutputDocInterface
 
     void parseDoc(const char *fileName,int startLine,
                   Definition *ctx,MemberDef *md,const QCString &docStr,
-                  bool indexWords,bool isExample,const char *exampleName=0);
+                  bool indexWords,bool isExample,const char *exampleName=0,
+                  bool singleLine=FALSE,bool linkFromIndex=FALSE);
     void parseText(const QCString &textStr);
     
 
@@ -92,6 +97,10 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::endTitle); }
     void newParagraph() 
     { forall(&OutputGenerator::newParagraph); }
+    void startParagraph() 
+    { forall(&OutputGenerator::startParagraph); }
+    void endParagraph() 
+    { forall(&OutputGenerator::endParagraph); }
     void writeString(const char *text) 
     { forall(&OutputGenerator::writeString,text); }
     void startIndexList() 
@@ -110,8 +119,10 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startItemList); }
     void endItemList() 
     { forall(&OutputGenerator::endItemList); }
-    void writeIndexItem(const char *ref,const char *file,const char *text)
-    { forall(&OutputGenerator::writeIndexItem,ref,file,text); }
+    void startIndexItem(const char *ref,const char *file)
+    { forall(&OutputGenerator::startIndexItem,ref,file); }
+    void endIndexItem(const char *ref,const char *file)
+    { forall(&OutputGenerator::endIndexItem,ref,file); }
     void docify(const char *s)
     { forall(&OutputGenerator::docify,s); }
     void codify(const char *s)
@@ -120,8 +131,9 @@ class OutputList : public OutputDocInterface
                          const char *anchor, const char *name)
     { forall(&OutputGenerator::writeObjectLink,ref,file,anchor,name); }
     void writeCodeLink(const char *ref,const char *file,
-                       const char *anchor,const char *name)
-    { forall(&OutputGenerator::writeCodeLink,ref,file,anchor,name); }
+                       const char *anchor,const char *name,
+                       const char *tooltip)
+    { forall(&OutputGenerator::writeCodeLink,ref,file,anchor,name,tooltip); }
     void startTextLink(const char *file,const char *anchor)
     { forall(&OutputGenerator::startTextLink,file,anchor); }
     void endTextLink()
@@ -157,6 +169,10 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startMemberSubtitle); }
     void endMemberSubtitle()
     { forall(&OutputGenerator::endMemberSubtitle); }
+    void startMemberDocList() 
+    { forall(&OutputGenerator::startMemberDocList); }
+    void endMemberDocList() 
+    { forall(&OutputGenerator::endMemberDocList); }
     void startMemberList() 
     { forall(&OutputGenerator::startMemberList); }
     void endMemberList() 
@@ -169,6 +185,10 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startMemberItem,i1); }
     void endMemberItem() 
     { forall(&OutputGenerator::endMemberItem); }
+    void startMemberTemplateParams() 
+    { forall(&OutputGenerator::startMemberTemplateParams); }
+    void endMemberTemplateParams() 
+    { forall(&OutputGenerator::endMemberTemplateParams); }
     void startMemberGroupHeader(bool b) 
     { forall(&OutputGenerator::startMemberGroupHeader,b); }
     void endMemberGroupHeader()
@@ -181,8 +201,8 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startMemberGroup); }
     void endMemberGroup(bool last)
     { forall(&OutputGenerator::endMemberGroup,last); }
-    void insertMemberAlign() 
-    { forall(&OutputGenerator::insertMemberAlign); }
+    void insertMemberAlign(bool templ=FALSE) 
+    { forall(&OutputGenerator::insertMemberAlign,templ); }
     void writeRuler() 
     { forall(&OutputGenerator::writeRuler); }
     void writeAnchor(const char *fileName,const char *name)
@@ -207,11 +227,12 @@ class OutputList : public OutputDocInterface
     void startMemberDoc(const char *clName,const char *memName,
                         const char *anchor,const char *title)
     { forall(&OutputGenerator::startMemberDoc,clName,memName,anchor,title); }
-    void endMemberDoc() 
-    { forall(&OutputGenerator::endMemberDoc); }
+    void endMemberDoc(bool hasArgs) 
+    { forall(&OutputGenerator::endMemberDoc,hasArgs); }
     void startDoxyAnchor(const char *fName,const char *manName,
-                         const char *anchor, const char *name)
-    { forall(&OutputGenerator::startDoxyAnchor,fName,manName,anchor,name); }
+                         const char *anchor, const char *name,
+                         const char *args)
+    { forall(&OutputGenerator::startDoxyAnchor,fName,manName,anchor,name,args); }
     void endDoxyAnchor(const char *fn,const char *anchor)
     { forall(&OutputGenerator::endDoxyAnchor,fn,anchor); }
     void startCodeAnchor(const char *label)
@@ -279,16 +300,12 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::writeSynopsis); }
     void startClassDiagram()
     { forall(&OutputGenerator::startClassDiagram); }
-    void endClassDiagram(ClassDiagram &d,const char *f,const char *n)
+    void endClassDiagram(const ClassDiagram &d,const char *f,const char *n)
     { forall(&OutputGenerator::endClassDiagram,d,f,n); }
     void startPageRef()
     { forall(&OutputGenerator::startPageRef); }
     void endPageRef(const char *c,const char *a)
     { forall(&OutputGenerator::endPageRef,c,a); }
-    //void startQuickIndexItem(const char *s,const char *l)
-    //{ forall(&OutputGenerator::startQuickIndexItem,s,l); }
-    //void endQuickIndexItem()
-    //{ forall(&OutputGenerator::endQuickIndexItem); }
     void writeQuickLinks(bool compact,HighlightedItem hli)
     { forall(&OutputGenerator::writeQuickLinks,compact,hli); }
     void writeNonBreakableSpace(int num)
@@ -307,44 +324,69 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::endDescTableData); }
     void startDotGraph()
     { forall(&OutputGenerator::startDotGraph); }
-    void endDotGraph(DotClassGraph &g)
+    void endDotGraph(const DotClassGraph &g)
     { forall(&OutputGenerator::endDotGraph,g); }
     void startInclDepGraph()
     { forall(&OutputGenerator::startInclDepGraph); }
-    void endInclDepGraph(DotInclDepGraph &g)
+    void endInclDepGraph(const DotInclDepGraph &g)
     { forall(&OutputGenerator::endInclDepGraph,g); }
     void startCallGraph()
     { forall(&OutputGenerator::startCallGraph); }
-    void endCallGraph(DotCallGraph &g)
+    void endCallGraph(const DotCallGraph &g)
     { forall(&OutputGenerator::endCallGraph,g); }
-    void writeGraphicalHierarchy(DotGfxHierarchyTable &g)
+    void startDirDepGraph()
+    { forall(&OutputGenerator::startDirDepGraph); }
+    void endDirDepGraph(const DotDirDeps &g)
+    { forall(&OutputGenerator::endDirDepGraph,g); }
+    void startGroupCollaboration()
+    { forall(&OutputGenerator::startGroupCollaboration); }
+    void endGroupCollaboration(const DotGroupCollaboration &g)
+    { forall(&OutputGenerator::endGroupCollaboration,g); }
+    void writeGraphicalHierarchy(const DotGfxHierarchyTable &g)
     { forall(&OutputGenerator::writeGraphicalHierarchy,g); }
     void startTextBlock(bool dense=FALSE)
     { forall(&OutputGenerator::startTextBlock,dense); }
-    void endTextBlock()
-    { forall(&OutputGenerator::endTextBlock); }
+    void endTextBlock(bool paraBreak=FALSE)
+    { forall(&OutputGenerator::endTextBlock,paraBreak); }
     void lastIndexPage()
     { forall(&OutputGenerator::lastIndexPage); }
     void startMemberDocPrefixItem()
     { forall(&OutputGenerator::startMemberDocPrefixItem); }
     void endMemberDocPrefixItem()
     { forall(&OutputGenerator::endMemberDocPrefixItem); }
-    void startMemberDocName()
-    { forall(&OutputGenerator::startMemberDocName); }
+    void startMemberDocName(bool align)
+    { forall(&OutputGenerator::startMemberDocName,align); }
     void endMemberDocName()
     { forall(&OutputGenerator::endMemberDocName); }
-    void startParameterType(bool first)
-    { forall(&OutputGenerator::startParameterType,first); }
+    void startParameterType(bool first,const char *key)
+    { forall(&OutputGenerator::startParameterType,first,key); }
     void endParameterType()
     { forall(&OutputGenerator::endParameterType); }
     void startParameterName(bool one)
     { forall(&OutputGenerator::startParameterName,one); }
-    void endParameterName(bool last,bool one)
-    { forall(&OutputGenerator::endParameterName,last,one); }
-    void startParameterList()
-    { forall(&OutputGenerator::startParameterList); }
+    void endParameterName(bool last,bool one,bool bracket)
+    { forall(&OutputGenerator::endParameterName,last,one,bracket); }
+    void startParameterList(bool openBracket)
+    { forall(&OutputGenerator::startParameterList,openBracket); }
     void endParameterList()
     { forall(&OutputGenerator::endParameterList); }
+
+    void startConstraintList(const char *header) 
+    { forall(&OutputGenerator::startConstraintList,header); }
+    void startConstraintParam() 
+    { forall(&OutputGenerator::startConstraintParam); }
+    void endConstraintParam() 
+    { forall(&OutputGenerator::endConstraintParam); }
+    void startConstraintType()
+    { forall(&OutputGenerator::startConstraintType); }
+    void endConstraintType()
+    { forall(&OutputGenerator::endConstraintType); }
+    void startConstraintDocs()
+    { forall(&OutputGenerator::startConstraintDocs); }
+    void endConstraintDocs()
+    { forall(&OutputGenerator::endConstraintDocs); }
+    void endConstraintList()
+    { forall(&OutputGenerator::endConstraintList); }
     void startFontClass(const char *c)
     { forall(&OutputGenerator::startFontClass,c); }
     void endFontClass()
@@ -369,6 +411,9 @@ class OutputList : public OutputDocInterface
         og=outputs->next();
       }
     }
+    void linkableSymbol(int,const char *,Definition *,Definition *) {}
+
+
 
   private:
     void debug();
@@ -379,10 +424,12 @@ class OutputList : public OutputDocInterface
     FORALLPROTO1(char);
     FORALLPROTO1(IndexSections);
     FORALLPROTO1(int);
-    FORALLPROTO1(DotClassGraph &);
-    FORALLPROTO1(DotInclDepGraph &);
-    FORALLPROTO1(DotCallGraph &);
-    FORALLPROTO1(DotGfxHierarchyTable &);
+    FORALLPROTO1(const DotClassGraph &);
+    FORALLPROTO1(const DotInclDepGraph &);
+    FORALLPROTO1(const DotCallGraph &);
+    FORALLPROTO1(const DotGroupCollaboration &);
+    FORALLPROTO1(const DotDirDeps &);
+    FORALLPROTO1(const DotGfxHierarchyTable &);
     FORALLPROTO1(SectionTypes);
 #if defined(HAS_BOOL_TYPE) || defined(Q_HAS_BOOL_TYPE)
     FORALLPROTO1(bool);
@@ -392,18 +439,21 @@ class OutputList : public OutputDocInterface
 #endif
     FORALLPROTO2(int,bool);
     FORALLPROTO2(bool,HighlightedItem);
+    FORALLPROTO2(bool,const char *);
     FORALLPROTO2(ParamListTypes,const char *);
     FORALLPROTO2(const char *,const char *);
     FORALLPROTO2(const char *,bool);
     FORALLPROTO2(const char *,SectionInfo::SectionType);
+    FORALLPROTO3(bool,bool,bool);
     FORALLPROTO3(const char *,const char *,bool);
     FORALLPROTO3(const char *,const char *,SectionInfo::SectionType);
     FORALLPROTO3(uchar,uchar,uchar);
     FORALLPROTO3(const char *,const char *,const char *);
-    FORALLPROTO3(ClassDiagram &,const char *,const char *);
+    FORALLPROTO3(const ClassDiagram &,const char *,const char *);
     FORALLPROTO4(SectionTypes,const char *,const char *,const char *);
     FORALLPROTO4(const char *,const char *,const char *,const char *);
     FORALLPROTO4(const char *,const char *,const char *,bool);
+    FORALLPROTO5(const char *,const char *,const char *,const char *,const char *);
   
     OutputList(const OutputList &ol);
     QList<OutputGenerator> *outputs;
