@@ -14,6 +14,7 @@ PageDef::PageDef(const char *f,int l,const char *n,
 {
   setDocumentation(d,f,l);
   subPageDict = new PageSDict(7);
+  pageScope = 0;
 }
 
 PageDef::~PageDef()
@@ -28,7 +29,8 @@ void PageDef::findSectionsInDocumentation()
 
 GroupDef *PageDef::getGroupDef() const 
 { 
-  return partOfGroups() ? partOfGroups()->getFirst() : 0; 
+  LockingPtr<GroupList> groups = partOfGroups();
+  return groups!=0 ? groups->getFirst() : 0; 
 }
 
 QCString PageDef::getOutputFileBase() const 
@@ -77,7 +79,7 @@ void PageDef::writeDocumentation(OutputList &ol)
       (si=Doxygen::sectionDict.find(pageName))!=0)
   {
     ol.startSection(si->label,si->title,si->type);
-    ol.docify(si->title);
+    ol.parseDoc(docFile(),docLine(),this,0,si->title,TRUE,FALSE);
     stringToSearchIndex(getOutputFileBase(),
                         theTranslator->trPage(TRUE,TRUE)+" "+si->title,
                         si->title);
