@@ -2,7 +2,7 @@
  *
  * $Id: rtfgen.h,v 1.13 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2005 by Parker Waechter & Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Parker Waechter & Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -31,10 +31,8 @@ class RTFGenerator : public OutputGenerator
     static void writeStyleSheetFile(QFile &f);
     static void writeExtensionsFile(QFile &file);
 
-    //OutputGenerator *copy();
-    //OutputGenerator *clone() { return new RTFGenerator(*this); }
-    //void append(const OutputGenerator *o);
-    void enable() { active=TRUE; }
+    void enable() 
+    { if (genStack->top()) active=*genStack->top(); else active=TRUE; }
     void disable() { active=FALSE; }
     void enableIf(OutputType o)  { if (o==RTF) active=TRUE;  }
     void disableIf(OutputType o) { if (o==RTF) active=FALSE; }
@@ -42,15 +40,14 @@ class RTFGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==RTF && active); } 
     OutputGenerator *get(OutputType o) { return (o==RTF) ? this : 0; }
 
-    void printDoc(DocNode *);
+    void printDoc(DocNode *,const char *);
 
     void startFile(const char *name,const char *manName,const char *title);
     void writeFooter() {}
     void endFile();
     void clearBuffer();
+    void postProcess(QByteArray &);
     
-    //void writeIndex();
-	
     void startIndexSection(IndexSections);
     void endIndexSection(IndexSections);
     void startProjectNumber();
@@ -62,6 +59,8 @@ class RTFGenerator : public OutputGenerator
     void endTitle() {} 
 
     void newParagraph();
+    void startParagraph();
+    void endParagraph();
     void writeString(const char *text);
     void startIndexList();
     void endIndexList();
@@ -71,23 +70,19 @@ class RTFGenerator : public OutputGenerator
     void endIndexValue(const char *,bool);
     void startItemList();
     void endItemList();
-    //void startEnumList();
-    //void endEnumList();
-    //void startAlphabeticalIndexList() {}
-    //void endAlphabeticalIndexList() {} 
-    //void writeIndexHeading(const char *) {}
-    void writeIndexItem(const char *ref,const char *file,const char *name);
+    void startIndexItem(const char *ref,const char *file);
+    void endIndexItem(const char *ref,const char *file);
     void docify(const char *text);
     void codify(const char *text);
     void writeObjectLink(const char *ref,const char *file,
                          const char *anchor,const char *name);
     void writeCodeLink(const char *ref, const char *file,
-                       const char *anchor,const char *name);
+                       const char *anchor,const char *name,
+                       const char *tooltip);
     void startTextLink(const char *f,const char *anchor);
     void endTextLink();
     void startHtmlLink(const char *url);
     void endHtmlLink();
-    //void writeMailLink(const char *url);
     void startTypewriter() { t << "{\\f2 "; }
     void endTypewriter()   { t << "}";      }
     void startGroupHeader();
@@ -100,6 +95,8 @@ class RTFGenerator : public OutputGenerator
     void endMemberHeader() { endGroupHeader(); }
     void startMemberSubtitle(); 
     void endMemberSubtitle(); 
+    void startMemberDocList() {}
+    void endMemberDocList() {}
     void startMemberList();
     void endMemberList();
     void startAnonTypeScope(int) {}
@@ -108,8 +105,6 @@ class RTFGenerator : public OutputGenerator
     void endMemberItem();
     void startMemberTemplateParams() {}
     void endMemberTemplateParams() {}
-    //void memberGroupSpacing(bool) {}
-    //void memberGroupSeparator() {}
     void insertMemberAlign(bool) {}
 
     void writeRuler() { rtfwriteRuler_thin(); }
@@ -117,15 +112,9 @@ class RTFGenerator : public OutputGenerator
     void writeAnchor(const char *fileName,const char *name);
     void startCodeFragment();
     void endCodeFragment();
-    //void startPreFragment()      { startCodeFragment(); }
-    //void endPreFragment()        { endCodeFragment(); }
-    //void startVerbatimFragment() { startCodeFragment(); }
-    //void endVerbatimFragment()   { endCodeFragment(); }
     void writeLineNumber(const char *,const char *,const char *,int l) { t << l << " "; }
     void startCodeLine() { col=0; }
     void endCodeLine() { lineBreak(); }
-    //void writeBoldString(const char *text) 
-    //                     { t << "{\\b "; docify(text); t << "}"; }
     void startEmphasis() { t << "{\\i ";  }
     void endEmphasis()   { t << "}"; }
     void startBold()     { t << "{\\b "; }
@@ -137,13 +126,12 @@ class RTFGenerator : public OutputGenerator
     void lineBreak();
     void startMemberDoc(const char *,const char *,const char *,const char *);
     void endMemberDoc(bool);
-    void startDoxyAnchor(const char *,const char *,const char *,const char *);
+    void startDoxyAnchor(const char *,const char *,const char *,const char *,const char *);
     void endDoxyAnchor(const char *,const char *);
     void startCodeAnchor(const char *) {};
     void endCodeAnchor() {};
     void writeChar(char c);
     void writeLatexSpacing() {};//{ t << "\\hspace{0.3cm}"; }
-    //void writeLatexLabel(const char *scope,const char *anchor);
     void writeStartAnnoItem(const char *type,const char *file, 
                             const char *path,const char *name);
     void writeEndAnnoItem(const char *name);
@@ -155,64 +143,27 @@ class RTFGenerator : public OutputGenerator
     void endCenter()        { t << "}"; }
     void startSmall()       { t << "{\\sub "; }
     void endSmall()         { t << "}"; }
-    //void startSubscript()   { t << "{\\sub " << endl;}
-    //void endSubscript()     { t << "}"; }
-    //void startSuperscript() { t << "{\\super " << endl;}
-    //void endSuperscript()   { t << "}"; }
-    //void startTable(bool,int);
-    //void endTable(bool);
-    //void startCaption();
-    //void endCaption();
-    //void nextTableRow();
-    //void endTableRow();
-    //void nextTableColumn();
-    //void endTableColumn();
-    //void writeCopyright()    { t << "\251"; }
-    //void writeQuote()        { t << "\""; }
-    //void writeUmlaut(char c);
-    //void writeAcute(char c);
-    //void writeGrave(char c);
-    //void writeCirc(char c);
-    //void writeTilde(char c);
-    //void writeRing(char c);
-    //void writeSharpS()       { t << "\337"; }
-    //void writeCCedil(char c);
 
     void startMemberDescription();
     void endMemberDescription();
     void startDescList(SectionTypes);
-    //void endDescList();
     void startSimpleSect(SectionTypes,const char *,const char *,const char *);
     void endSimpleSect();
     void startParamList(ParamListTypes,const char *);
     void endParamList();
-    //void endDescTitle();
     void writeDescItem();
     void startSection(const char *,const char *,SectionInfo::SectionType);
     void endSection(const char *,SectionInfo::SectionType);
-    //void writeSectionRef(const char *,const char *,const char *,const char *);
-    //void writeSectionRefItem(const char *,const char *,const char *);
-    //void writeSectionRefAnchor(const char *,const char *,const char *);
     void addIndexItem(const char *,const char *);
     void startIndent();
     void endIndent();
     void writeSynopsis()     {}
-    //void generateExternalIndex() {}
     void startClassDiagram();
-    void endClassDiagram(ClassDiagram &,const char *filename,const char *name);
-    //void startColorFont(uchar,uchar,uchar) {}
-    //void endColorFont()   {}
+    void endClassDiagram(const ClassDiagram &,const char *filename,const char *name);
     void startPageRef();
     void endPageRef(const char *,const char *);
-    //void startQuickIndexItem(const char *,const char *) {}
-    //void endQuickIndexItem() {}
     void writeQuickLinks(bool,HighlightedItem) {}
-    //void writeFormula(const char *,const char *);
     void writeNonBreakableSpace(int);
-    //void startImage(const char *,const char *,bool);
-    //void endImage(bool);
-    //void startDotFile(const char *,bool);
-    //void endDotFile(bool);
 	
     void startDescTable();
     void endDescTable();
@@ -222,16 +173,16 @@ class RTFGenerator : public OutputGenerator
     void endDescTableData();
     
     void startDotGraph();
-    void endDotGraph(DotClassGraph &);
+    void endDotGraph(const DotClassGraph &);
     void startInclDepGraph();
-    void endInclDepGraph(DotInclDepGraph &);
+    void endInclDepGraph(const DotInclDepGraph &);
     void startGroupCollaboration();
-    void endGroupCollaboration(DotGroupCollaboration &g);
+    void endGroupCollaboration(const DotGroupCollaboration &g);
     void startCallGraph();
-    void endCallGraph(DotCallGraph &);
+    void endCallGraph(const DotCallGraph &);
     void startDirDepGraph();
-    void endDirDepGraph(DotDirDeps &g);
-    void writeGraphicalHierarchy(DotGfxHierarchyTable &) {}
+    void endDirDepGraph(const DotDirDeps &g);
+    void writeGraphicalHierarchy(const DotGfxHierarchyTable &) {}
 
     void startMemberGroupHeader(bool);
     void endMemberGroupHeader();
@@ -255,18 +206,21 @@ class RTFGenerator : public OutputGenerator
     void startParameterList(bool) {}
     void endParameterList() {}
 
+    void startConstraintList(const char  *);
+    void startConstraintParam();
+    void endConstraintParam();
+    void startConstraintType();
+    void endConstraintType();
+    void startConstraintDocs();
+    void endConstraintDocs();
+    void endConstraintList();
+
+
     void startFontClass(const char *) {}
     void endFontClass() {}
 
-    //void startHtmlOnly()  {}
-    //void endHtmlOnly()    {}
-    //void startLatexOnly() {}
-    //void endLatexOnly()   {}
-
-    //void startSectionRefList() {}
-    //void endSectionRefList() {}
-
     void writeCodeAnchor(const char *) {}
+    void linkableSymbol(int,const char *,Definition *,Definition *) {}
 
     static bool preProcessFileInplace(const char *path,const char *name);
     
@@ -298,7 +252,7 @@ class RTFGenerator : public OutputGenerator
     void rtfwriteRuler_thick();
     void rtfwriteRuler_thin();
     void writeRTFReference(const char *label);
-    char *getMultiByte(int c);
+    //char *getMultiByte(int c);
 };
 
 #endif
