@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -10,26 +10,20 @@
  * Documents produced by Doxygen are derivative works derived from the
  * input used in their production; they are not affected by this license.
  *
- * Brazilian Portuguese version by
- *   Fabio "FJTC" Jun Takada Chino <jun-chino at uol.com.br>
- *   Version: 1.3.8 (2004/06/30)
+ * Brazilian Portuguese translation version 20070914
+ *    Maintainer: Fabio "FJTC" Jun Takada Chino <jun-chino at uol.com.br>
+ *    Thanks to Jorge Ramos and others
  *
  * History:
- * 	 1.3.9
- *		- Updated to Doxygen 1.3.9
- * 	 1.3.8
- *		- Updated to Doxygen 1.3.8
- *   1.3.3:
- *      - Updated to Doxygen 1.3.3
- *   1.3.2:
- *      - Updated to Doxygen 1.3.2
- *   1.3:
- *      - Updated to Doxygen 1.3.x.
- *   1.2.19:
- *      - Updated to Doxygen 1.2.19
- *      - Translation Review (Thanks to Jorge Ramos and others)
- *      - Small fixes in C output.
- *      - Latex generation support fixed (invalid package name).
+ *   20070914
+ *      - "Translate me" fixed
+ *      - Revision number changed from doxygen version to a date string.
+ *      - History cleanup
+ *      - Latex babel package fixed.
+ *   1.4.6
+ *      - trCallerGraph() added.
+ *      - trEnumerationValueDocumentation() added.
+ *   Previous history removed from this version.
  */
 #ifndef TRANSLATOR_BR_H
 #define TRANSLATOR_BR_H
@@ -59,7 +53,7 @@ class TranslatorBrazilian : public Translator
      */
     virtual QCString latexLanguageSupportCommand()
     {
-      return "";
+      return "\\usepackage[brazil]{babel}";
     }
 
     /*! return the language charset. This will be used for the HTML output */
@@ -178,10 +172,6 @@ class TranslatorBrazilian : public Translator
     virtual QCString trFileList()
     { return "Lista de Arquivos"; }
 
-    /*! This is put above each page as a link to the list of all verbatim headers */
-    virtual QCString trHeaderFiles()
-    { return "Arquivos de Cabealho (Headers)"; }
-
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     {
@@ -316,10 +306,6 @@ class TranslatorBrazilian : public Translator
       return result;
     }
 
-    /*! This is an introduction to the page with the list of all header files. */
-    virtual QCString trHeaderFilesDescription()
-    { return "Estes so os arquivos de cabealho que compe a API:"; }
-
     /*! This is an introduction to the page with the list of all examples */
     virtual QCString trExamplesDescription()
     { return "Esta  a lista com todos os exemplos:"; }
@@ -332,15 +318,6 @@ class TranslatorBrazilian : public Translator
     virtual QCString trModulesDescription()
     { return "Esta  a lista com todos os mdulos:"; }
 
-    /*! This sentences is used in the annotated class/file lists if no brief
-     * description is given.
-     */
-    virtual QCString trNoDescriptionAvailable()
-    { return "Sem descrio disponvel"; }
-
-    // index titles (the project name is prepended for these)
-
-
     /*! This is used in HTML as the title of index.html. */
     virtual QCString trDocumentation()
     { return "\nDocumentao"; }
@@ -467,7 +444,7 @@ class TranslatorBrazilian : public Translator
      *  documentation blocks for defines
      */
     virtual QCString trDefineDocumentation()
-    { return "Deinies e macros"; }
+    { return "Definies e macros"; }
 
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for function prototypes
@@ -487,12 +464,6 @@ class TranslatorBrazilian : public Translator
     virtual QCString trEnumerationTypeDocumentation()
     { return "Enumeraes"; }
 
-    /*! This is used in the documentation of a file/namespace before the list
-     *  of documentation blocks for enumeration values
-     */
-    virtual QCString trEnumerationValueDocumentation()
-    { return "Valores enumerados"; }
-
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for functions
      */
@@ -547,18 +518,10 @@ class TranslatorBrazilian : public Translator
     virtual QCString trForInternalUseOnly()
     { return "Apenas para uso interno."; }
 
-    /*! this text is generated when the \\reimp command is used. */
-    virtual QCString trReimplementedForInternalReasons()
-    { return "Reimplementado por razes internas. A API no foi modificada."; }
-
     /*! this text is generated when the \\warning command is used. */
     virtual QCString trWarning()
     { return "Aviso"; }
 
-    /*! this text is generated when the \\bug command is used. */
-    virtual QCString trBugsAndLimitations()
-    { return "Problemas e Limitaes"; }
-
     /*! this text is generated when the \\version command is used. */
     virtual QCString trVersion()
     { return "Verso"; }
@@ -637,8 +600,8 @@ class TranslatorBrazilian : public Translator
         case ClassDef::Struct:     result+=" Estrutura "; break;
         case ClassDef::Union:      result+=" Unio "; break;
         case ClassDef::Interface:  result+=" Interface "; break;
-        case ClassDef::Protocol:   result+=" Protocol "; break; // translate me!
-        case ClassDef::Category:   result+=" Category "; break; // translate me!
+        case ClassDef::Protocol:   result+=" Protocolo "; break;
+        case ClassDef::Category:   result+=" Categoria "; break;
         case ClassDef::Exception:  result+=" Exceo "; break;
       }
       result+=(QCString)clName;
@@ -795,8 +758,8 @@ class TranslatorBrazilian : public Translator
         case ClassDef::Struct:     result+="estrutura "; break;
         case ClassDef::Union:      result+="unio "; break;
         case ClassDef::Interface:  result+="interface "; break;
-        case ClassDef::Protocol:   result+="protocol "; break; // translate me!
-        case ClassDef::Category:   result+="category "; break; // translate me!
+        case ClassDef::Protocol:   result+="protocolo "; break;
+        case ClassDef::Category:   result+="categoria "; break;
         case ClassDef::Exception:  result+="exceo "; break;
       }
       result+=" foi gerada a partir ";
@@ -833,10 +796,6 @@ class TranslatorBrazilian : public Translator
 // new since 0.49-991003
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trSources()
-    {
-      return "Cdigos-Fonte";
-    }
     virtual QCString trDefinedAtLineInSourceFile()
     {
       return "Definio na linha @0 do arquivo @1.";
@@ -1046,6 +1005,8 @@ class TranslatorBrazilian : public Translator
         "class Undocumented { };\n\n"
         "/*! Class that is inherited using public inheritance */\n"
         "class PublicBase : public Truncated { };\n\n"
+        "/*! A template class */\n"
+        "template<class T> class Templ { };\n\n"
         "/*! Class that is inherited using protected inheritance */\n"
         "class ProtectedBase { };\n\n"
         "/*! Class that is inherited using private inheritance */\n"
@@ -1056,7 +1017,8 @@ class TranslatorBrazilian : public Translator
         "class Inherited : public PublicBase,\n"
         "                  protected ProtectedBase,\n"
         "                  private PrivateBase,\n"
-        "                  public Undocumented\n"
+        "                  public Undocumented,\n"
+        "                  public Templ<int>\n"
         "{\n"
         "  private:\n"
         "    Used *m_usedClass;\n"
@@ -1139,11 +1101,6 @@ class TranslatorBrazilian : public Translator
 // new since 1.2.4
 //////////////////////////////////////////////////////////////////////////
 
-    /*! Used for Java interfaces in the summary section of Java packages */
-    virtual QCString trInterfaces()
-    {
-      return "Interfaces";
-    }
     /*! Used for Java classes in the summary section of Java packages */
     virtual QCString trClasses()
     {
@@ -1176,11 +1133,6 @@ class TranslatorBrazilian : public Translator
     {
       return "Pacotes";
     }
-    /*! Used as a chapter title for Latex & RTF output */
-    virtual QCString trPackageDocumentation()
-    {
-      return "Pacotes";
-    }
     /*! Text shown before a multi-line define */
     virtual QCString trDefineValue()
     {
@@ -1317,17 +1269,6 @@ class TranslatorBrazilian : public Translator
       return result;
     }
 
-    /*! This is used for translation of the word that will possibly
-     *  be followed by a single name or by a list of names
-     *  of the category.
-     */
-    virtual QCString trField(bool first_capital, bool singular)
-    {
-      QCString result((first_capital ? "Campo" : "campo"));
-      if (!singular)  result+="s";
-      return result;
-    }
-
     /*! This is used for translation of the word that will possibly
      *  be followed by a single name or by a list of names
      *  of the category.
@@ -1597,5 +1538,35 @@ class TranslatorBrazilian : public Translator
       if (!singular) result+="s";
       return result; 
     }
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! This text is added to the documentation when the \\overload command
+     *  is used for a overloaded function.
+     */
+    virtual QCString trOverloadText()
+    {
+       return "Este  um mtodo provido por convenincia. "
+       		"Ele difere do mtodo acima apenas na lista de "
+       		"argumentos que devem ser utilizados.";
+    }    
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.6
+//////////////////////////////////////////////////////////////////////////
+
+    /*! This is used to introduce a caller (or called-by) graph */
+    virtual QCString trCallerGraph()
+    {
+      return "Este  o diagrama de chamadas para esta funo:";
+    }
+
+    /*! This is used in the documentation of a file/namespace before the list 
+     *  of documentation blocks for enumeration values
+     */
+    virtual QCString trEnumerationValueDocumentation()
+    { return "Documentao da enumerao"; }
+  
 };
 #endif
