@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2006 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -24,7 +24,7 @@
 */
 #define CN_SPC
 
-class TranslatorChinese : public TranslatorAdapter_1_4_1
+class TranslatorChinese : public Translator
 {
   public:
 		/*! Used for identification of the language. The identification
@@ -33,6 +33,7 @@ class TranslatorChinese : public TranslatorAdapter_1_4_1
 		 * (e.g. "czech", "japanese", "russian", etc.). It sould be equal to
 		 * the identification used in the language.cpp.
 		 */
+    
     virtual QCString idLanguage()
     { return "chinese"; }
 
@@ -474,8 +475,8 @@ class TranslatorChinese : public TranslatorAdapter_1_4_1
         case ClassDef::Struct: result+="ṹ"; break;
         case ClassDef::Union:  result+=""; break;
         case ClassDef::Interface:  result+="ӿ"; break;
-        case ClassDef::Protocol:   result+="Э"; break; // translate me!
-        case ClassDef::Category:   result+=""; break; // translate me!
+        case ClassDef::Protocol:   result+="Э"; break;
+        case ClassDef::Category:   result+=""; break;
         case ClassDef::Exception:  result+="쳣"; break;
       }
       result+="ο";
@@ -619,7 +620,8 @@ class TranslatorChinese : public TranslatorAdapter_1_4_1
      */
  		virtual QCString trNamespaces()
     {
-      return "Namespaces";
+      // return "Namespaces";
+      return "ֿռ";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -640,8 +642,8 @@ class TranslatorChinese : public TranslatorAdapter_1_4_1
         case ClassDef::Struct:     result+="ṹ"; break;
         case ClassDef::Union:      result+=""; break;
         case ClassDef::Interface:  result+="ӿ"; break;
-        case ClassDef::Protocol:   result+="Э"; break; // translate me!
-        case ClassDef::Category:   result+=""; break; // translate me!
+        case ClassDef::Protocol:   result+="Э"; break;
+        case ClassDef::Category:   result+=""; break;
         case ClassDef::Exception:  result+="쳣"; break;
       }
       result+="ĵļɣ";
@@ -911,6 +913,8 @@ class TranslatorChinese : public TranslatorAdapter_1_4_1
         "class Undocumented { };\n\n"
         "/*! м̳е */\n"
         "class PublicBase : public Truncated { };\n\n"
+        "/*! A template class */\n"
+        "template<class T> class Templ { };\n\n"
         "/*! ̳е */\n"
         "class ProtectedBase { };\n\n"
         "/*! ˽м̳е */\n"
@@ -921,7 +925,8 @@ class TranslatorChinese : public TranslatorAdapter_1_4_1
         "class Inherited : public PublicBase,\n"
         "                  protected ProtectedBase,\n"
         "                  private PrivateBase,\n"
-        "                  public Undocumented\n"
+        "                  public Undocumented,\n"
+        "                  public Templ<int>\n"
         "{\n"
         "  private:\n"
         "    Used *m_usedClass;\n"
@@ -1494,6 +1499,43 @@ class TranslatorChinese : public TranslatorAdapter_1_4_1
       return "Ŀ¼";
     }
     
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! This text is added to the documentation when the \\overload command
+     *  is used for a overloaded function.
+     */
+    virtual QCString trOverloadText()
+    {
+       /* return "This is an overloaded member function, "
+              "provided for convenience. It differs from the above "
+              "function only in what argument(s) it accepts."; */
+      return "ΪʹöṩһسԱ"
+             "ĺȣܲͬ͵Ĳ";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.6
+//////////////////////////////////////////////////////////////////////////
+
+    /*! This is used to introduce a caller (or called-by) graph */
+    virtual QCString trCallerGraph()
+    {
+      /* return "Here is the caller graph for this function:"; */
+      return "ĵͼ";
+    }
+
+    /*! This is used in the documentation of a file/namespace before the list 
+     *  of documentation blocks for enumeration values
+     */
+    virtual QCString trEnumerationValueDocumentation()
+    { 
+      /* return "Enumerator Documentation"; */
+      return "öٱĵ";
+    }
+
+
 };
 
 #endif
