@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,16 +21,18 @@
 // translation by Dejan D. M. Milosavljevic <dmilos@email.com>;<dmilosx@ptt.yu>;<office@ddmrm.com>
 // // 10x 2 Ivana Miletic for gramatical consutation.
 
-class TranslatorSerbian : public Translator
+// UTF-8 patch by Nenad Bulatovic <buletina@gmail.com>
+
+class TranslatorSerbian : public TranslatorAdapter_1_4_1
 {                                
 private:
    QCString decode(const QCString& sInput)
    { 
-#ifdef _WIN32
-     return ISO88592ToWin1250(sInput);
-#else
+//#ifdef _WIN32
+//     return ISO88592ToWin1250(sInput);
+//#else
      return sInput;
-#endif
+//#endif
    }
 
 
@@ -63,17 +65,18 @@ private:
      */
     virtual QCString latexLanguageSupportCommand()
     {
-      return "";
+      QCString result="\\usepackage[serbian]{babel}\n";
+      return result;
     }
 
     /*! return the language charset. This will be used for the HTML output */
     virtual QCString idLanguageCharset()
     {
-#ifdef _WIN32
-        { return "windows-1250"; }
-#else
-        { return "iso-8859-2"; }
-#endif
+//#ifdef _WIN32
+//        { return "windows-1250"; }
+//#else
+        { return "UTF-8"; }
+//#endif
     }
 
     // --- Language translation methods -------------------
@@ -84,56 +87,56 @@ private:
 
     /*! subscript for the related functions. */
     virtual QCString trRelatedSubscript()
-    { return decode( "(To nisu funkcije lanice.)" ); } 
+    { return decode( "(To nisu funkcije članice.)" ); } 
 
     /*! header that is put before the detailed description of files, classes and namespaces. */
     virtual QCString trDetailedDescription()
-    { return decode( "Opirnije objanjenje" ); } // detalj je francuska rec.
+    { return decode( "Opširnije objašnjenje" ); } // detalj je francuska rec.
 
     /*! header that is put before the list of typedefs. */
     virtual QCString trMemberTypedefDocumentation()
-    { return decode( "Dokumentacija typedef lanova" ); }
+    { return decode( "Dokumentacija typedef članova" ); }
     
     /*! header that is put before the list of enumerations. */
     virtual QCString trMemberEnumerationDocumentation()
-    { return decode( "Dokumentacija enum lanova" ); }
+    { return decode( "Dokumentacija enum članova" ); }
     
     /*! header that is put before the list of member functions. */
     virtual QCString trMemberFunctionDocumentation()
-    { return decode( "Dokumentacija funkcija lanica" ); }
+    { return decode( "Dokumentacija funkcija članica" ); }
     
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return decode( "Dokumentacija lanova" ); 
+        return decode( "Dokumentacija članova" ); 
       }
       else
       {
-        return decode( "Dokumentacija javnih lanova" ); 
+        return decode( "Dokumentacija javnih članova" ); 
       }
     }
 
     /*! this is the text of a link put after brief descriptions. */
     virtual QCString trMore() 
-    { return decode( "Jo..." ); }
+    { return decode( "Još..." ); }
 
     /*! put in the class documentation */
     virtual QCString trListOfAllMembers()
-    { return decode( "Spisak svih lanova." ); }
+    { return decode( "Spisak svih članova." ); }
 
     /*! used as the title of the "list of all members" page of a class */
     virtual QCString trMemberList()
-    { return decode( "Spisak lanova" ); }
+    { return decode( "Spisak članova" ); }
 
     /*! this is the first part of a sentence that is followed by a class name */
     virtual QCString trThisIsTheListOfAllMembers()
-    { return decode( "Ovo je spisak svih lanova " ); }
+    { return decode( "Ovo je spisak svih članova " ); }
 
     /*! this is the remainder of the sentence after the class name */
     virtual QCString trIncludingInheritedMembers()
-    { return decode( ", ukljuujui nasleene lanove." ); }
+    { return decode( ", uključujući nasleđene članove." ); }
     
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
@@ -195,11 +198,11 @@ private:
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return decode( "Svi lanovi struktura" ); 
+        return decode( "Svi članovi struktura" ); 
       }             
       else
       {
-        return decode( "Svi lanovi klasa" ); 
+        return decode( "Svi članovi klasa" ); 
       }
     }
 
@@ -208,11 +211,11 @@ private:
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-       return decode( "lanovi datoteke" ); 
+       return decode( "Članovi datoteke" ); 
       }
       else
       {
-       return decode( "lanovi datoteke" ); 
+       return decode( "Članovi datoteke" ); 
       }
     }
 
@@ -226,12 +229,12 @@ private:
 
     /*! This is put above each page as a link to the search engine. */
     virtual QCString trSearch()
-    { return decode( "Trai" ); }
+    { return decode( "Traži" ); }
 
     /*! This is an introduction to the class hierarchy. */
     virtual QCString trClassHierarchyDescription()
-    { return decode( "Stablo nasleivanja je sloeno "
-                     "priblino po abecedi:" ); }
+    { return decode( "Stablo nasleđivanja je složeno "
+                     "približno po abecedi:" ); }
 
     /*! This is an introduction to the list with all files. */
     virtual QCString trFileListDescription(bool extractAll)
@@ -252,7 +255,7 @@ private:
       }
       else
       {
-        return decode( "Spisak klasa, struktura, unija i interjfejsa sa kratkim opisom:" );
+        return decode( "Spisak klasa, struktura, unija i interfejsa sa kratkim opisom:" );
                
       }
     }
@@ -267,22 +270,22 @@ private:
       }
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        result+="lanova struktura ili unija";
+        result+="članova struktura ili unija";
       }
       else
       {
-        result+="lanova klasa";
+        result+="članova klasa";
       }
       result+=" sa linkovima na ";
       if (extractAll) 
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
-          result+="dokumentaciju lanova struktura/unija:";  
+          result+="dokumentaciju članova struktura/unija:";  
         }
         else
         {
-          result+="dokumentaciju svakog lana klase:";       
+          result+="dokumentaciju svakog člana klase:";       
         }
       }
       else 
@@ -311,7 +314,7 @@ private:
       }
       else
       {
-        result+="lanova";               
+        result+="članova";               
       }
      
       result+=" sa linkovima na ";
@@ -326,7 +329,7 @@ private:
 
     /*! This is an introduction to the page with the list of all header files. */
     virtual QCString trHeaderFilesDescription()
-    { return decode( "Zaglavlja koje izgrauju API:" ); }
+    { return decode( "Zaglavlja koje izgrađuju API:" ); }
 
     /*! This is an introduction to the page with the list of all examples */
     virtual QCString trExamplesDescription()
@@ -334,7 +337,7 @@ private:
 
     /*! This is an introduction to the page with the list of related pages */
     virtual QCString trRelatedPagesDescription()
-    { return decode( "Spisak svih slinih stranica:" ); }
+    { return decode( "Spisak svih sličnih stranica:" ); }
 
     /*! This is an introduction to the page with the list of class/file groups */
     virtual QCString trModulesDescription()
@@ -363,7 +366,7 @@ private:
      * class hierarchy.
      */
     virtual QCString trHierarchicalIndex()
-    { return decode( "Hijerarhijski sadraj" ); }
+    { return decode( "Hijerarhijski sadžraj" ); }
 
     /*! This is used in LaTeX as the title of the chapter with the 
      * annotated compound index.
@@ -372,11 +375,11 @@ private:
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       { 
-        return decode( "Glavni sadraj" ); 
+        return decode( "Glavni sadržaj" ); 
       }
       else
       {
-        return decode( "Glavni sadraj" ); 
+        return decode( "Glavni sadržaj" ); 
       }
     }
 
@@ -427,7 +430,7 @@ private:
 
     /*! This is used in LaTeX as the title of the document */
     virtual QCString trReferenceManual()
-    { return decode( "Prirunik" ); }
+    { return decode( "Priručnik" ); }
     
     /*! This is used in the documentation of a file as a header before the 
      *  list of defines
@@ -548,16 +551,16 @@ private:
     /*! this text is put before a class diagram */
     virtual QCString trClassDiagram(const char *clName)
     {
-      return decode( QCString("Dijagram nasleivanja za klasu ") + clName + ":" );
+      return decode( QCString("Dijagram nasleđivanja za klasu ") + clName + ":" );
     }
     
     /*! this text is generated when the \\internal command is used. */
     virtual QCString trForInternalUseOnly()
-    { return decode( "Samo za unutranju upotrebu." ); }
+    { return decode( "Samo za unutrašnju upotrebu." ); }
 
     /*! this text is generated when the \\reimp command is used. */
     virtual QCString trReimplementedForInternalReasons()
-    { return decode("Preuraeno zbog unutranjih razloga; Nema uticaja na API." ); }
+    { return decode("Preurađeno zbog unutrašnjih razloga; Nema uticaja na API." ); }
 
     /*! this text is generated when the \\warning command is used. */
     virtual QCString trWarning()
@@ -565,7 +568,7 @@ private:
 
     /*! this text is generated when the \\bug command is used. */
     virtual QCString trBugsAndLimitations()
-    { return decode( "Greke i ogranienja" ); }
+    { return decode( "Greške i ograničenja" ); }
 
     /*! this text is generated when the \\version command is used. */
     virtual QCString trVersion()
@@ -577,11 +580,11 @@ private:
 
     /*! this text is generated when the \\return command is used. */
     virtual QCString trReturns()
-    { return decode( "Vraene vrednosti" ); }
+    { return decode( "Vraćene vrednosti" ); }
 
     /*! this text is generated when the \\sa command is used. */
     virtual QCString trSeeAlso()
-    { return decode( "Takoe pogledati" ); }
+    { return decode( "Takođe pogledati" ); }
 
     /*! this text is generated when the \\param command is used. */
     virtual QCString trParameters()
@@ -649,7 +652,7 @@ private:
         case ClassDef::Category:   result+="kategorije "; break; // 
         case ClassDef::Exception:  result+="izuzetka "; break;
       }
-      if (isTemplate) result += "ablona ";  
+      if (isTemplate) result += "šablona ";  
       
       result += clName;
       return decode( result );
@@ -672,25 +675,25 @@ private:
     }
     
     virtual QCString trPublicMembers()
-    { return decode("Javni lanovi"); }
+    { return decode("Javni članovi"); }
     virtual QCString trPublicSlots()
     { return "Javni slotovi"; }
     virtual QCString trSignals()
     { return "Signali"; }
     virtual QCString trStaticPublicMembers()
-    { return decode("Zajedniki javni lanovi"); }
+    { return decode("Zajednički javni članovi"); }
     virtual QCString trProtectedMembers()
-    { return decode("Zatieni lanovi");  }
+    { return decode("Zaštićeni članovi");  }
     virtual QCString trProtectedSlots()
-    { return decode("Zatieni slotovi"); }
+    { return decode("Zaštićeni slotovi"); }
     virtual QCString trStaticProtectedMembers()
-    { return decode("Zajedniki zatieni lanovi"); }
+    { return decode("Zajednički zaštićeni članovi"); }
     virtual QCString trPrivateMembers()
-    { return decode("Privani lanovi"); }
+    { return decode("Privatni članovi"); }
     virtual QCString trPrivateSlots()
     { return decode("Privatni slotovi"); }
     virtual QCString trStaticPrivateMembers()
-    { return decode("Zajedniki privatni lanovi"); }
+    { return decode("Zajednički privatni članovi"); }
     
     /*! this function is used to produce a comma-separated list of items.
      *  use generateMarker(i) to indicate where item i should be put.
@@ -722,7 +725,7 @@ private:
      */
     virtual QCString trInheritsList(int numEntries)
     {
-      return decode("Nasleeno od "+trWriteList(numEntries)+".");
+      return decode("Nasleđeno od "+trWriteList(numEntries)+".");
     }
 
     /*! used in class documentation to produce a list of super classes,
@@ -730,7 +733,7 @@ private:
      */
     virtual QCString trInheritedByList(int numEntries)
     {
-      return decode( "Nasleeno u "+trWriteList(numEntries)+"." );
+      return decode( "Nasleđeno u "+trWriteList(numEntries)+"." );
     }
 
     /*! used in member documentation blocks to produce a list of 
@@ -738,7 +741,7 @@ private:
      */
     virtual QCString trReimplementedFromList(int numEntries)
     {
-      return decode("Preuraeno od "+trWriteList(numEntries)+"." );
+      return decode("Preurađeno od "+trWriteList(numEntries)+"." );
     }
 
     /*! used in member documentation blocks to produce a list of
@@ -746,21 +749,21 @@ private:
      */
     virtual QCString trReimplementedInList(int numEntries)
     {
-      return decode("Preuraeno u "+trWriteList(numEntries)+"." );
+      return decode("Preurađeno u "+trWriteList(numEntries)+"." );
     }
 
     /*! This is put above each page as a link to all members of namespaces. */
     virtual QCString trNamespaceMembers()
-    { return decode("lanovi prostora imena"); }
+    { return decode("Članovi prostora imena"); }
 
     /*! This is an introduction to the page with all namespace members */
     virtual QCString trNamespaceMemberDescription(bool extractAll)
     { 
       QCString result="Spisak svih ";
       if (!extractAll) result+="dokumentovanih ";
-      result+="lanova prostora imena sa linkovima na ";
+      result+="članova prostora imena sa linkovima na ";
       if (extractAll) 
-        result+="dokumentaciju svakog lana prostora imena: ";
+        result+="dokumentaciju svakog člana prostora imena: ";
       else 
         result+="prostor imena kojima pripadaju: ";
       return decode( result );
@@ -826,7 +829,7 @@ private:
 
     /*! This is used as the heading text for the retval command. */
     virtual QCString trReturnValues()
-    { return decode( "Vraena vrednost" ); }
+    { return decode( "Vraćena vrednost" ); }
 
     /*! This is in the (quick) index as a link to the main page (index.html)
      */
@@ -872,7 +875,7 @@ private:
     /*! this text is put before a collaboration diagram */
     virtual QCString trCollaborationDiagram(const char *clName)
     {
-      return decode( (QCString)"Dijagram nasleenih lanova za "+clName+":" );
+      return decode( (QCString)"Dijagram nasleđenih članova za "+clName+":" );
     }
     /*! this text is put before an include dependency graph */
     virtual QCString trInclDepGraph(const char *fName)
@@ -912,7 +915,7 @@ private:
     /*! Text shown before a multi-line variable/enum initialization */
     virtual QCString trInitialValue()
     {
-      return decode( "Poetna vriednost:" );
+      return decode( "Početna vrednost:" );
     }
     /*! Text used the source code in the file index */
     virtual QCString trCode()
@@ -921,15 +924,15 @@ private:
     }
     virtual QCString trGraphicalHierarchy()
     {
-      return decode("Grafiki prikaz stabla klasa");
+      return decode("Grafički prikaz stabla klasa");
     }
     virtual QCString trGotoGraphicalHierarchy()
     {
-      return decode("Prikai stablo klasa u grafikom obliku");
+      return decode("Prikaži stablo klasa u grafičkom obliku");
     }
     virtual QCString trGotoTextualHierarchy()
     {
-      return decode( "Prikai stablo klasa u tekstualnom obliku" );
+      return decode( "Prikaži stablo klasa u tekstualnom obliku" );
     }
     virtual QCString trPageIndex()
     {
@@ -952,28 +955,28 @@ private:
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return decode( "lanovi" );        
+        return decode( "Članovi" );        
       }
       else
       {
-        return decode( "Javni lanovi" );
+        return decode( "Javni članovi" );
       }
     }
     virtual QCString trStaticPublicAttribs()
     {
-      return decode( "Zajedniki javni lanovi");
+      return decode( "Zajednički javni članovi");
     }
     virtual QCString trProtectedTypes()
     {
-      return decode( "Zatieni tipovi" );
+      return decode( "Zaštićeni tipovi" );
     }
     virtual QCString trProtectedAttribs()
     {
-      return decode( "Zatieni lanovi" );
+      return decode( "Zaštićeni članovi" );
     }
     virtual QCString trStaticProtectedAttribs()
     {
-      return decode( "Zajedniki zatieni lanovi" );
+      return decode( "Zajednički zaštićeni članovi" );
     }
     virtual QCString trPrivateTypes()
     {
@@ -981,11 +984,11 @@ private:
     }
     virtual QCString trPrivateAttribs()
     {
-      return decode( "Privatni lanovi" );
+      return decode( "Privatni članovi" );
     }
     virtual QCString trStaticPrivateAttribs()
     {
-      return decode( "Zajedniki privatni lanovi" );
+      return decode( "Zajednički privatni članovi" );
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -995,12 +998,12 @@ private:
     /*! Used as a marker that is put before a \\todo item */
     virtual QCString trTodo()
     {
-      return decode( "Neuraeno" );
+      return decode( "Neurađeno" );
     }
     /*! Used as the header of the todo list */
     virtual QCString trTodoList()
     {
-      return decode("Spisak neuraenog");
+      return decode("Spisak neurađenog");
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1009,20 +1012,20 @@ private:
 
     virtual QCString trReferencedBy()
     {
-      return decode( "Korieno od" ); 
+      return decode( "Korišćeno od" ); 
     }
     virtual QCString trRemarks()
     {
-      return decode( "Zapaanja" );
+      return decode( "Zapažanja" );
     }
     virtual QCString trAttention()
     {
-      return decode( "Panja" );
+      return decode( "Pažnja" );
     }
     virtual QCString trInclByDepGraph()
     {
       return decode("Ovaj graf pokazuje koja datoteka direktno "
-      "ili indirektno ukljuuju ovu datoteku: ");
+      "ili indirektno uključuju ovu datoteku: ");
     }
     virtual QCString trSince()
     {
@@ -1036,7 +1039,7 @@ private:
     /*! title of the graph legend page */
     virtual QCString trLegendTitle()
     {
-      return decode( "Objanjenje korienih simbola" );
+      return decode( "Objašnjenje korišćenih simbola" );
     }
     /*! page explaining how the dot graph's should be interpreted 
      *  The %A in the text below are to prevent link to classes called "A".
@@ -1044,7 +1047,7 @@ private:
     virtual QCString trLegendDocs()
     {
       return decode(        
-       "Ova stranica objanjava kako tumaiti grafikone koje je napravio "
+       "Ova stranica objašnjava kako tumačiti grafikone koje je napravio "
         "doxygen.<p>\n"
         "Na primer:\n"
         "\\code\n"
@@ -1068,7 +1071,7 @@ private:
         "class Inherited : public PublicBase,\n"
         "                  protected ProtectedBase,\n"
         "                  private PrivateBase,\n"
-        "                  public Undocumented\n"
+        "                  public Undocumented,\n"
         "                  public Templ<int>\n"
         "{\n"
         "  private:\n"
@@ -1076,29 +1079,29 @@ private:
         "};\n"
         "\\endcode\n"
         "Ako je \\c MAX_DOT_GRAPH_HEIGHT tag u konfiguracionoj datoteci "
-        "postavljen na  200 gornje veze e izgledati:"
+        "postavljen na  200 gornje veze će izgledati:"
         "<p><center><img src=\"graph_legend."+Config_getEnum("DOT_IMAGE_FORMAT")+"\"></center>\n"
         "<p>\n"
         
-        "Pravougaonici imaju sledea znaenja:\n"
+        "Pravougaonici imaju sledeća značenja:\n"
         "<ul>\n"
          "<li>Puni crni predstavlja strukturu ili klasu za koju je graf napravljen.\n"
          "<li>Sa crnom ivicom predstavlja dokumentovanu strukturu ili klasu.\n"
          "<li>Sa sivom ivicom predstavlja nedokumentovanu strukturu ili klasu.\n"
          "<li>Sa crvenom ivicom predstavlja dokumentovanu strukturu ili klasu\n"
-         "za koju nije prikazan graf nasleivanja/korienja. Graf je odseen "
-         "ako ne stane unutar odreenih granica." 
+         "za koju nije prikazan graf nasleđivanja/korišćenja. Graf je odsečen "
+         "ako ne stane unutar određenih granica." 
         "</ul>" 
-        "Strelice imaju sledea znaenja:\n"
+        "Strelice imaju sledeća značenja:\n"
         "<ul>\n"
-         "<li>Tamnoplava strelica oznaava javno nasleivanje.\n"
-         "<li>Tamnozelena strelica oznaava zatieno nasleivanje.\n"
-         "<li>Tamnocrvena strelica oznaava privatno nasleivanje.\n"
-         "<li>Ljubiasta isprekidana strelica ako je klasa koristi ili je njen lan. "
-          "Strelica je oznaena imenom lana.\n"
-         "<li>Zuta strelica oznaava vezu izmeu primerka ablona"
-          " i ablon klase od koje je napravljena. "
-          "Strelica je oznaena imenom prarametra ablona.\n"
+         "<li>Tamnoplava strelica označava javno nasleđivanje.\n"
+         "<li>Tamnozelena strelica označava zaštićeno nasleđivanje.\n"
+         "<li>Tamnocrvena strelica označava privatno nasleđivanje.\n"
+         "<li>Ljubičasta isprekidana strelica ako je klasa koristi ili je njen član. "
+          "Strelica je označena imenom člana.\n"
+         "<li>Zuta strelica označava vezu između primerka šablona"
+          " i šablon klase od koje je napravljena. "
+          "Strelica je označena imenom parametra šablona.\n"
         "</ul>\n"
          );
 
@@ -1106,7 +1109,7 @@ private:
     /*! text for the link to the legend page */
     virtual QCString trLegend()
     {
-      return decode( "Objanjenje korienih simbola" );
+      return decode( "Objašnjenje korišćenih simbola" );
     }
     
 //////////////////////////////////////////////////////////////////////////
@@ -1163,7 +1166,7 @@ private:
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Strikture";
+        return "Strukture";
       }
       else
       {
@@ -1208,12 +1211,12 @@ private:
     /*! Used as a marker that is put before a \\bug item */
     virtual QCString trBug()
     {
-      return decode( "Greka" );
+      return decode( "Greška" );
     }
     /*! Used as the header of the bug list */
     virtual QCString trBugList()
     {
-      return decode( "Spisak greaka" );
+      return decode( "Spisak grešaka" );
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1262,7 +1265,7 @@ private:
     /*! Used as header RTF general index */
     virtual QCString trRTFGeneralIndex()
     {
-      return decode( "Sadraj" );
+      return decode( "Sadržaj" );
     }
    
     /*! This is used for translation of the word that will possibly
@@ -1327,7 +1330,7 @@ private:
      */
     virtual QCString trMember(bool first_capital, bool singular)
     { 
-      QCString result((first_capital ? "lan" : "lan"));
+      QCString result((first_capital ? "Član" : "član"));
       result+= (singular ? "" : "ovi");
       return decode( result ); 
     }
@@ -1387,7 +1390,7 @@ private:
      */
     virtual QCString trImplementedFromList(int numEntries)
     {
-      return decode( "Definie "+trWriteList(numEntries)+"." );
+      return decode( "Definiše "+trWriteList(numEntries)+"." );
     }
 
     /*! used in member documentation blocks to produce a list of
@@ -1407,7 +1410,7 @@ private:
          */
     virtual QCString trRTFTableOfContents()
      {
-      return decode( "Sadraj" );
+      return decode( "Sadržaj" );
      }
      
 //////////////////////////////////////////////////////////////////////////
@@ -1419,7 +1422,7 @@ private:
      */
     virtual QCString trDeprecatedList()
     {
-      return decode( "Spisak zastarelih funkcija lanica" );
+      return decode( "Spisak zastarelih funkcija članica" );
     } 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.18
@@ -1430,12 +1433,12 @@ private:
      */
     virtual QCString trEvents()
     {
-      return decode( "Dogaaji" );
+      return decode( "Događaji" );
     }
     /*! Header used for the documentation section of a class' events. */
     virtual QCString trEventDocumentation()
     {
-      return decode( "Dokumentacija dogaaja" );
+      return decode( "Dokumentacija događaja" );
     }
 //////////////////////////////////////////////////////////////////////////
 // new since 1.3
@@ -1459,7 +1462,7 @@ private:
      */
     virtual QCString trStaticPackageMembers()
     { 
-      return decode( "Statike funkcije u paketu" );  // Zajednicke funkcije u paketu
+      return decode( "Statičke funkcije u paketu" );  // Zajednicke funkcije u paketu
     }
     /*! Used as a heading for a list of Java class variables with package 
      * scope.
@@ -1473,7 +1476,7 @@ private:
      */
     virtual QCString trStaticPackageAttribs()
     { 
-      return decode( "Statiki atributi u paketu" );  // Zajednicki clanovi u paketu
+      return decode( "Statički atributi u paketu" );  // Zajednicki clanovi u paketu
     }
     
 //////////////////////////////////////////////////////////////////////////
@@ -1503,14 +1506,14 @@ private:
      */
     virtual QCString trSearchForIndex()
     {
-      return decode("Trai");
+      return decode("Traži");
     }
     /*! This string is used as the title for the page listing the search
      *  results.
      */
     virtual QCString trSearchResultsTitle()
     {
-      return decode( "Rezultati pretraivanja" );
+      return decode( "Rezultati pretraživanja" );
     }
     /*! This string is put just before listing the search results. The
      *  text can be different depending on the number of documents found.
@@ -1524,20 +1527,20 @@ private:
     {
       if (numDocuments==0)
       {
-        return decode("Nema dokumenta koji odgovaraju vaem upitu");
+        return decode("Nema dokumenta koji odgovaraju vašem upitu");
       }
       else if (numDocuments==1)
       { // 10x 2 Ivana Miletic 4 gramatical consutation.
-        return decode("Naen je <b>1</b> dokument koji odgovara vaem upitu.");
+        return decode("Nađen je <b>1</b> dokument koji odgovara vašem upitu.");
       }
     else if (numDocuments<5) 
     {  // 10x 2 Ivana Miletic 4 gramatical consutation.
-      return decode("Naena su <b>$num</b> dokumenta koji odgovaraju vaem upitu."
+      return decode("Nađena su <b>$num</b> dokumenta koji odgovaraju vašem upitu."
                     "Najbolji su prikazani prvi.");
     }
     else 
     {  // 10x 2 Ivana Miletic 4 gramatical consutation.
-      return decode("Naeno je <b>$num</b> dokumenata koji odgovaraju vaem upitu."
+      return decode("Nađeno je <b>$num</b> dokumenata koji odgovaraju vašem upitu."
         "Najbolji su prikazani prvi.");
     }
     }
@@ -1546,7 +1549,7 @@ private:
      */
     virtual QCString trSearchMatches()
     {
-      return decode( "Pronaeno:" );
+      return decode( "Pronađeno:" );
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1586,7 +1589,7 @@ private:
      *  and the fact that it is sorted alphabetically per level
      */
     virtual QCString trDirDescription()
-    { return decode( "Stablo direktorijuma slozeno priblizno abecednim redom" );
+    { return decode( "Stablo direktorijuma slozeno približno po abecednom redu" );
     }
 
     /*! This returns the title of a directory page. The name of the
