@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2007 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -108,7 +108,8 @@ void XmlDocVisitor::visit(DocURL *u)
   if (m_hide) return;
   m_t << "<ulink url=\""; 
   if (u->isEmail()) m_t << "mailto:";
-  m_t << u->url() << "\">";
+  filter(u->url());
+  m_t << "\">";
   filter(u->url());
   m_t << "</ulink>";
 }
@@ -208,6 +209,11 @@ void XmlDocVisitor::visit(DocVerbatim *s)
       filter(s->text());
       m_t << "</dot>";
       break;
+    case DocVerbatim::Msc: 
+      m_t << "<msc>";
+      filter(s->text());
+      m_t << "</msc>";
+      break;
   }
 }
 
@@ -721,7 +727,7 @@ void XmlDocVisitor::visitPost(DocRef *ref)
 {
   if (m_hide) return;
   if (!ref->file().isEmpty()) endLink();
-  m_t << " ";
+  //m_t << " ";
 }
 
 void XmlDocVisitor::visitPre(DocSecRefItem *ref)
@@ -870,7 +876,7 @@ void XmlDocVisitor::visitPost(DocInternalRef *)
 void XmlDocVisitor::visitPre(DocCopy *c)
 {
   if (m_hide) return;
-  m_t << "<copydoc link=\"" << c->link() << "\">";
+  m_t << "<copydoc link=\"" << convertToXML(c->link()) << "\">";
 }
 
 void XmlDocVisitor::visitPost(DocCopy *)
